// TestCaseAocsData_4.cpp
//
// Version	1.0
// Date		10.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include <math.h>

#include "TestCaseAocsData_4.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/AocsData.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsData/DataItemRead.h"
#include "../../OperatingSystemObjects/DummyAocsClock.h"

// -------------------------------------------------------------------
TestCaseAocsData_4::TestCaseAocsData_4():
	TestCaseGenericSetUp(ID_AOCSDATA*10+4,"TestCaseAocsData_4") {
		return;
}

// -------------------------------------------------------------------

void TestCaseAocsData_4::runTestCase() {

	TwoEulerAngles e1, e2;

	// Set closeness thresholds
	Real epsilon=(Real)0.2;
	e1.setClosenessThreshold(epsilon);
	e2.setClosenessThreshold(epsilon);
	
	// Reset the two AOCS data
	e1.reset();
	e2.reset();

	// Get the write access to the AOCS data item
	DataItemWrite w10 = e1.getDataItemWrite(0);
	DataItemWrite w11 = e1.getDataItemWrite(1);
	DataItemWrite w20 = e2.getDataItemWrite(0);
	DataItemWrite w21 = e2.getDataItemWrite(1);

	// Set the TwoEulerAngles equal to (1,2) and (4,6) 
	w10.set(1);
	w11.set(2);
	w20.set(4);
	w21.set(6);

	// Check the distance between the AOCS data (allow for possible rounding errors!)
	if ( fabs(e1.distance(&e2) - 5)>0.0000001 )
	{	setTestResult(TEST_FAILURE, "Wrong TwoEulerAngles distance");
		return;
	}

	if ( fabs(e2.distance(&e1) - 5)>0.0000001 )
	{	setTestResult(TEST_FAILURE, "Wrong TwoEulerAngles distance");
		return;
	}

	// Check that the AOCS data are not "close" (using default closeness threshold)
	if (e1.close(&e2))
	{	setTestResult(TEST_FAILURE, "TwoEulerAngles should be close");
		return;
	}

	// Check that the AOCS data are "close" (using very large closeness threshold)
	if (!e2.close( &e1, (Real)(1000.0) ))
	{	setTestResult(TEST_FAILURE, "TwoEulerAngles should be close");
		return;
	}

	// Check the correct size of the AOCS data
	if ( fabs(e1.size()-sqrt(5.0)) > 0.0000001 )
	{	setTestResult(TEST_FAILURE, "Incorrect TwoEulerAngles size");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
