// TestCaseAocsData_7.cpp
//
// Version	1.0
// Date		13.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include <math.h>

#include "TestCaseAocsData_7.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/AttitudeDataPool.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsData/DataItemRead.h"
#include "../../OperatingSystemObjects/DummyAocsClock.h"

// -------------------------------------------------------------------
TestCaseAocsData_7::TestCaseAocsData_7():
	TestCaseGenericSetUp(ID_AOCSDATA*10+7,"TestCaseAocsData_7") {
		return;
}

// -------------------------------------------------------------------

void TestCaseAocsData_7::runTestCase() {

	AttitudeDataPool attitudeDataPool;

	// Reset the data pool
	attitudeDataPool.reset();

	// Set the data item values
	attitudeDataPool.getFssData()->getDataItemWrite(0).set(1);
	attitudeDataPool.getFssData()->getDataItemWrite(1).set(2);
	attitudeDataPool.getSas1Data()->getDataItemWrite(0).set(3);
	attitudeDataPool.getSas1Data()->getDataItemWrite(1).set(4);
	attitudeDataPool.getSas2Data()->getDataItemWrite(0).set(5);
	attitudeDataPool.getSas2Data()->getDataItemWrite(1).set(6);
	attitudeDataPool.getSas3Data()->getDataItemWrite(0).set(7);
	attitudeDataPool.getSas3Data()->getDataItemWrite(1).set(8);

	// Check the size of the data in the pool
	if (attitudeDataPool.getFssData()->size() != sqrt(5))
	{	setTestResult(TEST_FAILURE, "Aocs data size is wrong");
		return;
	}

	if (attitudeDataPool.getSas1Data()->size() != sqrt(9+16))
	{	setTestResult(TEST_FAILURE, "Aocs data size is wrong");
		return;
	}

	if (attitudeDataPool.getSas2Data()->size() != sqrt(25+36))
	{	setTestResult(TEST_FAILURE, "Aocs data size is wrong");
		return;
	}

	if (attitudeDataPool.getSas3Data()->size() != sqrt(49+64))
	{	setTestResult(TEST_FAILURE, "Aocs data size is wrong");
		return;
	}

	// Reset the data pool
	attitudeDataPool.reset();

	// Check that all data in the pool have size 0
	if (attitudeDataPool.getFssData()->size() != 0)
	{	setTestResult(TEST_FAILURE, "Aocs data size is wrong");
		return;
	}

	if (attitudeDataPool.getSas1Data()->size() != 0)
	{	setTestResult(TEST_FAILURE, "Aocs data size is wrong");
		return;
	}

	if (attitudeDataPool.getSas2Data()->size() != 0)
	{	setTestResult(TEST_FAILURE, "Aocs data size is wrong");
		return;
	}

	if (attitudeDataPool.getSas3Data()->size() != 0)
	{	setTestResult(TEST_FAILURE, "Aocs data size is wrong");
		return;
	}

	// Check that no failure or configuration events have been created
	if ( failureEvtRep.getCounter() != failureEvtN )
	{	setTestResult(TEST_FAILURE, "Failure event was generated");
		return;
	}

	if ( configEvtRep.getCounter() != configEvtN )
	{	setTestResult(TEST_FAILURE, "Configuration event was generated");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
