// AttitudeDataPool.h
//
// Version	1.0
// Date		15.03.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Class for the attitude data pool for prototype AOCS
//
// -------------------------------------------------------------------

#ifndef AttitudeDataPoolH
#define AttitudeDataPoolH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../AocsData/AocsDataPool.h"
#include "../AocsData/TwoEulerAngles.h"
#include "../AocsData/ThreeEulerAngles.h"
#include "../AocsData/Scalar.h"
#include "../AocsData/NVector.h"

// -------------------------------------------------------------------

class AttitudeDataPool : public AocsDataPool {

	// Sensor data
	TwoEulerAngles* fssData;
	TwoEulerAngles* sas1Data;
	TwoEulerAngles* sas2Data;
	TwoEulerAngles* sas3Data;
	Scalar* gyrRate;

	// Actuator data 
	NVector* scTorque;
	NVector* rwAngularMomentum;		// expressed in s/c ref. frame
	NVector* rwTorque;
	NVector* rwSpeed;
	NVector* thuOnTime;
	NVector* thuDelayTime;

	// Attitude data
	ThreeEulerAngles* attEstimate;
	ThreeEulerAngles* attSetPoint;
	ThreeEulerAngles* attError;

public :

	AttitudeDataPool();

	AocsData* getFssData()
		{ return fssData; }

	AocsData* getSas1Data()
		{ return sas1Data; }

	AocsData* getSas2Data()
		{ return sas2Data; }

	AocsData* getSas3Data()
		{ return sas3Data; }

	AocsData* getGyrRate()
		{ return gyrRate; }

	AocsData* getScTorque()
		{ return scTorque; }

	AocsData* getRwTorque()
		{ return rwTorque; }

	AocsData* getRwSpeed()
		{ return rwSpeed; }

	AocsData* getRwAngularMomentum()
		{ return rwAngularMomentum; }

	AocsData* getThuOnTime()
		{ return thuOnTime; }

	AocsData* getThuDelayTime()
		{ return thuDelayTime; }

	AocsData* getAttEstimate()
		{ return attEstimate; }

	AocsData* getAttSetPoint()
		{ return attSetPoint; }

	AocsData* getAttError()
		{ return attError; }

};

// -------------------------------------------------------------------
#endif
