// ChangeEvent.cpp
//
// Version	1.0
// Date		07.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsEvent/ChangeEvent.h"
#include "../AocsEvent/AocsEvent.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/ClassId.h"

// -------------------------------------------------------------------
// Define static variables
int ChangeEvent::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
// Default constructor.

ChangeEvent::ChangeEvent()
{
	setClassId(ID_CHANGEEVENT);			// assign class identifier

	tmImageLength[shortTm]  = 0;		// Not implemented in prototype
	tmImageLength[normalTm] = 0; 
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;
	
	localResetConfiguration();				// reset object's configuration

}

// -------------------------------------------------------------------

void ChangeEvent::initialize(AocsObject* creator, 
							   EventType evtType,
								 PropertyId p, 
								   ChangeObject* c,
									 Real v) 
{
	AocsEvent::initialize(creator, evtType);
	propertyId = p;
	changeObject = c;
	lastValue = v;
}

// -------------------------------------------------------------------

void ChangeEvent::localResetConfiguration()
{
	propertyId = NULL;
	changeObject = NULL;
	lastValue = 0;
}

// -------------------------------------------------------------------

void ChangeEvent::resetConfiguration()
{
	AocsEvent::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool ChangeEvent::isConfigured()
{
	if (!AocsEvent::isConfigured())
		return false;

	if ( changeObject == NULL ) 
		return false;

	return true;
}

// -------------------------------------------------------------------
// From Telemeterable interface

int ChangeEvent::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsEvent::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void ChangeEvent::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsEvent::writeToTelemetry(tmStream);		// write super object's state

	// Not implemented in prototype 
}