// ConfigurationEvent.cpp
//
// Version	1.0
// Date		07.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsEvent/ConfigurationEvent.h"
#include "../AocsEvent/AocsEvent.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../TelemetryManagement/TelemetryStream.h"

// -------------------------------------------------------------------
// Define static variables
int ConfigurationEvent::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
// Default constructor.

ConfigurationEvent::ConfigurationEvent()
{
	setClassId(ID_CONFIGURATIONEVENT);			// assign class identifier

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 8*sizeof(InstanceId);
	tmImageLength[debugTm]  = 8*sizeof(InstanceId);
	
	localResetConfiguration();				// reset object's configuration

}

// -------------------------------------------------------------------

void ConfigurationEvent::initialize(AocsObject* creator, 
				  					  EventType evtType,
										AocsObject* loc) 
{
	AocsEvent::initialize(creator, evtType);
	location = loc;
}

// -------------------------------------------------------------------

void ConfigurationEvent::localResetConfiguration()
{
	location = NULL;
}

// -------------------------------------------------------------------

void ConfigurationEvent::resetConfiguration()
{
	AocsEvent::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool ConfigurationEvent::isConfigured()
{
	if (!AocsEvent::isConfigured())
		return false;

	if (location == NULL)
		return false;

	return true;
}

// -------------------------------------------------------------------
// From Telemeterable interface

int ConfigurationEvent::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsEvent::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void ConfigurationEvent::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsEvent::writeToTelemetry(tmStream);		// write super object's state

	if ( (getTelemetryFormat()==longTm) || (getTelemetryFormat()==debugTm) )
		tmStream->write(location->getInstanceId());
}