// COnfigurationEventRepository.cpp
//
// Version	1.0
// Date		09.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../AocsEvent/ConfigurationEvent.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/ClassId.h"
#include "../AocsEvent/EventRepository.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------
// Default constructor.

ConfigurationEventRepository::ConfigurationEventRepository()
{
	setClassId(ID_CONFIGURATIONEVENTREPOSITORY);		// assign class identifier
	buffer = NULL;
}

// -------------------------------------------------------------------
// Allocate memory for the event buffer and for the list of event pointers.
// Load the pointer list with the addresses of the events in the buffer. 
// This method can only be called once.

void ConfigurationEventRepository::setRepositorySize(int n)
{
	if (buffer != NULL)			// method was already called!
	{	reportConfigurationError(this,MEMORY_ALREADY_ALLOCATED,this);
		return;
	}

	buffer = new ConfigurationEvent[n];
	EventRepository::setRepositorySize(n);	
	
	for (int i=0; i<n; i++)
		setList(i,&buffer[i]);
}

// -------------------------------------------------------------------
// Insert an event in the repository buffer

void ConfigurationEventRepository::create (AocsObject* creator, 
											EventType evtType,
											 AocsObject* location)
{
	ConfigurationEvent* evt = (ConfigurationEvent*) insert();	// insert new event
	evt->initialize(creator,evtType,location);					// initialize new event
}

// -------------------------------------------------------------------
// Return pointer to most recently stored event.

ConfigurationEvent* ConfigurationEventRepository::latestConfigurationEvent()
{
	return (ConfigurationEvent*)EventRepository::latest();
}

// -------------------------------------------------------------------
// Method to iterate through the events in the list

ConfigurationEvent* ConfigurationEventRepository::previousConfigurationEvent()
{
	return (ConfigurationEvent*)EventRepository::previous();
}


