// EventRepository.h
//
// Version	1.0
// Date		14.01.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Base class for event repositories
//
// This clas is not meant to be instantiated. Its function is to provide
// implementations for services offered by its subclasses.
// -------------------------------------------------------------------

#ifndef EventRepositoryH
#define EventRepositoryH

// -------------------------------------------------------------------

#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"
#include "../FailureDetection/ConsistencyCheckable.h"

// -------------------------------------------------------------------

class EventRepository : public AocsObject, public ConsistencyCheckable {

	AocsEvent** list;		// array of pointers to events in the repository
	int latestEvent;		// pointer to most recent event in 'list'
	int listSize;			// number of elements in 'list'
	int iteratorPointer;	
	int iterationCounter;	 
	unsigned short counter;	// number of events inserted in repository

	RecoveryAction*	recoveryAction;		
	unsigned short  counterOld;			

	static int tmImageLength[NTMFORMATS];

	void localResetConfiguration();		
	void localReset();	

protected:

	EventRepository();				// class cannot be instantiated	

	void setRepositorySize(int n);	// allocates memory for 'list'. This method can
									// only be called once!

	AocsEvent* insert();			// insert new event and returns pointer to it

	void setList(int i,AocsEvent* evt);	// load list of event pointers

public:

	AocsEvent* latest();	// methods to iterate through events in the repository
	AocsEvent* previous();	// events are returned starting from the most recent
	bool isIterationEnded();

	int getCounter()		
		{ return counter; }

	int getListSize()
		{ return listSize; }

	// Virtual methods from interface Resettable
	virtual void reset();				

	// Virtual methods from interface Configurable
	virtual void resetConfiguration();		
	virtual bool isConfigured();		
	
	// Virtual methods from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual void setTelemetryFormat(TelemetryFormat newFormat);
	virtual int getTelemetryImageLength();

	// Virtual methods from interface ConsistencyCheckable
	virtual bool doConsistencyCheck();
	virtual RecoveryAction* getRecoveryAction();
	virtual void setRecoveryAction(RecoveryAction* r);

};

// -------------------------------------------------------------------
#endif