// FailureEvent.cpp
//
// Version	1.0
// Date		07.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsEvent/FailureEvent.h"
#include "../AocsEvent/AocsEvent.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../TelemetryManagement/TelemetryStream.h"
#include "../FailureRecovery/RecoveryAction.h"

// -------------------------------------------------------------------
// Define static variables
int FailureEvent::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

FailureEvent::FailureEvent()
{
	setClassId(ID_FAILUREEVENT);		

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 8*2*sizeof(InstanceId);
	tmImageLength[debugTm]  = 8*2*sizeof(InstanceId);
	
	localResetConfiguration();				// reset object's configuration

}

// -------------------------------------------------------------------

void FailureEvent::initialize(AocsObject* creator, 
				  			    EventType evtType,
								  AocsObject* loc,
								    RecoveryAction* recAction) 
{
	AocsEvent::initialize(creator, evtType);
	location = loc;
	recoveryAction = recAction;
}

// -------------------------------------------------------------------

void FailureEvent::localResetConfiguration()
{
	location = NULL;
	recoveryAction = NULL;
}

// -------------------------------------------------------------------

void FailureEvent::resetConfiguration()
{
	AocsEvent::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool FailureEvent::isConfigured()
{
	if (!AocsEvent::isConfigured())
		return false;

	if (location == NULL) 
		return false;

	if (recoveryAction == NULL) 
		return false;

	return true;
}

// -------------------------------------------------------------------
// From Telemeterable interface

int FailureEvent::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsEvent::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void FailureEvent::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsEvent::writeToTelemetry(tmStream);		// write super object's state

	if ( (getTelemetryFormat()==longTm) || (getTelemetryFormat()==debugTm) )
	{	tmStream->write(recoveryAction->getInstanceId());
		tmStream->write(location->getInstanceId());
	}
}