// ModeEventRepository.cpp
//
// Version	1.0
// Date		08.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsEvent/ModeEventRepository.h"
#include "../AocsEvent/ModeEvent.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/ClassId.h"
#include "../AocsEvent/EventRepository.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------
// Default constructor.

ModeEventRepository::ModeEventRepository()
{
	setClassId(ID_MODEEVENTREPOSITORY);			// assign class identifier
	buffer = NULL;
}

// -------------------------------------------------------------------
// Allocate memory for the event buffer and for the list of event pointers.
// Load the pointer list with the addresses of the events in the buffer. 
// This method can only be called once.

void ModeEventRepository::setRepositorySize(int n)
{
	if (buffer != NULL)			// method was already called!
	{	reportConfigurationError(this,MEMORY_ALREADY_ALLOCATED,this);
		return;
	}

	buffer = new ModeEvent[n];
	EventRepository::setRepositorySize(n);	
	
	for (int i=0; i<n; i++)
		setList(i,&buffer[i]);
}

// -------------------------------------------------------------------
// Insert an event in the repository buffer

void ModeEventRepository::create (AocsObject* creator, 
									   EventType evtType,
										 int oldMode, 
										   int newMode,
					  						 ModeManager* modeManager)
{
	ModeEvent* evt = (ModeEvent*) insert();		// insert new event
	evt->initialize(creator,evtType,oldMode,newMode,modeManager);	// initialize new event
}

// -------------------------------------------------------------------
// Return pointer to most recently stored event.

ModeEvent* ModeEventRepository::latestModeEvent()
{
	return (ModeEvent*)EventRepository::latest();
}

// -------------------------------------------------------------------
// Method to iterate through the events in the list

ModeEvent* ModeEventRepository::previousModeEvent()
{
	return (ModeEvent*)EventRepository::previous();
}



