// ReconfigurationEvent.cpp
//
// Version	1.0
// Date		26.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsEvent/ReconfigurationEvent.h"
#include "../AocsEvent/EventTypeId.h"
#include "../AocsEvent/AocsEvent.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../TelemetryManagement/TelemetryStream.h"
#include "../FailureRecovery/RecoveryAction.h"

// -------------------------------------------------------------------
// Define static variables
int ReconfigurationEvent::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

ReconfigurationEvent::ReconfigurationEvent()
{
	setClassId(ID_RECONFIGURATIONEVENT);			

	tmImageLength[shortTm]  = 8*sizeof(unsigned char);	
	tmImageLength[normalTm] = tmImageLength[shortTm]+8*sizeof(InstanceId);
	tmImageLength[longTm]   = tmImageLength[shortTm];
	tmImageLength[debugTm]  = tmImageLength[shortTm];
	
	localResetConfiguration();				

}

// -------------------------------------------------------------------

void ReconfigurationEvent::initialize(AocsObject* creator, 
										EventType evtType,
										  ConfigurationId oldConfig,
										    ConfigurationId newConfig,
											  InstanceId configManagerId) 
{
	AocsEvent::initialize(creator, evtType);
	oldConfiguration = oldConfig;					
	newConfiguration = newConfig;					
	configurationManagerId = configManagerId;
}

// -------------------------------------------------------------------

void ReconfigurationEvent::localResetConfiguration()
{
	oldConfiguration = 0;					
	newConfiguration = 0;					
	configurationManagerId = NULL;
}

// -------------------------------------------------------------------

void ReconfigurationEvent::resetConfiguration()
{
	AocsEvent::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool ReconfigurationEvent::isConfigured()
{
	if (!AocsEvent::isConfigured())
		return false;

	if ( (oldConfiguration == newConfiguration) && 
			(getEventType() != RECONFIGURATION_FAILURE) )
		return false;

	if (configurationManagerId == NULL) 
		return false;

	return true;
}

// -------------------------------------------------------------------
// From Telemeterable interface

int ReconfigurationEvent::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsEvent::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// Only normal and long TM are implemented

void ReconfigurationEvent::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsEvent::writeToTelemetry(tmStream);	

	// Combine old and new configuration in a single byte (assume that 
	// the configuration indicator is in the range 0-15)
	unsigned char config;
	config = (unsigned char)oldConfiguration;
	config = config<<4;
	config = config + (unsigned char)newConfiguration;

	// Write mode word and instance ID of the reconfiguration manager
	tmStream->write((unsigned char)config);
	tmStream->write((InstanceId)configurationManagerId);
}