// RecoveryEvent.cpp
//
// Version	1.0
// Date		07.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsEvent/RecoveryEvent.h"
#include "../AocsEvent/AocsEvent.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../TelemetryManagement/TelemetryStream.h"
#include "../FailureRecovery/RecoveryAction.h"
#include "../FailureRecovery/RecoveryStrategy.h"

// -------------------------------------------------------------------
// Define static variables
int RecoveryEvent::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

RecoveryEvent::RecoveryEvent()
{
	setClassId(ID_FAILUREEVENT);			// assign class identifier

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 8*2*sizeof(InstanceId);
	tmImageLength[debugTm]  = 8*2*sizeof(InstanceId);
	
	localResetConfiguration();				// reset object's configuration
}

// -------------------------------------------------------------------

void RecoveryEvent::initialize(AocsObject* creator, 
				  			    EventType evtType,
								  RecoveryAction* recAction,
								    RecoveryStrategy* recStrategy) 
{
	AocsEvent::initialize(creator, evtType);
	recoveryAction = recAction;
	recoveryStrategy = recStrategy;
}

// -------------------------------------------------------------------

void RecoveryEvent::localResetConfiguration()
{
	recoveryAction = NULL;
	recoveryStrategy = NULL;
}

// -------------------------------------------------------------------

void RecoveryEvent::resetConfiguration()
{
	AocsEvent::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool RecoveryEvent::isConfigured()
{
	if (!AocsEvent::isConfigured())
		return false;

	if (recoveryStrategy == NULL) 
		return false;

	return true;
}

// -------------------------------------------------------------------
// From Telemeterable interface

int RecoveryEvent::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsEvent::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void RecoveryEvent::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsEvent::writeToTelemetry(tmStream);		// write super object's state

	if ( (getTelemetryFormat()==longTm) || (getTelemetryFormat()==debugTm) )
	{	tmStream->write(recoveryAction->getInstanceId());
		tmStream->write(recoveryStrategy->getInstanceId());
	}
}