// TelecommandEvent.cpp
//
// Version	1.0
// Date		05.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsEvent/TelecommandEvent.h"
#include "../AocsEvent/AocsEvent.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../TelemetryManagement/TelemetryStream.h"

// -------------------------------------------------------------------
// Define static variables
int TelecommandEvent::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

TelecommandEvent::TelecommandEvent()
{
	setClassId(ID_TELECOMMANDEVENT);			

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = 8*sizeof(InstanceId);
	tmImageLength[longTm]   = tmImageLength[normalTm];
	tmImageLength[debugTm]  = tmImageLength[longTm];
	
	localResetConfiguration();				
}

// -------------------------------------------------------------------

void TelecommandEvent::initialize(AocsObject* creator, 
				  			    EventType evtType,
								  TelecommandId tcId) 
{
	AocsEvent::initialize(creator, evtType);
	telecommandId = tcId;
}

// -------------------------------------------------------------------

void TelecommandEvent::localResetConfiguration()
{
	telecommandId = 0;
}

// -------------------------------------------------------------------

void TelecommandEvent::resetConfiguration()
{
	AocsEvent::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool TelecommandEvent::isConfigured()
{
	if (!AocsEvent::isConfigured())
		return false;

	if (telecommandId == 0) 
		return false;

	return true;
}

// -------------------------------------------------------------------

int TelecommandEvent::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsEvent::getTelemetryImageLength());
}

// -------------------------------------------------------------------

void TelecommandEvent::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsEvent::writeToTelemetry(tmStream);		

	if (getTelemetryFormat()==shortTm)
		return;

	tmStream->write(telecommandId);
}