// TelecommandEventRepository.cpp
//
// Version	1.0
// Date		05.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsEvent/TelecommandEventRepository.h"
#include "../AocsEvent/TelecommandEvent.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/ClassId.h"
#include "../AocsEvent/EventRepository.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------

TelecommandEventRepository::TelecommandEventRepository()
{
	setClassId(ID_TELECOMMANDEVENTREPOSITORY);			
	buffer = NULL;
}

// -------------------------------------------------------------------
// Allocate memory for the event buffer and for the list of event pointers.
// Load the pointer list with the addresses of the events in the buffer. 
// This method can only be called once.

void TelecommandEventRepository::setRepositorySize(int n)
{
	if (buffer != NULL)			// method was already called!
	{	reportConfigurationError(this,MEMORY_ALREADY_ALLOCATED,this);
		return;
	}

	buffer = new TelecommandEvent[n];
	EventRepository::setRepositorySize(n);	
	
	for (int i=0; i<n; i++)
		setList(i,&buffer[i]);
}

// -------------------------------------------------------------------
// Insert an event in the repository buffer

void TelecommandEventRepository::create (AocsObject* creator, 
										  EventType evtType,
										   TelecommandId telecommandId)
{
	TelecommandEvent* evt = (TelecommandEvent*) insert();	// insert new event
	evt->initialize(creator,evtType,telecommandId);		// initialize new event
}

// -------------------------------------------------------------------
// Return pointer to most recently stored event.

TelecommandEvent* TelecommandEventRepository::latestTelecommandEvent()
{
	return (TelecommandEvent*)EventRepository::latest();
}

// -------------------------------------------------------------------
// Method to iterate through the events in the list

TelecommandEvent* TelecommandEventRepository::previousTelecommandEvent()
{
	return (TelecommandEvent*)EventRepository::previous();
}



