// AocsConfiguration.h
//
// Version	1.0
// Date		14.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Define the AOCS configuration constants
// -------------------------------------------------------------------

#ifndef AocsConfigurationH
#define AocsConfigurationH

// -------------------------------------------------------------------

// Define the mode indicators
#define N_OF_MODES										2
#define CSP												0
#define FSP												1

// Define the event repository sizes
#define SYSTEM_EVENT_REPOSITORY_SIZE					5
#define CONFIGURATION_EVENT_REPOSITORY_SIZE				10
#define FAILURE_EVENT_REPOSITORY_SIZE					10
#define MODE_EVENT_REPOSITORY_SIZE						5
#define CHANGE_EVENT_REPOSITORY_SIZE					5
#define RECONFIGURATION_EVENT_REPOSITORY_SIZE			4
#define RECOVERY_EVENT_REPOSITORY_SIZE					4
#define TELECOMMAND_EVENT_REPOSITORY_SIZE				4
#define MANOEUVRE_EVENT_REPOSITORY_SIZE					4

// Maximum number of AOCS objects (for the SystemManager)
#define MAX_N_AOCS_OBJECTS								350

// Maximum number of reconfiguration managers (for the SystemManager)
#define MAX_N_RECONFIGURER								20

// Maximum number of mode managers (for the mission mode manager)
#define MAX_N_MODE_MANAGERS								10

// Maximum number of telecommands that can be loaded in TC manager
#define MAX_N_TC										4

// Failure Detection manager configuration constants
#define N_FD_MODES			N_OF_MODES		// number of fd modes
#define MAX_FD_MONITORS		0				// number of fd mode manager monitors						
#define CC_LIST_SIZE		3				// consistency checkable list size
#define MC_LIST_SIZE		10				// monitoring check list size

// Failure Recovery manager configuration constants
#define N_FR_MODES			N_OF_MODES		// number of fr modes
#define MAX_FR_MONITORS		0				// number of fr mode manager monitors						

// Controller manager configuration constants
// NB: The controller constants do not take effect when the controller
//	   is instantiated from an Xmath-generated superblock!
#define N_CONT_MODES		N_OF_MODES		// number of controller modes
#define MAX_CONT_MONITORS	0				// number of controller mode manager monitors						
#define N_OF_CONTROLLERS	3				// number of controllers managed
											// by the controller manager
#define X_AXIS_CONT			0
#define Y_AXIS_CONT			1
#define Z_AXIS_CONT			2
#define CSP_XY_KP			1000			// PID parameter for X-/Y-axis controllers
#define CSP_XY_KD			0				// PID parameter for X-/Y-axis controllers
#define CSP_Z_KP			1000			// PID parameter for Z-axis controllers
#define CSP_Z_KD			0				// PID parameter for Z-axis controllers
#define FSP_XY_KP			1000			// PID parameter for X-/Y-axis controllers
#define FSP_XY_KD			0				// PID parameter for X-/Y-axis controllers
#define FSP_Z_KP			1000			// PID parameter for Z-axis controllers
#define FSP_Z_KD			0				// PID parameter for Z-axis controllers
#define DER_MAX				10				// Max value of D-block derivative 

// Telemetry manager configuration constants
#define N_TM_MODES			2				// number of tm modes
#define MAX_TM_MONITORS		0				// number of tm monitors						
#define TM_LIST_SIZE		30				// tm list size
#define TM_BUFFER_SIZE		400				// TM DMA buffer size in bytes

// Number of concrete manoeuvre objects created at initialization
#define N_ATT_SLEW_MAN		3				// attitude slews

// Maximum number of pending manoeuvres in manoeuvre manager
#define MAX_N_PEND_MAN		3				

// Maximum number of monitors that can be attached to a manoeuvre
#define MAX_N_MAN_MON		0

// Number of RTEMS ticks for AOCS cycle 
#define TICKS_PER_CYCLE		10000

// MACS unit addresses 
#define MACS_FSS_A			1
#define MACS_FSS_B			2
#define MACS_GYR_A			3
#define MACS_GYR_B			4
#define MACS_SAP_A			5
#define MACS_SAP_B			6
#define MACS_RW				7

// MACS unit subaddresses
#define FSS_SA_X			10
#define FSS_SA_Y			11
#define GYR_SA_RATE			10
#define SAS_SA_X_1			1
#define THU_SA_DT_1			12
#define RW_SA_SPEED			10
#define RW_SA_TOR			10

// Conversion coefficients from raw to engineering level for unit data
#define FSS_Q				0.00001			
#define GYR_RAT_Q			0.00001			
#define SAS_Q				0.00001	
#define RW_SPD_Q			0.5
#define RW_TOR_Q			100.0

// Correction coefficients for the FSS raw data
#define	BIAS_FSS_X_A		-0.00001
#define	SCALING_FSS_X_A		1.0
#define	BIAS_FSS_X_B		-0.00002
#define	SCALING_FSS_X_B		1.0

#define	BIAS_FSS_Y_A		-0.00001
#define	SCALING_FSS_Y_A		1.0
#define	BIAS_FSS_Y_B		-0.00002
#define	SCALING_FSS_Y_B		1.0

// Correction coefficients for the GYR raw data
#define	BIAS_GYR_A			0.00001
#define	SCALING_GYR_A		1.0
#define	BIAS_GYR_B			0.00002
#define	SCALING_GYR_B		1.0

// Correction coefficients for the SAS raw data
#define	BIAS_SAS_1_A_X		-0.00001
#define	SCALING_SAS_1_A_X	1.0
#define	BIAS_SAS_2_A_X		-0.00001
#define	SCALING_SAS_2_A_X	1.0
#define	BIAS_SAS_3_A_X		-0.00001
#define	SCALING_SAS_3_A_X	1.0

#define	BIAS_SAS_1_A_Y		-0.00001
#define	SCALING_SAS_1_A_Y	1.0
#define	BIAS_SAS_2_A_Y		-0.00001
#define	SCALING_SAS_2_A_Y	1.0
#define	BIAS_SAS_3_A_Y		-0.00001
#define	SCALING_SAS_3_A_Y	1.0

#define	BIAS_SAS_1_B_X		0.0
#define	SCALING_SAS_1_B_X	1.0
#define	BIAS_SAS_2_B_X		0.0
#define	SCALING_SAS_2_B_X	1.0
#define	BIAS_SAS_3_B_X		0.0
#define	SCALING_SAS_3_B_X	1.0

#define	BIAS_SAS_1_B_Y		0.0
#define	SCALING_SAS_1_B_Y	1.0
#define	BIAS_SAS_2_B_Y		0.0
#define	SCALING_SAS_2_B_Y	1.0
#define	BIAS_SAS_3_B_Y		0.0
#define	SCALING_SAS_3_B_Y	1.0

// Thruster torque level (in Nm) and thruster control period (in milliseconds)
#define T_THU				10.0
#define THU_PERIOD			500

// Reaction wheel angular inertia
#define RW_INERTIA			0.1

// Maximum delta change on an FSS output before a failure is triggered
#define MAX_DELTA_FSS		0.001

// Maximum delta change on a SAS output before a failure is triggered
#define MAX_DELTA_SAS		0.001

// Maximum delta change on a GYR rate output before a failure is triggered
#define MAX_DELTA_GYR		0.001

// Legal range for FSS outputs
#define UPPER_RANGE_FSS		0.5
#define LOWER_RANGE_FSS		-0.5

// Legal range for SAS outputs
#define UPPER_RANGE_SAS		0.7
#define LOWER_RANGE_SAS		-0.7

// Legal range for GYR outputs
#define UPPER_RANGE_GYR		0.02
#define LOWER_RANGE_GYR		-0.02

// Max value for the attitude error size in FSP before a fallback to CSP
// is triggered
#define MAX_ATT_ERR_FSP		0.001

// Max number of failure events in a cycle before a system reset is triggered
#define MAX_FAIL_EVT		30

#endif
