// FailureDetectionFactory.h
//
// Version	1.0
// Date		14.09.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Factory Class for Failure Detection Framelet.
//
// Methods are exposed to construct the failure detection manager, its
// mode manager and monitoring check objects.
//
// This class should be customized (by inheritance) for each AOCS
// application.
// -------------------------------------------------------------------

#ifndef FailureDetectionFactoryH
#define FailureDetectionFactoryH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"

// -------------------------------------------------------------------

class FailureDetectionFactory {

	FailureDetectionManager* fdManager;
	FailureDetectionModeManager* fdModeManager;

	ObjectListTemplate<ConsistencyCheckable>** ccListArray;
	ObjectListTemplate<MonitoringCheck>** mcListArray;

protected:

	virtual void makeFailureDetectionLists();

public:

	FailureDetectionFactory();	

	// Construct and configure the failure detection mode manager
	virtual void makeFailureDetectionModeManager(
			ModeManager* masterModeManager, ModeChangeAction* mca);

	// Construct and configure the failure detection manager
	void makeFailureDetectionManager(ChangeEventRepository* rep);

	// Retrieve failure detection lists
	ObjectListTemplate<ConsistencyCheckable>* 
								getConsistencyCheckableList(int mode);
	ObjectListTemplate<MonitoringCheck>* 
								getMonitoringCheckList(int mode);

	// Construct a monitoring check object
	MonitoringCheck* makeMonitoringCheck(
						Property monitoredProperty, 
							ChangeObject* changeObject,
								RecoveryAction* recoveryAction);
	
	// Getter methods 
	FailureDetectionManager* getFailureDetectionManager()
		{ return fdManager; }
	FailureDetectionModeManager* getFailureDetectionModeManager()
		{ return fdModeManager; }
};

#endif
