// ManoeuvreFactory.cpp 
//
// Version	1.0
// Date		19.09.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Manoeuvre Management Framelet Factory Class 
// -------------------------------------------------------------------

// Include framework header files
#include "../ManoeuvreManagement/Manoeuvre.h"
#include "../ManoeuvreManagement/ManoeuvreManager.h"
#include "../ManoeuvreManagement/AttitudeSlew.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsEvent/EventTypeId.h"
#include "../AocsEvent/ConfigurationEventRepository.h"

// Include the AOCS configuration file
#include "../AocsFactory/AocsConfiguration.h"

// Include the AOCS factory files
#include "../AocsFactory/ManoeuvreFactory.h"

// -------------------------------------------------------------------

ManoeuvreFactory::ManoeuvreFactory()
{
	mmManager = NULL;

	// Construct the arrays to hold the concrete manoeuvre objects
	attitudeSlew = new AttitudeSlew*[N_ATT_SLEW_MAN];
}
	
// -------------------------------------------------------------------
// Create and configure the manoeuvre manager.

void ManoeuvreFactory::makeManoeuvreManager()
{
	if (mmManager == NULL)					// create only once
	{	mmManager = new ManoeuvreManager();
		
		mmManager->initializeManoeuvreList(MAX_N_PEND_MAN);
	}
}

// -------------------------------------------------------------------
// Make an attitude slew manoeuvre.

void ManoeuvreFactory::makeAttitudeSlews()
{
	for (int i=0; i<N_ATT_SLEW_MAN; i++)
	{	attitudeSlew[i] = new AttitudeSlew();
		attitudeSlew[i]->setMonitorListSize(MAX_N_MAN_MON);
	}
}

// -------------------------------------------------------------------
// Make concrete manoeuvre objects (at present, only attitude slews
// are constructed)

void ManoeuvreFactory::makeConcreteManoeuvres()
{
	makeAttitudeSlews();
}

// -------------------------------------------------------------------
// Return the i-th pre-constructed attitude slew

AttitudeSlew* ManoeuvreFactory::getAttitudeSlew(int i)
{
	if ((i<0) || (i>N_ATT_SLEW_MAN-1))
	{	AocsObject::getConfigurationEventRepository()->
								create(NULL,ILLEGAL_MANOEUVRE,NULL);
		return NULL;
	}
	else
		return attitudeSlew[i];
}
