// ManoeuvreFactory.h
//
// Version	1.0
// Date		19.09.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Factory Class for Manoeuvre Management Framelet.
//
// The factory constructs a manoeuvre manager and arrays of
// unconfigured manoeuvre components that are then available to be 
// configured and used by other components.
// In the present implementation, the only concrete manoeuvre  thus
// constructed are attitude slews.
//
// This class should be customized (by inheritance) for each AOCS
// application.
// -------------------------------------------------------------------

#ifndef ManoeuvreFactoryH
#define ManoeuvreFactoryH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"

// -------------------------------------------------------------------

class ManoeuvreFactory {

	ManoeuvreManager* mmManager;

	AttitudeSlew** attitudeSlew;		// array of attitude slews

protected:

	void makeAttitudeSlews();

public:

	ManoeuvreFactory();	

	// Construct and configure the manoeuvre manager
	void makeManoeuvreManager();

	// Construct the concrete manoeuvre objects
	void makeConcreteManoeuvres();

	// Retrieve i-th attitude slew object
	AttitudeSlew* getAttitudeSlew(int i);

	// Getter methods 
	ManoeuvreManager* getManoeuvreManager()
		{ return mmManager; }
};

#endif
