// TelecommandFactory.cpp 
//
// Version	1.0
// Date		08.09.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Factory for telecommand management framelet
// -------------------------------------------------------------------

// Include framework header files
#include "../GeneralInclude/CompilerSwitches.h"
#include "../TelecommandManagement/TelecommandManager.h"
#include "../TelecommandManagement/TransactionTelecommand.h"
#include "../TelecommandManagement/VsDmaTelecommandLoader.h"
#include "../TelecommandManagement/Erc32DmaTelecommandLoader.h"
#include "../TelecommandManagement/DmaTelecommandLoader.h"

// Include the factory declaration files
#include "../AocsFactory/TelecommandFactory.h"

// Include the AOCS configuration file
#include "../AocsFactory/AocsConfiguration.h"

// -------------------------------------------------------------------

TelecommandFactory::TelecommandFactory()
{
	// Initialize all internal variables
	tcManager = NULL;
	tcLoader = NULL;
	tcBuffer = NULL;
}

// -------------------------------------------------------------------
// Create and configure the telecommand manager

void TelecommandFactory::makeTelecommandManager(
	RecoveryAction* tcListFull, RecoveryAction* tcStackError)
{
	if (tcManager == NULL)	// only do once!
	{	tcManager = new TelecommandManager(MAX_N_TC); 

		tcManager->setTelecommandListFullRecoveryAction(tcListFull);
		tcManager->setTelecommandStackErrorRecoveryAction(tcStackError);

		TransactionTelecommand::setTelecommandManager(tcManager);
	}
}

// -------------------------------------------------------------------
// Create the telecommand loader.
// This version of the factory can produce two DMA telecommand loader:
// the Visual Studio and the ERC32 DMA telecommand loader. The 
// compiler switch MS_HOST controls which version is manufactured.
// The first parameter is the start address of the TC buffer. If this
// parameter is NULL, the factory method assumes that there is no
// hardware TC interface and that the TC buffer has to be created
// internally.

void TelecommandFactory::makeTelecommandLoader(
						int* tcBuf, RecoveryAction* tcLoaderError)
{
	if (tcLoader == NULL)	// only do once!
	{	
		if (tcBuf!=NULL)
			tcBuffer = tcBuf;
		else
			tcBuffer = new int[TC_BUFFER_SIZE/4];

		DmaTelecommandLoader* dmaTcLoader;

#ifdef MS_HOST
		dmaTcLoader = new VsDmaTelecommandLoader(tcBuffer); 
#else
		dmaTcLoader = new Erc32DmaTelecommandLoader(tcBuffer); 
#endif

		dmaTcLoader->setTelecommandLoaderRecoveryAction(tcLoaderError);

		tcLoader = dmaTcLoader;
	}
}

