// TelecommandFactory.h
//
// Version	1.0
// Date		08.09.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Factory Class for the Telecommand Management Framelet.
//
// This class should be customized (by inheritance) for each AOCS
// application.
//
// The factory produces the following items:
//	- the telecommand manager
//	- the telecommand loader
//
// This version of the factory can produce two DMA telecommand loader:
// the Visual Studio and the ERC32 DMA telecommand loader. The 
// compiler switch MS_HOST controls which version is manufactured by
// method makeTelecommandLoader.
// -------------------------------------------------------------------

#ifndef TelecommandFactoryH
#define TelecommandFactoryH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"

// -------------------------------------------------------------------

class TelecommandFactory {

protected :

	TelecommandManager* tcManager;

	TelecommandLoader* tcLoader;

	int* tcBuffer;

public:

	// The argument is the address of the telecommand buffer  
	TelecommandFactory();	

	// Make the TC manager and the TC loader
	// - tcBuffer : address of DMA telecommand buffer (only useful in 
	//		the case of DMA-based TC loaders)
	// - tcListFull : recovery action invoked when the TC loader
	//		attempts to load a TC in a TC manager which is already full
	// - tcStackError : recovery action invoked when a transaction TC
	//		performs an illegal stack operation
	// - tcLoaderError : recovery action invoked when the TC loader
	//		encounters an error
	void makeTelecommandManager(
		RecoveryAction* tcListFull,RecoveryAction* tcStackError);

	void makeTelecommandLoader(
					int* tcBuffer,RecoveryAction* tcLoaderError);

	// Getter methods for TC manager, TC loader and TC buffer
	TelecommandManager* getTelecommandManager()
		{ return tcManager; }

	TelecommandLoader* getTelecommandLoader()
		{ return tcLoader; }

	int* getTelecommandBuffer()
		{ return tcBuffer;}
};

#endif
