// AocsTestHarness.cpp
//
// Version	1.0
// Date		16.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include system header files
#include <stdio.h>  

// Include framework header files
#include "../TelemetryManagement/DmaTelemetryStream.h"
#include "../AocsUnit/AocsUnit.h"

// Include the AOCS factory and AOCS configuration headers
#include "../AocsFactory/AocsUnitFactory.h"
#include "../AocsFactory/TelemetryFactory.h"
#include "../AocsFactory/AocsConfiguration.h"

// Include test header files
#include "../AocsPrototypeTest/AocsTestHarness.h"
#include "../GeneralUtilities/TestUtilities.h"

// -------------------------------------------------------------------

AocsTestHarness::AocsTestHarness(FrameworkFactories f, FILE* od)
{
	out = od;

	tmCounter = 0;
	macsCounter = 0;
	fwFact = f;

	tmBuffer = new char[TM_SAVE*TM_BUFFER_SIZE];
	macsBuffer = new unsigned short[MACS_SAVE*N_MACS_OUT_HW_BUF];
}

// -------------------------------------------------------------------
// Store the current TM frame in an internal buffer

void AocsTestHarness::saveTelemetry()
{
	DmaTelemetryStream* tmStream = 
		(DmaTelemetryStream*)fwFact.telemetryFactory->getTelemetryStream();

	for (int i=0; i<TM_BUFFER_SIZE; i++)
		tmBuffer[ tmCounter*TM_BUFFER_SIZE + i] = *(tmStream->getDmaBufferStartAddress()+i);

	tmCounter++;
	if (tmCounter==TM_SAVE)
		tmCounter=0;
}

// -------------------------------------------------------------------
// Save the content of the MACS hardware buffers holding outgoing data

void AocsTestHarness::saveMacs()
{
	int counter=0;
	AocsUnitFactory* unitFactory = fwFact.aocsUnitFactory;

	for (int j=0; j<4; j++)
	{	macsBuffer[macsCounter*N_MACS_OUT_HW_BUF+counter] = unitFactory->
			getRw(j)->getFcHwOutBuff(0);
		counter++;
	}

	for (j=0; j<12; j++)
	{	macsBuffer[macsCounter*N_MACS_OUT_HW_BUF+counter] = unitFactory->
			getSapA()->getFcHwOutBuff(j);
		counter++;
	}

	for (j=0; j<12; j++)
	{	macsBuffer[macsCounter*N_MACS_OUT_HW_BUF+counter] = unitFactory->
			getSapB()->getFcHwOutBuff(j);
		counter++;
	}

	macsCounter++;
	if (macsCounter==MACS_SAVE)
		macsCounter=0;
	
	// Check that the number of saved MACS words does not exceed the 
	// MACS buffer capacity
	if (counter>N_MACS_OUT_HW_BUF)
		printMessage(out,"MACS buffer is not large enough");
}

// -------------------------------------------------------------------

void AocsTestHarness::run(AocsTime t)
{
	loadTelecommands();
	loadMacs();
	generateActuatorCommands();
	saveTelemetry();
	saveMacs();
}



