// TestCaseAocsUnit_1.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseAocsUnit_1.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../AocsUnit/MacsTcController.h"

// -------------------------------------------------------------------
TestCaseAocsUnit_1::TestCaseAocsUnit_1():
	TestCaseGenericSetUp(ID_UNITTRIGGER*10+1,
									"TestCaseAocsUnit_1") {
		return;
}

// -------------------------------------------------------------------

void TestCaseAocsUnit_1::runTestCase() {

	// The following locations represent the MACS registers
	short macsCs=0;
	short macsInstr=0;
	short macsData=0;

	// Create the MACS TC object
	MacsTcController macsTcCont(&macsCs, &macsInstr, &macsData);

	// Check that the reet bit in the MACS_CS reguister was reset 
	if (macsCs!=0x0020)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 1 failed");
		return;
	}

	// Check the reset transaction method
	macsCs=0;
	macsTcCont.resetTransaction();
	if (macsCs!=0x0020)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 1 failed");
		return;
	}

	// Build a MACS instruction
	UnitInstruction instr;
	instr.command=2;
	instr.destAddress=4;
	instr.priority=6;
	instr.subAddress=5;
	unsigned short datum=0x0033;
	instr.data=&datum;

	// Test the non-blocking send (blocking send cannot be tested)
	macsTcCont.sendAndReturn(instr);
	if (macsData!=0x0033)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 1 failed");
		return;
	}
	if (macsInstr!=0x4526)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 1 failed");
		return;
	}
	if ((macsCs & 0x0001) != 1)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 1 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}


