// TestCaseAocsUnit_3.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseAocsUnit_3.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../SequentialDataProcessing/TwoByTwoMatrixBlock.h"
#include "../../SequentialDataProcessing/RawBusDataConverterIn.h"
#include "../../SequentialDataProcessing/RawBusDataConverterOut.h"
#include "../../AocsUnit/MacsTcController.h"
#include "../../AocsUnit/RwPrototype.h"
#include "../../FailureRecovery/NullRecoveryAction.h"

// -------------------------------------------------------------------
TestCaseAocsUnit_3::TestCaseAocsUnit_3():
	TestCaseGenericSetUp(ID_UNITTRIGGER*10+3,
									"TestCaseAocsUnit_3") {
		return;
}

// -------------------------------------------------------------------

void TestCaseAocsUnit_3::runTestCase() {

	// Retrieve the number of failure and configuration events
	int fen = AocsObject::getFailureEventRepository()->getCounter();
	int cen = AocsObject::getConfigurationEventRepository()->getCounter();

	// The following locations represent the MACS registers
	short macsCs=0;
	short macsInstr=0;
	short macsData=0;

	// Create and configure the MACS TC object
	MacsTcController macsTcCont(&macsCs, &macsInstr, &macsData);
	macsTcCont.setMaxPolling(2);

	// Create and configure raw bus data converters for incoming and
	// outgoing data
	RawBusDataConverterIn convIn(1,0.1);
	RawBusDataConverterOut convOut(1,10);

	// Create a dummy recovery action
	NullRecoveryAction recAct;

	// Create the destination buffer for the speed measurements and
	// their time tag
	Real rwSpeed;
	AocsTime rwSpeedTag;
	DataItemWrite rwSpeedDiw(&rwSpeedTag, &rwSpeed);

	// Create the source buffer for the torque request
	Real rwTorque;
	AocsTime rwTorqueTag;
	DataItemWrite rwTorqueDir(&rwTorqueTag, &rwTorque);

	// Create and configure the RW object
	RwPrototype rw(10,11,12);
	rw.setAocsUnitHardware(&macsTcCont);
	rw.setFunctionalInConverter(&convIn);
	rw.setFunctionalInLink(rwSpeedDiw,0);
	rw.setFunctionalOutConverter(&convOut);
	rw.setFunctionalOutLink(rwTorqueDir,0);
	rw.setHwFailureRecoveryAction(&recAct);

	// Send a torque request to the RW
	rwTorque = -0.4;
	rw.refreshFunctionalOut();
	rw.sendFunctionalData();

	// Check that the MACS data and instruction registers have the expected values 
	if (macsInstr!=0x4C56)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 3 failed");
		return;
	}
	if (macsData!=0x0004)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 3 failed");
		return;
	}

	// Since there is no real external unit, a bus error should have been reported
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+1)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 3 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->latestFailureEvent()->
			getEventType() != NO_BUS_TRANSMISSION)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 3 failed");
		return;
	}

	// Simulate acquisition of a speed measurement by the RW
	unsigned short speedMeas = 0x8030;
	rw.setFcHwInpBuff(speedMeas,0);
	rw.refreshFunctionalIn();
	if ( (rwSpeed > 4.81) || (rwSpeed < 4.79) )
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 3 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}


