// TestCaseAocsUnit_4.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseAocsUnit_4.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../SequentialDataProcessing/TwoByTwoMatrixBlock.h"
#include "../../SequentialDataProcessing/RawBusDataConverterIn.h"
#include "../../SequentialDataProcessing/RawBusDataConverterOut.h"
#include "../../SequentialDataProcessing/BiasScalingCompensator.h"
#include "../../SequentialDataProcessing/ControlChannelSuperBlock.h"
#include "../../AocsUnit/MacsTcController.h"
#include "../../AocsUnit/FssPrototype.h"
#include "../../FailureRecovery/NullRecoveryAction.h"

// -------------------------------------------------------------------
TestCaseAocsUnit_4::TestCaseAocsUnit_4():
	TestCaseGenericSetUp(ID_UNITTRIGGER*10+4,
									"TestCaseAocsUnit_4") {
		return;
}

// -------------------------------------------------------------------

void TestCaseAocsUnit_4::runTestCase() {

	// Retrieve the number of failure and configuration events
	int fen = AocsObject::getFailureEventRepository()->getCounter();
	int cen = AocsObject::getConfigurationEventRepository()->getCounter();

	// The following locations represent the MACS registers
	short macsCs=0;
	short macsInstr=0;
	short macsData=0;

	// Create and configure the MACS TC object
	MacsTcController macsTcCont(&macsCs, &macsInstr, &macsData);
	macsTcCont.setMaxPolling(2);

	// Create the raw data converters for the FSS measurements
	RawBusDataConverterIn convIn(2,0.1);

	// Create the bias and scaling factor converter for the FSS measurements
	BiasScalingCompensator bsc_x(4.0,2.0);
	BiasScalingCompensator bsc_y(4.0,4.0);

	// Link the raw data converter and the bias/scaling compensator
	bsc_x.linkInput(&convIn,0,0);
	bsc_y.linkInput(&convIn,1,0);

	// Create and configure the converter superblock for the FSS
	ControlChannelSuperBlock fssConv(2,2);
	fssConv.linkInputInternal(&convIn,0,0);
	fssConv.linkInputInternal(&convIn,1,1);
	fssConv.linkOutputInternal(&bsc_x,0,0);
	fssConv.linkOutputInternal(&bsc_y,0,1);

	// Create a dummy recovery action
	NullRecoveryAction recAct;

	// Create the destination buffer for the FSS measurements and
	// their time tag
	TwoEulerAngles fssData;

	// Create and configure the FSS object
	FssPrototype fss(10,11,12);
	fss.setAocsUnitHardware(&macsTcCont);
	fss.setFunctionalInConverter(&fssConv);
	fss.setFunctionalInLink(&fssData);
	fss.setHwFailureRecoveryAction(&recAct);

	// Check that no configuration errors were generated
	if (AocsObject::getConfigurationEventRepository()->getCounter() != cen)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 4 failed");
		return;
	}

	// Check that the FSS object is configured
	if (!fss.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 4 failed");
		return;
	}

	// Simulate acquisition of x and y channels measurement from the FSS
	unsigned short raw_fss_x = 0x8030;
	unsigned short raw_fss_y = 0x0060;

	fss.setFcHwInpBuff(raw_fss_x,0);
	fss.setFcHwInpBuff(raw_fss_y,1);
	fss.refreshFunctionalIn();
	if ( (fssData.getDataItemRead(0).get() < 13.5) || 
		 (fssData.getDataItemRead(0).get() > 13.7) )
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 4 failed");
		return;
	}
	if ( (fssData.getDataItemRead(1).get() < -34.5) || 
		 (fssData.getDataItemRead(1).get() > -34.3) )
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 4 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}


