// FssPrototype.cpp
//
// Version	1.0
// Date		29.05.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../AocsUnit/FssPrototype.h"			
#include "../AocsUnit/AocsUnitHardware.h"
#include "../AocsUnit/MacsConstants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../AocsEvent/EventTypeId.h"				

// -------------------------------------------------------------------

FssPrototype::FssPrototype(unsigned short a, 
							unsigned short b, 
							 unsigned short c):
					AocsUnit(0,0,2,2,0,0)
{
	setClassId(ID_FSSPROTOTYPE);			

	address = a;
	sa_fss_x = b;
	sa_fss_y = c;
}

// -------------------------------------------------------------------
// Send the MACS RC to initialize the MACS bus head of the FSS

void FssPrototype::initialize()
{
	UnitInstruction instr={MACS_INSTR,address,1,MACS_RC,(unsigned short*)fcHwInpBuf,0};
	if (!hwUnit->sendAndWait(instr))
		reportFailure(this,NO_BUS_TRANSMISSION,this,hwFailureRecAct);
	return;
}

// -------------------------------------------------------------------
// Send a MACS broadcast. 

void FssPrototype::synchronizeFunctional()
{
	UnitInstruction instr={MACS_BRDC,address,1,MACS_RC,(unsigned short*)fcHwInpBuf,0};
	if (!hwUnit->sendAndWait(instr))
		reportFailure(this,NO_BUS_TRANSMISSION,this,hwFailureRecAct);
	return;
}

// -------------------------------------------------------------------
// Send two MACS TI instructions to the subaddresses identifying the 
// the FSS X and FSS Y channels. 

void FssPrototype::acquireFunctionalData()
{
	UnitInstruction instr;
	
	instr.priority = MACS_INSTR;
	instr.destAddress = address;
	instr.subAddress = sa_fss_x;
	instr.command = MACS_TI;
	instr.data=(unsigned short*)&fcHwInpBuf[0];

	if (!hwUnit->sendAndWait(instr))		// acquire channel X measurement
		reportFailure(this,NO_BUS_TRANSMISSION,this,hwFailureRecAct);

	instr.subAddress = sa_fss_y;
	instr.data=(unsigned short*)&fcHwInpBuf[1];

	if (!hwUnit->sendAndWait(instr))		// acquire channel Y measurement
		reportFailure(this,NO_BUS_TRANSMISSION,this,hwFailureRecAct);

	return;
}




