// FunctionalTriggerList.cpp
//
// Version	1.0
// Date		08.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsUnit/FunctionalTriggerList.h"			

// -------------------------------------------------------------------
// Define static variables
int FunctionalTriggerList::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

FunctionalTriggerList::FunctionalTriggerList(int fcSyn, int fcAcq, 
											 int fcSnd, int hkIni)
{
	tmImageLength[shortTm]  = 0;	// Not implemented in fw prototype	
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	setClassId(ID_TRIGGERLIST);	

	fcSynch.allocateMemory(fcSyn);
	fcAcquire.allocateMemory(fcAcq);
	fcSend.allocateMemory(fcSnd);
	hkInit.allocateMemory(hkIni);

	localResetConfiguration();
	localReset();
}

// -------------------------------------------------------------------

void FunctionalTriggerList::localReset()
{
	fcSynch.reset();
	fcAcquire.reset();
	fcSend.reset();
	hkInit.reset();
}
	
// -------------------------------------------------------------------

void FunctionalTriggerList::localResetConfiguration()
{
	fcSynch.resetConfiguration();
	fcAcquire.resetConfiguration();
	fcSend.resetConfiguration();
	hkInit.resetConfiguration();
}	

// -------------------------------------------------------------------

void FunctionalTriggerList::reset()
{
	AocsObject::reset();
	localReset();
}


// -------------------------------------------------------------------

void FunctionalTriggerList::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool FunctionalTriggerList::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (!fcSynch.isConfigured())
		return false;

	if (!fcAcquire.isConfigured())
		return false;

	if (!fcSend.isConfigured())
		return false;

	if (!hkInit.isConfigured())
		return false;

	return true;
}

// -------------------------------------------------------------------

void FunctionalTriggerList::initialize()
{
	for (AocsUnitHousekeeping* h=hkInit.first();
			!hkInit.isLast(); 
				h=hkInit.next())
	{
		h->switchOn();
		h->initialize();
	}
}

// -------------------------------------------------------------------

void FunctionalTriggerList::shutdown()
{
	for (AocsUnitHousekeeping* h = hkInit.first(); 
			!hkInit.isLast();
				h = hkInit.next())
		h->switchOff();
}

// -------------------------------------------------------------------

void FunctionalTriggerList::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// not implemented in framework prototype
}

// -------------------------------------------------------------------

int FunctionalTriggerList::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}
