// MacsTcController.h
//
// Version	1.0
// Date		27.05.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Encapsulation of interface to MACS telecom controller.
// -------------------------------------------------------------------

#ifndef MacsTcControllerH
#define MacsTcControllerH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/RootObject.h"
#include "../AocsUnit/AocsUnitHardware.h"

// -------------------------------------------------------------------

class MacsTcController : public RootObject, public AocsUnitHardware {

	// Masks to identify important bits in MACS_CS register
	#define SEND			0x0001 
	#define FINISHED		0x0002
	#define FINISHED_INV	0xFFFD
	#define NO_ACK			0x0004 
	#define PARITY_ERR		0x0008
	#define ERROR_SET		0x0010
	#define RESET			0x0020
	#define ERROR			0x001C 

	// Masks to identify bit fields in the MACS_INSTR register
	#define EXT		0xFFF8
	#define DEST	0xFF07
	#define SUBADD	0xE0FF
	#define CMD		0x1FFF

	// Number of times the MACS_CS register is polled when
	// waiting for a bus transaction to  be completed
	int maxPoll;

	volatile short* MacsCs;		// address of MACS_CS register
	volatile short* MacsInstr;	// address of MACS_INSTR register
	volatile short* MacsData;	// address of MACS_DATA register

public:

	MacsTcController(short* MacsCs, short* MacsInstr, short* MacsData);
		
	virtual void resetInterface();

	virtual void resetTransaction();
	virtual bool sendAndWait(UnitInstruction instr);
	virtual void sendAndReturn(UnitInstruction instr);
	virtual bool wasTransactionSuccessful();
	virtual bool isTransactionFinished();

	// Methods that cannot be implemented by a MACS TC controller 
	virtual void switchOn()
		{ return; }

	virtual void switchOff()
		{ return; }

	virtual bool isSwitchOn()
		{ return false; }

	virtual int getAddress()
		{ return 0; }

	// Setter and getter methods for the polling ceiling
	int getMaxPolling()
		{ return maxPoll;}
	void setMaxPolling(int n)
		{ maxPoll = n;}
};

// -------------------------------------------------------------------
#endif