// UnitTrigger.cpp
//
// Version	1.0
// Date		02.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../AocsUnit/UnitTrigger.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../AocsUnit/UnitTriggerModeManager.h"
#include "../AocsUnit/TriggerList.h"
#include "../AocsUnit/AocsUnitFunctional.h"
#include "../AocsUnit/AocsUnitHousekeeping.h"

// -------------------------------------------------------------------
// Define static variables
int UnitTrigger::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

UnitTrigger::UnitTrigger()
{
	setClassId(ID_UNITTRIGGER);	

	tmImageLength[shortTm]  = 0;			// not implemented in f/w prototype
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	localResetConfiguration();
}

// -------------------------------------------------------------------

void UnitTrigger::localResetConfiguration()
{
	modeManager = NULL;
}

// -------------------------------------------------------------------

void UnitTrigger::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool UnitTrigger::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (modeManager==NULL)
		return false;
	
	return true;
}

// -------------------------------------------------------------------

void UnitTrigger::run(AocsTime deadline)
{
	// Load the trigger list
	TriggerList* list = modeManager->getTriggerList();

	// Send synchronization messages to hk units in the list 
	for (AocsUnitHousekeeping* h=list->firstHousekeepingSynchronize();
			!list->isLastHousekeepingSynchronize(); 
				h=list->nextHousekeepingSynchronize())
		h->synchronizeHousekeeping();

	// Acquire housekeeping data from units in the list 
	for (h=list->firstHousekeepingAcquire();
			!list->isLastHousekeepingAcquire(); 
				h=list->nextHousekeepingAcquire())
	{	h->acquireHousekeepingData();
		h->refreshHousekeepingIn();
	}

	// Send synchronization messages to fc units in the list 
	for (AocsUnitFunctional* f=list->firstFunctionalSynchronize();
			!list->isLastFunctionalSynchronize(); 
				f=list->nextFunctionalSynchronize())
		f->synchronizeFunctional();

	// Acquire functional data from units in the list 
	for (f=list->firstAcquireFunctional();
			!list->isLastAcquireFunctional(); 
				f=list->nextAcquireFunctional())
	{	f->acquireFunctionalData();
		f->refreshFunctionalIn();
	}

	// Send functional data to units in the list 
	for (f=list->firstSendFunctional();
			!list->isLastSendFunctional(); 
				f=list->nextSendFunctional())
	{	f->sendFunctionalData();
		f->refreshFunctionalOut();
	}
}

// -------------------------------------------------------------------
// From Telemeterable interface

int UnitTrigger::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void UnitTrigger::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}
