// AocsObject.cpp
//
// Version	1.0
// Date		14.01.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../AocsEvent/EventTypeId.h"
#include "../BasicObjects/AocsObject.h"
#include "../OperatingSystemObjects/AocsClock.h"
#include "../AocsEvent/FailureEventRepository.h"
#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../TelemetryManagement/Telemeterable.h"
#include "../TelemetryManagement/TelemetryStream.h"
#include "../SystemManagement/SystemManager.h"

// -------------------------------------------------------------------
// Define static variables
AocsClock*						AocsObject::aocsClock=NULL;
FailureEventRepository*			AocsObject::failureEvtRep=NULL;
ConfigurationEventRepository*	AocsObject::configEvtRep=NULL;
SystemManager*					AocsObject::systemManager=NULL;
RecoveryAction*					AocsObject::aocsDestructionRecAct=NULL;

int AocsObject::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

AocsObject::AocsObject()
{
	setClassId(ID_AOCSOBJECT);			

	if (systemManager!=NULL)
		systemManager->add(this);		

	tmImageLength[shortTm]  = 8*sizeof(short)+8*sizeof(ClassId);			
	tmImageLength[normalTm] = tmImageLength[shortTm] ;
	tmImageLength[longTm]   = tmImageLength[normalTm] ;
	tmImageLength[debugTm]  = tmImageLength[longTm] + 24*sizeof(InstanceId);

	reset();							// reset object's state
	
	resetConfiguration(); 				// reset object's configuration
}

// -------------------------------------------------------------------

AocsObject::~AocsObject()
{
	reportFailure(this,AOCS_OBJECT_DESTRUCTION,this,aocsDestructionRecAct);
}
	
// -------------------------------------------------------------------

AocsTime AocsObject::getTime()
{
	return aocsClock->getTime();
}

// -------------------------------------------------------------------

AocsCycle AocsObject::getCycle()
{
	return aocsClock->getCycle();
}

// -------------------------------------------------------------------

void AocsObject::reportFailure(AocsObject* creator, 
								EventType failureType,
								 AocsObject* location, 
								  RecoveryAction* recoveryAction)
{
	failureEvtRep->create(creator,failureType,location,recoveryAction);
}

// -------------------------------------------------------------------

void AocsObject::reportConfigurationError(AocsObject* creator, 
										   EventType configErrType,
										    AocsObject* location)
{
	configEvtRep->create(creator,configErrType,location);
}

// -------------------------------------------------------------------
// From Resettable interface

void AocsObject::reset()
{
}

// -------------------------------------------------------------------
// From Configurable interface

bool AocsObject::isConfigured()
{
	if ((aocsClock!=NULL) && 
		(failureEvtRep!=NULL) &&
		(configEvtRep!=NULL) &&
		(aocsDestructionRecAct!=NULL))
		return true;
	else
		return false;

}

// -------------------------------------------------------------------
// From Configurable interface

void AocsObject::resetConfiguration()
{
	currentTmFormat = normalTm;
}

// -------------------------------------------------------------------

void AocsObject::resetStaticConfiguration()
{
	aocsClock=NULL;
	failureEvtRep=NULL;
	configEvtRep=NULL;
	aocsDestructionRecAct=NULL;
	systemManager=NULL;
}

// -------------------------------------------------------------------
// From Telemeterable interface

void AocsObject::setTelemetryFormat(TelemetryFormat newFormat)
{
	currentTmFormat = newFormat;
}

// -------------------------------------------------------------------
// From Telemeterable interface

TelemetryFormat AocsObject::getTelemetryFormat()
{
	return currentTmFormat;
}

// -------------------------------------------------------------------
// From Telemeterable interface

int AocsObject::getTelemetryImageLength()
{
	return tmImageLength[getTelemetryFormat()];
}

// -------------------------------------------------------------------
// From Telemeterable interface

void AocsObject::writeToTelemetry(TelemetryStream* tmStream)
{
	// Write class ID 
	tmStream->write(getClassId());

	// Write instance ID + current TM format
	short temp1;	// assumes a short has 16 bits

	temp1 = getInstanceId();	// assumes: 0<instanceId<16383 (=2**14-1)
	temp1 = temp1<<2;			// put instanceId in leftmost 14 bits

	temp1 += currentTmFormat;	// put TM format in rightmost 2 bits

	tmStream->write(temp1);

	if ( currentTmFormat == debugTm )
	{	tmStream->write(((RootObject*)aocsClock)->getInstanceId());
		tmStream->write(failureEvtRep->getInstanceId());
		tmStream->write(configEvtRep->getInstanceId());
	}
}

