// TestCaseController_0.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseController_0.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsEvent/AocsEvent.h"
#include "../../SequentialDataProcessing/P_Block.h"
#include "../../SequentialDataProcessing/D_Block.h"
#include "../../SequentialDataProcessing/I_Block.h"
#include "../../SequentialDataProcessing/AbstractControlChannel.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../ControllerManagement/Controller.h"

// -------------------------------------------------------------------
TestCaseController_0::TestCaseController_0():
	TestCaseGenericSetUp(ID_CONTROLLERMANAGER*10+0,
									"TestCaseController_0") {
		return;
}

// -------------------------------------------------------------------

void TestCaseController_0::runTestCase() {

    // Create a Controller
    Controller myController;

    // Set and check the Compensator
    P_Block myProportionalTransferBlock;
    myProportionalTransferBlock.setGain(1.0);

    myController.setCompensator(&myProportionalTransferBlock);
    AbstractControlChannel* compensator = myController.getCompensator();

    if (compensator != &myProportionalTransferBlock)
	{	setTestResult(TEST_FAILURE, "Controller test 0 failed");
		return;
	}

    // Set and check the measurement filter
    D_Block myDerivativeBlock;
    myDerivativeBlock.setGain(2.0);

    myController.setMeasurementFilter(&myDerivativeBlock);
    AbstractControlChannel* measurementFilter = myController.getMeasurementFilter();

    if (measurementFilter != &myDerivativeBlock)
	{	setTestResult(TEST_FAILURE, "Controller test 0 failed");
		return;
	}

    // Set and check the feedforward filter
    I_Block myIntegratorBlock;
    myIntegratorBlock.setGain(3.0);

    myController.setFeedforwardFilter(&myIntegratorBlock);
    AbstractControlChannel* feedforwardFilter = myController.getFeedforwardFilter();

    if (feedforwardFilter != &myIntegratorBlock)
	{	setTestResult(TEST_FAILURE, "Controller test 0 failed");
		return;
	}

    // Set and check the reference signal filter
    LimitBlock myLimitBlock;
    myLimitBlock.setLimits(3.0, 6.0);

    myController.setReferenceSignalFilter(&myLimitBlock);
    AbstractControlChannel* referenceSignalFilter = myController.getReferenceSignalFilter();

    if (referenceSignalFilter != &myLimitBlock)
	{	setTestResult(TEST_FAILURE, "Controller test 0 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}

