// ControllerManager.cpp
//
// Version  1.0
// Date     04.11.2000
// Author   T. Brown
//
// -------------------------------------------------------------------

#include "../ControllerManagement/ControllerManager.h"
#include "../ControllerManagement/ControllerModeManager.h"
#include "../ControllerManagement/Controller.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../ObjectList/ObjectListTemplate.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------
// Sending data to telemetry is not implemented in f/w prototype,
// so all Telemetry Image Lengths are 0.
int ControllerManager::tmImageLength[NTMFORMATS] = {0, 0, 0, 0};

// -------------------------------------------------------------------
ControllerManager::ControllerManager()
{
    setClassId(ID_CONTROLLERMANAGER);

    localResetConfiguration();
}

// -------------------------------------------------------------------
void ControllerManager::localResetConfiguration()
{
    modeManager = NULL;
}

// -------------------------------------------------------------------
void ControllerManager::resetConfiguration()
{
    AocsObject::resetConfiguration();
    localResetConfiguration();
}

// -------------------------------------------------------------------
bool ControllerManager::isConfigured()
{
    if (!AocsObject::isConfigured()) return false;
    if (modeManager==NULL) return false;
    return true;
}

// -------------------------------------------------------------------
void ControllerManager::run(AocsTime deadline)
{
    ObjectListTemplate<Controller>* cList;
    Controller* c;

    cList = modeManager->getControllerList();

    for (c=cList->first(); !cList->isLast(); c=cList->next())
    {
        if (c->isStable())
        {
            c->doControl();
        }
        else
        {
            reportFailure(this, CONTROLLER_NOT_STABLE, c, c->getInstabilityRecoveryAction());
        }
    }

    return;
}

// -------------------------------------------------------------------
int ControllerManager::getTelemetryImageLength()
{
    return (tmImageLength[getTelemetryFormat()] + AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
void ControllerManager::writeToTelemetry(TelemetryStream* tmStream)
{
    AocsObject::writeToTelemetry(tmStream);     // write super object's state

    // TBD : not implemented in framework prototype
}

// -------------------------------------------------------------------
void ControllerManager::setModeManager(ControllerModeManager* cmm)
{
    modeManager = cmm;
}

// -------------------------------------------------------------------
ControllerModeManager* ControllerManager::getModeManager()
{
    return modeManager;
}
