// FailureRecoveryManager.h
//
// Version	1.0
// Date		12.05.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Failure recovery manager class
// -------------------------------------------------------------------

#ifndef FailureRecoveryManagerH
#define FailureRecoveryManagerH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"
#include "../OperatingSystemObjects/Runnable.h"

// -------------------------------------------------------------------

class FailureRecoveryManager : public AocsObject, public Runnable {

	static int tmImageLength[NTMFORMATS];

	// Mode manager to supply the recovery strategy
	FailureRecoveryModeManager* modeManager;

	// Local reset configuration method
	void localResetConfiguration();

public:

	// Constructor
	FailureRecoveryManager();

	// Scheduler entry point
	virtual void run(AocsTime deadline);

	// Virtual methods inherited from interface Configurable
	virtual void resetConfiguration();		
	virtual bool isConfigured();		
	
	// Virtual methods inherited from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// Straightforward getter and setter methods
	void setFailureRecoveryModeManager(FailureRecoveryModeManager* tmm)
		{ modeManager = tmm; }
	FailureRecoveryModeManager* getFailureRecoveryModeManager()
		{ return modeManager; }

};

// -------------------------------------------------------------------
#endif