// LocalRecoveryActions.cpp
//
// Version	1.0
// Date		10.05.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../FailureRecovery/LocalRecoveryActions.h"
#include "../FailureRecovery/RecoveryAction.h"
#include "../AocsEvent/FailureEventRepository.h"
#include "../AocsEvent/FailureEvent.h"
#include "../AocsEvent/RecoveryEventRepository.h"
#include "../AocsEvent/EventTypeId.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"

// -------------------------------------------------------------------
// Define static variables
int LocalRecoveryActions::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

LocalRecoveryActions::LocalRecoveryActions()
{
	setClassId(ID_LOCALRECOVERYACTIONS);		

	tmImageLength[shortTm]  = 0;	// not implemented in f/w prototype
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	localReset();
}

// -------------------------------------------------------------------

void LocalRecoveryActions::doRecovery()
{
	if (!isEnabled())
	{	RecoveryStrategy::doRecovery();
		return;
	}
	
	FailureEventRepository* rep=AocsObject::getFailureEventRepository();

	int counter = rep->getCounter();
	for (FailureEvent* evt=rep->latestFailureEvent();
			!rep->isIterationEnded();
				evt=rep->previousFailureEvent())
	{	if ( (counter==lastEventCounter) || (counter==0) )
			break;
		counter--;
		RecoveryAction* recoveryAction=evt->getRecoveryAction();
		if (recoveryAction == NULL)
			reportConfigurationError(this,MISSING_RECOVERY_ACTION,this);
		else
		{	recoveryAction->doRecovery();
			RecoveryStrategy::getRecoveryEventRepository()->
				create(this,LOCAL_RECOVERY_ACTION,recoveryAction,this);
		}
	}
	lastEventCounter = rep->getCounter();
	RecoveryStrategy::doRecovery();
}	
	
// -------------------------------------------------------------------

void LocalRecoveryActions::localReset()
{
	lastEventCounter = AocsObject::getFailureEventRepository()->getCounter();
}

// -------------------------------------------------------------------

void LocalRecoveryActions::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

int LocalRecoveryActions::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------

void LocalRecoveryActions::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}
