// RecoveryStrategy.h
//
// Version	1.0
// Date		10.05.00
// Author	A. Pasetti

#ifndef RecoveryStrategyH
#define RecoveryStrategyH

// -------------------------------------------------------------------
// Base class for recovery strategies.
// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"

// -------------------------------------------------------------------

class RecoveryStrategy : public AocsObject {

	static int tmImageLength[NTMFORMATS];

	static RecoveryEventRepository* recoveryEventRepository;
	
	bool enabled;
	RecoveryStrategy* nextRecoveryStrategy;

	void localReset();
	void localResetConfiguration();

public :

	RecoveryStrategy();

	virtual void doRecovery();

	// Virtual methods inherited from interface Configurable
	virtual void resetConfiguration();	
	virtual bool isConfigured();

	// Reset the static part of the RecoveryStrategy configuration
	static void resetStaticConfiguration();

	// Virtual methods inherited from interface Resettable
	virtual void reset();		
	
	// Virtual methods inherited from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// Straightforward getter and setter methods
	void setNextRecoveryStrategy(RecoveryStrategy* r)
		{ nextRecoveryStrategy = r; }
	RecoveryStrategy* getNextRecoveryStrategy()
		{ return nextRecoveryStrategy; }

	static void setRecoveryEventRepository(RecoveryEventRepository* r) 
			{recoveryEventRepository = r;}
	static RecoveryEventRepository* getRecoveryEventRepository() 
			{return recoveryEventRepository;}

	// Methods to manage the enable status
	void enable()
		{ enabled = true; }
	void disable()
		{ enabled = false; }
	bool isEnabled()
		{ return enabled; }
};

// -------------------------------------------------------------------
#endif