// SystemResetOnTooManyFailures.cpp
//
// Version	1.0
// Date		10.05.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../FailureRecovery/SystemResetOnTooManyFailures.h"
#include "../FailureRecovery/RecoveryStrategy.h"
#include "../SystemManagement/SystemManager.h"
#include "../AocsEvent/FailureEventRepository.h"
#include "../AocsEvent/RecoveryEventRepository.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------
// Define static variables
int SystemResetOnTooManyFailures::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

SystemResetOnTooManyFailures::SystemResetOnTooManyFailures()
{
	setClassId(ID_SYSTEMRESETONTOOMANYFAILURES);		

	tmImageLength[shortTm]  = 0;	// not implemented in f/w prototype
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	localResetConfiguration();
	localReset();
}

// -------------------------------------------------------------------
// Warning: this version contains no overflow check on lastEventCounter!

void SystemResetOnTooManyFailures::doRecovery()
{
	if (!isEnabled())
	{	RecoveryStrategy::doRecovery();
		return;
	}

	if ( (AocsObject::getFailureEventRepository()->getCounter() - 
			lastEventCounter) > failureEventThreshold )
	{	AocsObject::getSystemManager()->systemReset();
		RecoveryStrategy::getRecoveryEventRepository()->
			create(this,SYSTEM_RESET_RECOVERY,(RecoveryAction*)NULL,this);
	}
	else
	{	lastEventCounter = AocsObject::getFailureEventRepository()->getCounter();
		RecoveryStrategy::doRecovery();
	}
}	
	
// -------------------------------------------------------------------

void SystemResetOnTooManyFailures::localReset()
{
	lastEventCounter = AocsObject::getFailureEventRepository()->getCounter();
}

// -------------------------------------------------------------------

void SystemResetOnTooManyFailures::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

void SystemResetOnTooManyFailures::localResetConfiguration()
{
	failureEventThreshold = 0;
}

// -------------------------------------------------------------------

void SystemResetOnTooManyFailures::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool SystemResetOnTooManyFailures::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (failureEventThreshold==0)
		return false;
	
	return true;
}

// -------------------------------------------------------------------

int SystemResetOnTooManyFailures::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------

void SystemResetOnTooManyFailures::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}
