// SystemResetOnTooManyFailures.h
//
// Version	1.0
// Date		10.05.00
// Author	A. Pasetti

#ifndef SystemResetOnTooManyFailuresH
#define SystemResetOnTooManyFailuresH

// -------------------------------------------------------------------
// Recovery strategy that commands a system reset if the number of
// failure events generated since the last activation exceeds a 
// pre-defined threshold .
// -------------------------------------------------------------------

#include "../FailureRecovery/RecoveryStrategy.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"

// -------------------------------------------------------------------

class SystemResetOnTooManyFailures : public RecoveryStrategy {

	static int tmImageLength[NTMFORMATS];

	int lastEventCounter;
	int failureEventThreshold;

	void localReset();
	void localResetConfiguration();

public :

	SystemResetOnTooManyFailures();

	virtual void doRecovery();

	// Virtual methods inherited from interface Configurable
	virtual void resetConfiguration();	
	virtual bool isConfigured();

	// Virtual methods inherited from interface Resettable
	virtual void reset();		

	// Virtual methods inherited from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// Straightforward getter and setter methods
	void setFailureEventThreshold(int n)
		{ failureEventThreshold = n; }
	int getFailureEventThreshold()
		{ return failureEventThreshold; }

};

// -------------------------------------------------------------------
#endif