// BasicTypes.h
//
// Version	1.0
// Date		12.01.00
// Author	A. Pasetti/T. Brown
//
// -------------------------------------------------------------------

#ifndef BasicTypesH
#define BasicTypesH

// -------------------------------------------------------------------
// NB Types that are treated as properties should be either 
// Real, int or bool

typedef unsigned short	InstanceId;			// instance identifier
typedef unsigned char	ClassId;			// class identifier
typedef unsigned int	PropertyId;			// property identifier
typedef double			Real;				// generic type for real numbers
typedef unsigned char	EventType;			// identifier for events
typedef unsigned int	AocsTime;			// absolute time
typedef unsigned int	AocsTimeInterval;	// relative time
typedef unsigned int	AocsCycle;			// AOCS cycle counter
typedef int				ConfigurationId;	// configuration indicator
typedef unsigned char	ConfigurationRank;	// configuration rank
typedef InstanceId		TelecommandId;		// TC identifier
typedef ClassId			TelecommandTypeId;	// TC type identifier

// -------------------------------------------------------------------
// Manoeuvres have a type attribute that can potentially 
// be used to influence the scheduling of the manoeuvre.
// In the current implementation, the type of a manoeuvre
// coincides with its class.  Therefore, the type ManoeuvreType
// is identical to ClassId.
typedef ClassId  ManoeuvreType;

// -------------------------------------------------------------------
// Manoeuvres have a state indicating whether they are ready to be 
// started, currently in work,aborted, etc. 
enum ManoeuvreState 
{
    ready=0,	  // ready to be started
	inProcess=1,  // in the process of being carried out
	finished=2,   // completed "naturally" 
	aborted=3,    // aborted (finished but not by natural completion) 
	onHold=4,     // temporarily suspended
	beenReset=5   // similar to aborted
};

// -------------------------------------------------------------------

enum TelemetryFormat {shortTm=0, normalTm=1, longTm=2, debugTm=3};

enum AocsDataFormat {scalar=0, twoVector=1, threeVector=2, eulerAngle=3, 
					 twoEulerAngles=4, threeEulerAngles=5, quaternions=6,
					 nVector=7};
									
enum PropagationFlag {noPropagation=0, channelPropagation=1, totalPropagation=2};

// -------------------------------------------------------------------

// Type for pointer to UCB hook function 
typedef void (*UcbHookFunction)(int*,Real*,Real*,int*,Real*,Real*,int*,Real*,int*,Real*,int*);

// -------------------------------------------------------------------

// Generic unit instruction
struct UnitInstruction {
	unsigned short priority;
	unsigned short destAddress;
	unsigned short subAddress;
	unsigned short command;
	unsigned short* data;
	unsigned short nDataWords;
};

// -------------------------------------------------------------------

#endif