// TestCaseGenericSetUp.cpp
//
// Version	1.0
// Date		10.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseGenericSetUp.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../AocsEvent/FailureEventRepository.h"
#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../AocsEvent/SystemEventRepository.h"
#include "../AocsEvent/ChangeEventRepository.h"
#include "../AocsEvent/ModeEventRepository.h"
#include "../AocsEvent/RecoveryEventRepository.h"
#include "../AocsEvent/ReconfigurationEventRepository.h"
#include "../AocsEvent/ManoeuvreEventRepository.h"
#include "../AocsEvent/TelecommandEventRepository.h"
#include "../ModeManagement/ModeManager.h"
#include "../AocsData/AocsData.h"
#include "../OperatingSystemObjects/DummyAocsClock.h"
#include "../FailureRecovery/NullRecoveryAction.h"
#include "../FailureRecovery/RecoveryStrategy.h"
#include "../BasicObjects/AocsObject.h"
#include "../SystemManagement/SystemManager.h"
#include "../ReconfigurationManagement/ReconfigurerHelper.h"
#include "../ManoeuvreManagement/Manoeuvre.h"
#include "../ManoeuvreManagement/ManoeuvreManager.h"
#include "../TelecommandManagement/TelecommandManager.h"

// -------------------------------------------------------------------

bool TestCaseGenericSetUp::setUpTestCase() {

	// Reset all static initializations
	AocsObject::resetStaticConfiguration();
	DataItemWrite::setAocsClock(NULL);
	ModeManager::resetStaticConfiguration();
	RecoveryStrategy::resetStaticConfiguration();
	ReconfigurerHelper::resetStaticConfiguration();
	Manoeuvre::resetStaticConfiguration();
	ManoeuvreManager::resetStaticConfiguration();
	TelecommandManager::resetStaticConfiguration();

	// Initialize variables
	systemManager.initialize(100,10,&systemEvtRep);
	
	// Allocate memory to event repositories
	int size = 4;
	failureEvtRep.setRepositorySize(size);	
	configEvtRep.setRepositorySize(size);	
	systemEvtRep.setRepositorySize(size);	
	modeEvtRep.setRepositorySize(size);	
	changeEvtRep.setRepositorySize(size);	
	recoveryEvtRep.setRepositorySize(size);	
	reconfigEvtRep.setRepositorySize(size);	
	manoeuvreEvtRep.setRepositorySize(size);	
	tcEvtRep.setRepositorySize(size);	

	// Advance dummy AOCS clock by two cycles
	aocsClock.run(0);
	aocsClock.run(0);

	// Configure the static part of AocsObject 
	AocsObject::setAocsClock(&aocsClock);
	AocsObject::setFailureEventRepository(&failureEvtRep);		
	AocsObject::setConfigurationEventRepository(&configEvtRep);
	AocsObject::setDestructionRecoveryAction(&nullRecAct);
	AocsObject::setSystemManager(&systemManager);

	// Set the clock on DataItemWrite objects
	DataItemWrite::setAocsClock(&aocsClock);

	// Configure the static part of the ModeManager class
	ModeManager::setChangeEventRepository(&changeEvtRep);
	ModeManager::setModeEventRepository(&modeEvtRep);

	// Configure the static part of the RecoveryStrategy class
	RecoveryStrategy::setRecoveryEventRepository(&recoveryEvtRep);

	// Configure the state part of the reconfigurer helper
	ReconfigurerHelper::setChangeEventRepository(&changeEvtRep);
	ReconfigurerHelper::setReconfigurationEventRepository(&reconfigEvtRep);

    // Configure the static parts of class Manoeuvre and ManoeuvreManager
    Manoeuvre::setManoeuvreEventRepository(&manoeuvreEvtRep);
    Manoeuvre::setChangeEventRepository(&changeEvtRep);
    ManoeuvreManager::setManoeuvreEventRepository(&manoeuvreEvtRep);

	// Configure the static part of the TelecommandManager
	TelecommandManager::setTelecommandEventRepository(&tcEvtRep);

	// Retrieve the inital number of events 
	failureEvtN = failureEvtRep.getCounter();
	configEvtN = configEvtRep.getCounter();
	modeEvtN = modeEvtRep.getCounter();
	changeEvtN = changeEvtRep.getCounter();
	recoveryEvtN = recoveryEvtRep.getCounter();
	tcEvtN = tcEvtRep.getCounter();
	
	return INITIALIZATION_SUCCESS;
}