// TestCaseGenericSetUp.h
//
// Version	1.0
// Date		07.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------
//
// Abstract base class for a generic test cases. Its set up method
// performs the following generic initialization actions:
//	- initialization of the static part of AocsObject
//	- initializatioon of the event repositories. 
// This base class can be used as base for all but the simplest tests.
//
// -------------------------------------------------------------------

#ifndef TestCaseGenericSetUpH
#define TestCaseGenericSetUpH

#include "../GeneralUtilities/TestCase.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../AocsEvent/FailureEventRepository.h"
#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../AocsEvent/SystemEventRepository.h"
#include "../AocsEvent/ModeEventRepository.h"
#include "../AocsEvent/ChangeEventRepository.h"
#include "../AocsEvent/RecoveryEventRepository.h"
#include "../AocsEvent/ManoeuvreEventRepository.h"
#include "../AocsEvent/TelecommandEventRepository.h"
#include "../AocsEvent/ReconfigurationEventRepository.h"
#include "../OperatingSystemObjects/DummyAocsClock.h"
#include "../FailureRecovery/NullRecoveryAction.h"
#include "../SystemManagement/SystemManager.h"

// -------------------------------------------------------------------

class TestCaseGenericSetUp : public TestCase {

protected :

	SystemEventRepository systemEvtRep;
	FailureEventRepository failureEvtRep;
	ConfigurationEventRepository configEvtRep;
	ModeEventRepository modeEvtRep;
	ChangeEventRepository changeEvtRep;
	RecoveryEventRepository recoveryEvtRep;
	ReconfigurationEventRepository reconfigEvtRep;
	ManoeuvreEventRepository manoeuvreEvtRep;
	TelecommandEventRepository tcEvtRep;

	SystemManager systemManager;

	DummyAocsClock aocsClock;

	NullRecoveryAction nullRecAct;

	int failureEvtN;
	int configEvtN;
	int modeEvtN;
	int changeEvtN;
	int recoveryEvtN;
	int tcEvtN;
	
public :

	TestCaseGenericSetUp(int testId, char* testName):
	   TestCase(testId,testName)
	   { return; }

	virtual bool setUpTestCase();

	virtual void runTestCase()
		{ return; }

	virtual bool shutDownTestCase()
		{ return SHUTDOWN_SUCCESS;}
};

// -------------------------------------------------------------------

#endif