// TestCaseManoeuvre_3.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseManoeuvre_3.h"
#include "ManoeuvreManagementTestSuite.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../OperatingSystemObjects/DummyAocsClock.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../ObjectMonitoring/TestPropertyMonitor.h"
#include "../../ObjectMonitoring/SimpleChange.h"
#include "../../ManoeuvreManagement/ManoeuvreManager.h"
#include "../../ManoeuvreManagement/DummyManoeuvre.h"

// -------------------------------------------------------------------
TestCaseManoeuvre_3::TestCaseManoeuvre_3():
	TestCaseGenericSetUp(ID_MANOEUVREMANAGER*10+3,
									"TestCaseManoeuvre_3") {
		return;
}

// -------------------------------------------------------------------

void TestCaseManoeuvre_3::runTestCase() {

    // Create and configure a ManoeuvreManager
    ManoeuvreManager mnManager;

    // Initialize the ManoeuvreManager's manoeuvre list.
    // This should work.
    mnManager.initializeManoeuvreList(20);

    // Create some dummy manoeuvres

    // a manoeuvre that should start right away
    AocsTime dm1StartTime = 20;
    int dm1StepCount = 25;
    DummyManoeuvre dm1(dm1StartTime, dm1StepCount, "dm1", NULL);
    dm1.setMonitorListSize(3);
    mnManager.addManoeuvre(&dm1);

    // add a monitor to the first manoeuvre
    TestPropertyMonitor monDm1;
    SimpleChange simpleChange1((Real)dm1.getState());
    dm1.addMonitor(&monDm1,&simpleChange1);
//  mnManager.addManoeuvreMonitor(&dm1, &monDm1);

    // a second manoeuvre that should start "pretty soon"
    AocsTime dm2StartTime = 60;
    int dm2StepCount = 3;
    DummyManoeuvre dm2(dm2StartTime, dm2StepCount, "dm2", NULL);
    dm2.setMonitorListSize(0);
    mnManager.addManoeuvre(&dm2);

    // Attempt to add a monitor to the second manoeuvre and verifies that
    // the operation fails (its monitor list size is zero)
    int configEvtN=AocsObject::getConfigurationEventRepository()->getCounter();
    TestPropertyMonitor monDm2;
    SimpleChange simpleChange2((Real)dm2.getState());
    dm2.addMonitor(&monDm2,&simpleChange2);
    if (AocsObject::getConfigurationEventRepository()->getCounter()!=configEvtN+2)
	{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
		return;
	}

    // a third manoeuvre that should start at the same time as the last one
    AocsTime dm3StartTime = dm2StartTime;
    int dm3StepCount = 12;
    DummyManoeuvre dm3(dm3StartTime, dm3StepCount, "dm3", NULL);
    dm3.setMonitorListSize(0);
    mnManager.addManoeuvre(&dm3);

    // a fourth manoeuvre that should not start for a while
    AocsTime dm4StartTime = 120;
    int dm4StepCount = 4;
    DummyManoeuvre dm4(dm4StartTime, dm4StepCount, "dm4", NULL);
    dm4.setMonitorListSize(3);
    mnManager.addManoeuvre(&dm4);

    // add a monitor to the manoeuvre
    TestPropertyMonitor monDm4;
    SimpleChange simpleChange4((Real)dm4.getState());
    dm4.addMonitor(&monDm4,&simpleChange4);

    // a fifth manoeuvre that should not start for a long time
    // but should start and end before we finish the test
    AocsTime dm5StartTime = 730;
    int dm5StepCount = 17;
    DummyManoeuvre dm5(dm5StartTime, dm5StepCount, "dm5", NULL);
    dm5.setMonitorListSize(0);
    mnManager.addManoeuvre(&dm5);

    // a sixth manoeuvre that should not start for a long time
    // and should start before we finish the test, but not end
    AocsTime dm6StartTime = 750;
    int dm6StepCount = 70;
    DummyManoeuvre dm6(dm6StartTime, dm6StepCount, "dm6", NULL);
    dm6.setMonitorListSize(0);
    mnManager.addManoeuvre(&dm6);

    // move the clock forward and run the manoeuvre manager a "few" times
    DummyAocsClock* myClock = (DummyAocsClock*) AocsObject::getAocsClock();
    int clockPeriod = myClock->getClockPeriod();

    for (int runCount=0; runCount<100; runCount++)
    {
        AocsTime currentTime = myClock->getTime();
        mnManager.run(0);

        // check state of dm1
        if (!checkDummyManoeuvreState( &dm1, currentTime, dm1StartTime,
                                        dm1StepCount, clockPeriod))
		{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
			return;
		}

        // check state of dm2
        if (!checkDummyManoeuvreState( &dm2, currentTime, dm2StartTime,
                                        dm2StepCount, clockPeriod))
		{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
			return;
		}

        // check state of dm3
        if (!checkDummyManoeuvreState( &dm3, currentTime, dm3StartTime,
                                        dm3StepCount, clockPeriod))
		{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
			return;
		}

        // check state of dm4
        if (!checkDummyManoeuvreState( &dm4, currentTime, dm4StartTime,
                                        dm4StepCount, clockPeriod))
		{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
			return;
		}

        // check state of dm5
        if (!checkDummyManoeuvreState( &dm5, currentTime, dm5StartTime,
                                        dm5StepCount, clockPeriod))
		{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
			return;
		}

        // check state of dm6
        if (!checkDummyManoeuvreState( &dm6, currentTime, dm6StartTime,
                                        dm6StepCount, clockPeriod))
		{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
			return;
		}

        // Advance the time and keep on going
        myClock->run(0);
    }

    // Check the manoeuvre monitors were notified the expected number of time
    if (monDm1.getNotificationCounter()!=2)
	{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
		return;
	}
    if (monDm4.getNotificationCounter()!=2)
	{	setTestResult(TEST_FAILURE, "Manoeuvre test 3 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}


