// FollowerModeManager.h
//
// Version	1.0
// Date		04.04.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Class for the follower mode manager (the mode manager follows mode
// changes in a master mode manager. It is assumed that the follower
// and master mode managers have the same set of modes)
//
// -------------------------------------------------------------------

#ifndef FollowerModeManagerH
#define FollowerModeManagerH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../ModeManagement/ModeManager.h"
#include "../ObjectMonitoring/PropertyMonitor.h"
#include "../ObjectMonitoring/Property.h"
#include "../ObjectMonitoring/SimpleChange.h"

// -------------------------------------------------------------------

class FollowerModeManager : public ModeManager, PropertyMonitor {

	ModeManager* masterModeManager;
	
	// This is the change object that defines changes in the master
	// mode manager that trigger mode changes in the follower mode manager.
	// The choice of a SimpleChange object implies that any change in the
	// master will trigger a change in the follower.
	SimpleChange simpleChange;

public :

	FollowerModeManager(int numberOfModes, 
			 			 int numberOfStrategies, 
						  int maxNumberOfMonitors,
						    ModeManager* masterModeManager);

	// Call-back method triggered by a mode change in the master
	virtual void propertyChange(AocsObject* changeOriginator,
								  Property monitoredProperty,
									ChangeObject* changeObject);

};

// -------------------------------------------------------------------
#endif
