// TestCaseModeManagement_2.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseModeManagement_2.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ModeEvent.h"
#include "../../AocsEvent/ChangeEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../BasicObjects/RootObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../ModeManagement/CyclingModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"

// -------------------------------------------------------------------
TestCaseModeManagement_2::TestCaseModeManagement_2():
	TestCaseGenericSetUp(ID_MODEMANAGER*10+2,
									"TestCaseModeManagement_2") {
		return;
}

// -------------------------------------------------------------------

void TestCaseModeManagement_2::runTestCase() {

	// Create a cyclical mode manager
	int nModes=3;
	int nMonitors=4;
	int defaultMode=2;
	CyclingModeManager mm(nModes,defaultMode,nMonitors);

	// Load the mode change actions in the cycling mode manager
	NullModeChangeAction mca;
	mm.setModeChangeAction(&mca);
	
	// Set a new mode 
	mm.setMode(1);

	// Verify change and mode event counters
	if (modeEvtRep.getCounter()!=modeEvtN+1)
	{	setTestResult(TEST_FAILURE, "Wrong number of mode events");
		return;
	}
	if (changeEvtRep.getCounter()!=changeEvtN)
	{	setTestResult(TEST_FAILURE, "Wrong number of change events");
		return;
	}

	// Verify that the correct event was logged
	if (modeEvtRep.latestModeEvent()->getEventType()!=OPERATIONAL_MODE_CHANGE)
	{	setTestResult(TEST_FAILURE, "Wrong event type");
		return;
	}

	// Check that the mode was correctly set
	if (mm.getModeProperty().get()!=1)
	{	setTestResult(TEST_FAILURE, "Wrong mode manager configuration");
		return;
	}

	// Inhibit transitions to mode 2
	mm.inhibitMode(2);

	// Attempt to force a transition to mode 2
	mm.setMode(2);

	// Check that no mode change took place and no mode event was generated
	if (modeEvtRep.getCounter()!=modeEvtN+1)
	{	setTestResult(TEST_FAILURE, "Wrong number of mode events");
		return;
	}
	if (mm.getModeProperty().get()!=1)
	{	setTestResult(TEST_FAILURE, "Wrong mode manager configuration");
		return;
	}

	// Enable mode 2
	mm.enableMode(2);

	// Force a transition to mode 2
	mm.forceMode(2);

	// Check mode and mode change event
	if (modeEvtRep.getCounter()!=modeEvtN+2)
	{	setTestResult(TEST_FAILURE, "Wrong number of mode events");
		return;
	}
	if (mm.getModeProperty().get()!=2)
	{	setTestResult(TEST_FAILURE, "Wrong mode manager configuration");
		return;
	}

	// Attempt a new mode transition
	mm.setMode(0);

	// Check that mode transition did not take place
	if (modeEvtRep.getCounter()!=modeEvtN+2)
	{	setTestResult(TEST_FAILURE, "Wrong number of mode events");
		return;
	}
	if (mm.getModeProperty().get()!=2)
	{	setTestResult(TEST_FAILURE, "Wrong mode manager configuration");
		return;
	}

	// Release the mode and attempt transition again
	mm.releaseMode();
	mm.setMode(0);
	if (modeEvtRep.getCounter()!=modeEvtN+3)
	{	setTestResult(TEST_FAILURE, "Wrong number of mode events");
		return;
	}
	if (mm.getModeProperty().get()!=0)
	{	setTestResult(TEST_FAILURE, "Wrong mode manager configuration");
		return;
	}

	// Reset the mode manager and check that it goes back to the default mode
	mm.reset();
	if (modeEvtRep.getCounter()!=modeEvtN+4)
	{	setTestResult(TEST_FAILURE, "Wrong number of mode events");
		return;
	}
	if (mm.getModeProperty().get()!=defaultMode)
	{	setTestResult(TEST_FAILURE, "Wrong mode manager configuration");
		return;
	}

	// Check that no configuration or change events were generated
	if ( configEvtRep.getCounter() != configEvtN )
	{	setTestResult(TEST_FAILURE, "Configuration event was generated");
		return;
	}
	if ( changeEvtRep.getCounter() != changeEvtN )
	{	setTestResult(TEST_FAILURE, "Change event was generated");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
