// TestCaseModeManagement_3.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseModeManagement_3.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ModeEvent.h"
#include "../../AocsEvent/ChangeEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../BasicObjects/RootObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../ModeManagement/CyclingModeManager.h"
#include "../../ModeManagement/FollowerModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"

// -------------------------------------------------------------------
TestCaseModeManagement_3::TestCaseModeManagement_3():
	TestCaseGenericSetUp(ID_MODEMANAGER*10+3,
									"TestCaseModeManagement_3") {
		return;
}

// -------------------------------------------------------------------

void TestCaseModeManagement_3::runTestCase() {

	// Create a cycling mode manager to act as mode master
	int nModes=3;
	int nMonitors=4;
	int defaultMode=2;
	CyclingModeManager mm(nModes,defaultMode,nMonitors);

	// Load the mode change actions in the cycling mode manager
	NullModeChangeAction mca;
	mm.setModeChangeAction(&mca);

	// Create dummy objects to serve as dummy implementers
	RootObject dummy00,dummy10,dummy20,dummy01,dummy11, dummy21;

	// Load implementers in the cycling mode manager
	mm.loadImplementer(0,0,&dummy00);
	mm.loadImplementer(1,0,&dummy10);
	mm.loadImplementer(2,0,&dummy20);

	// Create two follower mode managers and link them to the cycling mode manager
	FollowerModeManager fm1(nModes, 2, nMonitors, &mm);
	FollowerModeManager fm2(nModes, 2, nMonitors, &mm);

	// Load the mode change actions in the follower mode manager
	fm1.setModeChangeAction(&mca);
	fm2.setModeChangeAction(&mca);

	// Load implementers in the first follower mode managers
	fm1.loadImplementer(0,0,&dummy00);
	fm1.loadImplementer(1,0,&dummy10);
	fm1.loadImplementer(2,0,&dummy20);
	fm1.loadImplementer(0,1,&dummy01);
	fm1.loadImplementer(1,1,&dummy11);
	fm1.loadImplementer(2,1,&dummy21);

	// Check tha the first follower mode manager is configured
	if (!fm1.isConfigured())
	{	setTestResult(TEST_FAILURE, "Mode is not configured");
		return;
	}
	
	// Check that all three mode managers are in the same mode
	if (mm.getModeProperty().get()!=defaultMode)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}
	if (fm1.getModeProperty().get()!=defaultMode)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}
	if (fm2.getModeProperty().get()!=defaultMode)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}

	// Get the implementer from the cycling mode manager
	if (mm.getImplementer(0) != &dummy20)
	{	setTestResult(TEST_FAILURE, "Wrong implementer");
		return;
	}
	
	// The mode of the cycling mode manager should have been updated
	if (mm.getModeProperty().get()!=0)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}

	// Check that the followers have followed the master
	if (fm1.getModeProperty().get()!=0)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}
	if (fm2.getModeProperty().get()!=0)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}

	// Get another implemeter from the cycling mode manager
	if (mm.getImplementer(0) != &dummy00)
	{	setTestResult(TEST_FAILURE, "Wrong implementer");
		return;
	}
	
	// The mode of the cycling mode manager should have been updated again
	if (mm.getModeProperty().get()!=1)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}

	// Check that the followers have followed the master
	if (fm1.getModeProperty().get()!=1)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}
	if (fm2.getModeProperty().get()!=1)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}
	
	// Check that the correct events were generated
	if ( modeEvtRep.getCounter() != modeEvtN+6 )
	{	setTestResult(TEST_FAILURE, "Wrong number of mode events");
		return;
	}
	if ( changeEvtRep.getCounter() != changeEvtN+4 )
	{	setTestResult(TEST_FAILURE, "Wrong number of change events");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
