// TestCaseModeManagement_4.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseModeManagement_4.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ModeEvent.h"
#include "../../AocsEvent/ChangeEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../BasicObjects/RootObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../ModeManagement/FollowerModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"
#include "../../ModeManagement/AocsMissionModeManager.h"

// -------------------------------------------------------------------
TestCaseModeManagement_4::TestCaseModeManagement_4():
	TestCaseGenericSetUp(ID_MODEMANAGER*10+4,
									"TestCaseModeManagement_4") {
		return;
}

// -------------------------------------------------------------------

void TestCaseModeManagement_4::runTestCase() {

	// Create an AOCS mission mode manager 
	int nModes=3;
	int nMonitors=4;
	int defaultMode=2;
	AocsMissionModeManager ammm(nModes,defaultMode,nMonitors);

	// Load the mode change actions in the mission mode manager
	NullModeChangeAction mca;
	ammm.setModeChangeAction(&mca);

	// Create two follower mode managers and link them to the 
	// AOCS mission mode manager
	FollowerModeManager fm1(nModes, 1, nMonitors, &ammm);
	FollowerModeManager fm2(nModes, 1, nMonitors, &ammm);

	// Load the mode change actions in the follower mode manager
	fm1.setModeChangeAction(&mca);
	fm2.setModeChangeAction(&mca);

	// Check that all three mode managers are in the same mode
	if (ammm.getModeProperty().get()!=defaultMode)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}
	if (fm1.getModeProperty().get()!=defaultMode)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}
	if (fm2.getModeProperty().get()!=defaultMode)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}

	// Change the mode of the mission mode manager
	ammm.setMode(1);

	// Check that the followers have followed the master
	if (ammm.getModeProperty().get()!=1)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}
	if (fm1.getModeProperty().get()!=1)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}
	if (fm2.getModeProperty().get()!=1)
	{	setTestResult(TEST_FAILURE, "Wrong mode");
		return;
	}

	// Check that the correct events were generated
	if ( modeEvtRep.getCounter() != modeEvtN+3 )
	{	setTestResult(TEST_FAILURE, "Wrong number of mode events");
		return;
	}
	if ( changeEvtRep.getCounter() != changeEvtN+2 )
	{	setTestResult(TEST_FAILURE, "Wrong number of change events");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
