// TestCaseObjectList_2.cpp
//
// Version	1.0
// Date		07.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseObjectList_2.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../BasicObjects/AocsObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/ConfigurationEvent.h"

// -------------------------------------------------------------------
TestCaseObjectList_2::TestCaseObjectList_2():
	TestCaseObjectList(ID_OBJECTLIST*10+2,"TestCaseObjectList_2") {
		return;
}

// -------------------------------------------------------------------

void TestCaseObjectList_2::runTestCase() {

	// Retrieve the number of failure and configuration events
	int failureEvtN = AocsObject::getFailureEventRepository()->getCounter();
	int configEvtN = AocsObject::getConfigurationEventRepository()->getCounter();

	// Reset the configuration of the list (thus clearing all its entries)
	objectList.reset();
	objectList.resetConfiguration();

	// Check that there are no entries in the object list
	if (objectList.getNumberOfItems() != 0)
	{	setTestResult(TEST_FAILURE, "Incorrect object list initialization");
		return;
	}

	// Create dummy objects to be added to the list
	AocsObject dummy1, dummy2, dummy3;

	// Perform some add/remove operations in sequence
	objectList.addObject(&dummy1);
	objectList.addObject(&dummy2);
	objectList.addObject(&dummy3);
	objectList.removeObject(&dummy2);

	// Check correct number of items in list
	if (objectList.getNumberOfItems() != 2)
	{	setTestResult(TEST_FAILURE, "Incorrect number of items in object list");
		return;
	}

	// Attempt to remove a non-existent item from the list
	objectList.removeObject(&dummy2);

	// Check that a configuration event was generated
	if (AocsObject::getConfigurationEventRepository()->getCounter()!=configEvtN+1)
	{	setTestResult(TEST_FAILURE, "Incorrect event generation");
		return;
	}
	if (AocsObject::getConfigurationEventRepository()->latestConfigurationEvent()
					->getEventType() != OBJECT_NOT_IN_LIST )
	{	setTestResult(TEST_FAILURE, "Incorrect event generation");
		return;
	}

	// Attempt to add too many objects to the list
	objectList.addObject(&dummy1);
	objectList.addObject(&dummy1);
	objectList.addObject(&dummy1);

	// Check that a configuration event was generated
	if (AocsObject::getConfigurationEventRepository()->getCounter()!=configEvtN+2)
	{	setTestResult(TEST_FAILURE, "Incorrect event generation");
		return;
	}
	if ( AocsObject::getConfigurationEventRepository()->latestConfigurationEvent()
					->getEventType() != LIST_IS_FULL )
	{	setTestResult(TEST_FAILURE, "Incorrect event generation");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
