// DeltaChange.cpp
//
// Version	1.0
// Date		29.03.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../ObjectMonitoring/DeltaChange.h"
#include "../ObjectMonitoring/ChangeObject.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"

// -------------------------------------------------------------------
// Define static variables
int DeltaChange::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

DeltaChange::DeltaChange(Real r)
{
	setClassId(ID_DELTACHANGE);			// assign class identifier

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = sizeof(Real);
	tmImageLength[longTm]   = tmImageLength[normalTm]+sizeof(bool) ;
	tmImageLength[debugTm]  = tmImageLength[longTm] ;
	
	referenceValue = r;
	localReset();

}

// -------------------------------------------------------------------

bool DeltaChange::checkValue(Real value)
{
	if (firstCheck)
	{	firstCheck = false;
		oldValue = value;
		return false;
	}
		
	if ( ((value-oldValue)>referenceValue) ||
		 ((oldValue-value)>referenceValue) )
	{
		oldValue = value;
		return true;
	}

	oldValue = value;
	return false;
}

// -------------------------------------------------------------------
// From Resettable interface

void DeltaChange::reset()
{
	ChangeObject::reset();
	localReset();
}

// -------------------------------------------------------------------
// From Resettable interface

void DeltaChange::localReset()
{
	firstCheck = true;
}

// -------------------------------------------------------------------
// From Telemeterable interface

int DeltaChange::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void DeltaChange::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}
