// DummyAocsClock.cpp
//
// Version	1.0
// Date		18.01.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../BasicObjects/Configurable.h"
#include "../TelemetryManagement/Telemeterable.h"
#include "../BasicObjects/Resettable.h"
#include "../BasicObjects/RootObject.h"
#include "../BasicObjects/AocsObject.h"
#include "../OperatingSystemObjects/Runnable.h"
#include "../OperatingSystemObjects/AocsClock.h"
#include "../OperatingSystemObjects/DummyAocsClock.h"

// -------------------------------------------------------------------
// Define static variables
int DummyAocsClock::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
// Default constructor.

DummyAocsClock::DummyAocsClock()
{
	setClassId(ID_DUMMYAOCSCLOCK);		// assign class identifier

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = sizeof(AocsTime)+sizeof(int);
	tmImageLength[longTm]   = sizeof(AocsTime)+sizeof(int);
	tmImageLength[debugTm]  = sizeof(AocsTime)+sizeof(int);

	localReset();							// reset object's state
	
}

// -------------------------------------------------------------------

AocsTime DummyAocsClock::getTime()
{	
	return currentTime; 
}

// -------------------------------------------------------------------

AocsCycle DummyAocsClock::getCycle()
{	
	return currentCycle; 
}

// -------------------------------------------------------------------

void DummyAocsClock::synchronizeWithSystemTime()
{
	return;			// hardware-dependent
}

// -------------------------------------------------------------------
// From Runnable interface

void DummyAocsClock::run(AocsTime deadline)
{
	currentCycle++;
	currentTime=currentTime+dummyPeriod;
}

// -------------------------------------------------------------------
// Reset local state

void DummyAocsClock::localReset()
{
	currentTime = 0;
	currentCycle = 0;
}

// -------------------------------------------------------------------
// From Resettable interface

void DummyAocsClock::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------
// From Telemeterable interface

int DummyAocsClock::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void DummyAocsClock::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : in normal TM mode, write the current time and cycle to TM
}