// RtemsAocsClock.cpp
//
// Version	1.0
// Date		15.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../GeneralInclude/Include.h"
#include "../BasicObjects/Configurable.h"
#include "../TelemetryManagement/Telemeterable.h"
#include "../BasicObjects/Include.h"
#include "../OperatingSystemObjects/AocsClock.h"
#include "../OperatingSystemObjects/RtemsAocsClock.h"

// -------------------------------------------------------------------
// Define static variables
int RtemsAocsClock::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
// In order to allow testing under VS: define a dummy RTEMS timing routine

#ifdef MS_HOST
	void rtems_clock_get(int ignored,AocsTime* ticks)
	{	static int i=0;
		*ticks = i*1000; 
		i++;
	}
#endif

// -------------------------------------------------------------------

RtemsAocsClock::RtemsAocsClock(unsigned int tpc)
{
	setClassId(ID_RTEMSAOCSCLOCK);		

	tmImageLength[shortTm]  = 0;	// Not implemented in fw prototype	
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	ticksPerCycle = tpc;

	localReset();							
}

// -------------------------------------------------------------------

void RtemsAocsClock::localReset()
{
	cyclesAtLastReset = getTime()/ticksPerCycle;	
}

// -------------------------------------------------------------------

void RtemsAocsClock::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

void RtemsAocsClock::localResetConfiguration()
{
	ticksPerCycle = 0;
}

// -------------------------------------------------------------------

void RtemsAocsClock::resetConfiguration()
{
	AocsObject::reset();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool RtemsAocsClock::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (ticksPerCycle == 0)
		return false;

	return true;
}

// -------------------------------------------------------------------
// Return time as number of RTEMS ticks since last reset.
// NB There should be a check on the return value of rtems_clock_get

AocsTime RtemsAocsClock::getTime()
{
	rtems_interval ticksSinceBoot;
	rtems_clock_get(RTEMS_CLOCK_GET_TICKS_SINCE_BOOT,&ticksSinceBoot);
	return ticksSinceBoot; 
}

// -------------------------------------------------------------------

AocsCycle RtemsAocsClock::getCycle()
{	
	return (getTime()/ticksPerCycle - cyclesAtLastReset);
}

// -------------------------------------------------------------------

void RtemsAocsClock::synchronizeWithSystemTime()
{
	return;			
}

// -------------------------------------------------------------------

int RtemsAocsClock::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void RtemsAocsClock::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		

	// Not implemented in fw prototype
}