// Reconfigurable.h
//
// Version	1.0
// Date		26.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// This is an interface class.
// -------------------------------------------------------------------

#ifndef ReconfigurableH
#define ReconfigurableH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../ObjectMonitoring/Property.h"
#include "../ObjectMonitoring/PropertyMonitor.h"
#include "../ReconfigurationManagement/ConfigurationState.h"

// -------------------------------------------------------------------

class Reconfigurable {

public:

	// Return current configuration
	virtual ConfigurationId getConfiguration()=0;

	// Configuration property management
	virtual PropertyId getConfigurationPropertyId()=0;
	virtual Property getConfigurationProperty()=0;

	// Force the current configuration to n 
	virtual void setConfiguration(ConfigurationId n)=0;

	// Mark configuration n healthy (true) or unhealthy (false)
	virtual void markConfiguration(ConfigurationId n, bool mark)=0;

	// Set the rank of configuration n to r
	virtual void rankConfiguration(ConfigurationId n, ConfigurationRank r)=0;

	// Enable and disable a particular configuration
	virtual void enableConfiguration(ConfigurationId n)=0;
	virtual void disableConfiguration(ConfigurationId n)=0;

	// Enable and disable all reconfigurations
	virtual void enableReconfiguration()=0;
	virtual void disableReconfiguration()=0;

	// Command a reconfiguration
	virtual void reconfigure()=0;

	// Return true if a reconfiguration is possible
	virtual bool canReconfigure()=0;

	// Return true if configuration n is healthy
	virtual bool isConfigurationHealthy(ConfigurationId n)=0;

	// Return the reconfiguration order
	virtual int getReconfigurationOrder()=0;

	// Return the rank of configuration n
	virtual ConfigurationRank getConfigurationRank(ConfigurationId n)=0;

	// Setter and getter methods for the reconfiguration state
	virtual void setConfigurationState(ConfigurationState s)=0;
	virtual ConfigurationState getConfigurationState()=0;

	// Add and remove a configuration monitor
	virtual void addMonitor(PropertyMonitor* monitor, ChangeObject* changeObject)=0;
	virtual void removeMonitor(PropertyMonitor* monitor)=0;

	// Setter and getter method for recovery action associated to 
	// failed reconfiguration request
	virtual void setReconfigurationErrorRecoveryAction(RecoveryAction* r)=0;
	virtual RecoveryAction* getReconfigurationErrorRecoveryAction()=0;

};

// -------------------------------------------------------------------
#endif