// AbstractControlChannel.cpp
//
// Version	1.0
// Date		15.03.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include required for use of sqrt function
#include <stdio.h>

// Include framework header files
#include "../SequentialDataProcessing/AbstractControlChannel.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"

// -------------------------------------------------------------------
// Default constructor.

AbstractControlChannel::AbstractControlChannel()				
{
	setClassId(ID_ABSTRACTCONTROLCHANNELBLOCK);			// assign class identifier

	n = 0;
	m = 0; 
	propagationToken=0;

	localReset();
	localResetConfiguration();
}

// -------------------------------------------------------------------

void AbstractControlChannel::localReset()
{
	lastPropagatedTime = getTime();
}	

// -------------------------------------------------------------------

void AbstractControlChannel::reset()
{
	AocsObject::reset();	// reset super object state
	localReset();
}

// -------------------------------------------------------------------

void AbstractControlChannel::reset(PropagationFlag flag, int token)
{
	reset();	// reset this object

	if (flag == noPropagation)
		return;

	// Go through the list of source channels embedded in the same control
	// channel and, if they have not yet been reset, reset them
	if (flag == channelPropagation)
	{	for (int i=0; i<getInputOrder(); i++)
			if ( ccLink[i] != NULL )
				if ( ( ccLink[i]->getPropagationToken() != token ) &&
				     ( ccLink[i]->getEnclosingControlChannel() == getEnclosingControlChannel()) )
				{	ccLink[i]->setPropagationToken(token);
					ccLink[i]->reset(flag,token);
				}
		return;
	}
	
	// Go through the list of source channels and, if they have not 
	// yet been reset, reset them
	if (flag == totalPropagation)
	{	for (int i=0; i<getInputOrder(); i++)
			if ( ccLink[i] != NULL )
				if ( ccLink[i]->getPropagationToken() != token )
				{	ccLink[i]->setPropagationToken(token);
					ccLink[i]->reset(flag,token);
				}
		return;
	}
}
		
// -------------------------------------------------------------------
// The ccLink array is reset in the localResetConfiguration methods of
// the controlChannelBlock and controlChannelSuperBlock subclasses. It
// cannot be reset here because its size is still unknown (it depends on
// number of inputs).

void AbstractControlChannel::localResetConfiguration()
{
	enclosingControlChannel=NULL;
	recoveryAction = NULL;
}	

// -------------------------------------------------------------------

void AbstractControlChannel::resetConfiguration()
{
	AocsObject::resetConfiguration();	// reset super object state
	localResetConfiguration();
}
		
// -------------------------------------------------------------------

bool AbstractControlChannel::isConfigured(PropagationFlag flag, int token)
{
	if (!isConfigured())	// check the configuration of this object
		return false;

	if (flag == noPropagation)
		return true;

	// Go through the list of source channels embedded in the same control
	// channel and, if any of them is not configured, return false
	if (flag == channelPropagation)
	{	for (int i=0; i<getInputOrder(); i++)
			if ( ccLink[i] != NULL )
				if ( ( ccLink[i]->getPropagationToken() != token ) &&
				     ( ccLink[i]->getEnclosingControlChannel() == getEnclosingControlChannel()) )
				{	ccLink[i]->setPropagationToken(token);
					if ( !ccLink[i]->isConfigured(flag,token) )
						return false;
				}
		return true;
	}
	
	// Go through the list of source channels and, if their configuration 
	// has not yet been reset, call resetConfiguration upon them
	{	for (int i=0; i<getInputOrder(); i++)
			if ( ccLink[i] != NULL )
				if ( ccLink[i]->getPropagationToken() != token )
				{	ccLink[i]->setPropagationToken(token);
					if ( !ccLink[i]->isConfigured(flag,token) )
						return false;
				}
		return true;
	}
}