// ControlChannelBlock.h
//
// Version	1.0
// Date		16.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Abstract base class for all control channel blocks
//
// -------------------------------------------------------------------

#ifndef ControlChannelBlockH
#define ControlChannelBlockH

// -------------------------------------------------------------------

#include "../SequentialDataProcessing/AbstractControlChannel.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../AocsData/DataItemRead.h"

// -------------------------------------------------------------------

class ControlChannelBlock : public AbstractControlChannel {

	int p;								// number of states

	bool onHold;						// true if block is currently on hold
	bool initFlag;						// set to true when the block is reset

	DataItemRead* inputLink;			// array of links for input from external signals
	Real* outputBuffer;					// array holding last set of computed outputs

	AocsTime validityTime;				// validity time for output buffer

	void updateInputs(AocsTime t);		// load inputs into working buffer prior to propagation
	void updateOutputBuffer(AocsTime t);// load output buffer after propagation

	virtual void propagationAlgorithm(AocsTime t)=0;

	void setStateOrder(int i)
		{ p = i; }

protected :

	Real* input;								// working input buffer
	Real* output;								// working output buffer
	Real* state;								// state buffer

	void allocateMemory(int n, int m, int p);	// allocate memory for input, output 
												// and state buffers. This method 
												// can only be called once!
	void localReset();
	void localResetConfiguration();

public :

	ControlChannelBlock();

	virtual void propagate(AocsTime t);

	// link DataItemRead d to j-th input
	virtual void linkInput(DataItemRead d, int j);

	// link j-th input to i-th output of control channel c
	virtual void linkInput(AbstractControlChannel* c, int i, int j);

	virtual DataItemRead getOutput(int j);

	virtual void reset();				// pure virtual method from Resettable

	virtual void resetConfiguration();	// pure virtual methods from Configurable
	virtual bool isConfigured();

	// straightforward getter and setter methods
	int getStateOrder()
		{ return p; }

	AocsTime getValidityTime()
		{ return validityTime; }

	virtual void hold()
		{ onHold = true; }

	virtual void release()
		{ onHold = false; }

	bool isOnHold()
		{ return onHold; }

	Real getState(int i)				// get i-th component of state vector
		{ return state[i]; }

	void setState(int i, Real value)	// set i-th component of state vector
		{ state[i] = value; }

};

// -------------------------------------------------------------------
#endif
