// D_Block.h
//
// Version	1.0
// Date		18.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Control block to implement an derivative action with a fixed gain.
//
// The block implements the following single-input-single-output transfer
// function: y(k+1) = gain*( u(k+1)-u(k) )/DT
// Additionally, there is a rough filtering action on the derivative value
// in the sense that the absolute value of y(k+1) is constrained to lie
// between -maximumDerivative and +maximumDerivative
//
// -------------------------------------------------------------------

#ifndef D_BlockH
#define D_BlockH

// -------------------------------------------------------------------

#include "../SequentialDataProcessing/ControlChannelBlock.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"

// -------------------------------------------------------------------

class D_Block : public ControlChannelBlock {

	static int tmImageLength[NTMFORMATS];

	Real gain;
	Real maximumDerivative;

	virtual void propagationAlgorithm(AocsTime t);

	void localResetConfiguration();

public :

	D_Block();

	void setGain(Real newGain)
		{ gain = newGain; }

	void setMaximumDerivative(Real newMaximumDerivative)
		{ maximumDerivative = newMaximumDerivative; }

	virtual void resetConfiguration();

	virtual int getTelemetryImageLength();
	virtual void writeToTelemetry(TelemetryStream* stream);
};

#endif
