// I_Block.cpp
//
// Version	1.0
// Date		18.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../SequentialDataProcessing/I_Block.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"

// -------------------------------------------------------------------
// Define static variables
int I_Block::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
// Default constructor.

I_Block::I_Block()				
{
	setClassId(ID_I_BLOCK);					

	allocateMemory(1,1,2);				// 1 input, 1 output, 2 states

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = 0;	
	tmImageLength[longTm]   = getStateOrder()*sizeof(Real)+sizeof(AocsTime);
	tmImageLength[debugTm]  = tmImageLength[longTm]+getInputOrder()*sizeof(Real)+
									getOutputOrder()*sizeof(Real);

	localResetConfiguration();					
}

// -------------------------------------------------------------------
// Implemented transfer function: y(k+1) = y(k)+[u(k)+u(k+1)]*DT/2
// At time k+1: X[0] stores y(k)
//				X[1] stores u(k)

void I_Block::propagationAlgorithm(AocsTime t)
{
	output[0] = state[0]+gain*(state[1]+input[0])*(t-lastPropagatedTime)/2;
	state[0] = output[0];
	state[1] = input[0];
}

// -------------------------------------------------------------------

void I_Block::localResetConfiguration()
{
	gain = 0.0;
}

// -------------------------------------------------------------------

void I_Block::resetConfiguration()
{
	ControlChannelBlock::resetConfiguration();		// reset super object's state
	localResetConfiguration();
}

// -------------------------------------------------------------------

void I_Block::writeToTelemetry(TelemetryStream* stream)
{
	ControlChannelBlock::writeToTelemetry(stream);		// write super object's state

	switch(getTelemetryFormat()) 
	{
		case shortTm:
			break;
		case normalTm:
			// write gain 
			break;
		case longTm:
			// write gain and state arrays  
			break;
		case debugTm:
			// write gain and input, output and state arrays 
			break;
	}
}

// -------------------------------------------------------------------

int I_Block::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]+
				ControlChannelBlock::getTelemetryImageLength());
}


