// I_Block.h
//
// Version	1.0
// Date		18.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Control block to implement an integrator with a fixed gain.
//
// The block implements the following single-input-single-output transfer
// function: y(k+1) = y(k)+gain*[u(k)+u(k+1)]*DT/2
//
// -------------------------------------------------------------------

#ifndef I_BlockH
#define I_BlockH

// -------------------------------------------------------------------

#include "../SequentialDataProcessing/ControlChannelBlock.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"

// -------------------------------------------------------------------

class I_Block : public ControlChannelBlock {

	static int tmImageLength[NTMFORMATS];

	Real gain;

	virtual void propagationAlgorithm(AocsTime t);

	void localResetConfiguration();

public :

	I_Block();

	void setGain(Real newGain)
		{ gain = newGain; }

	virtual void resetConfiguration();

	virtual int getTelemetryImageLength();
	virtual void writeToTelemetry(TelemetryStream* stream);
};

#endif
