// LimitBlock.h
//
// Version	1.0
// Date		13.10.00
// Author	T. Brown
//
// Change History:
//
// -------------------------------------------------------------------
// Control block to implement a simple limited output.
//
// The block implements the following multi-input-single-output transfer
// function: if (u > max), y = max
// 			 else if (u < min), y = min
//           else y = u
//
// -------------------------------------------------------------------

#ifndef LimitBlockH
#define LimitBlockH

// -------------------------------------------------------------------

#include "../SequentialDataProcessing/ControlChannelBlock.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"

// -------------------------------------------------------------------

class LimitBlock : public ControlChannelBlock {

	static ClassId classId;
	static int tmImageLength[NTMFORMATS];

	virtual void propagationAlgorithm(AocsTime t);

	void localResetConfiguration();

	Real highLimit;
	Real lowLimit;

public :

	LimitBlock();

	virtual void setLimits(Real high, Real low);

	virtual void resetConfiguration();

	virtual int getTelemetryImageLength();
	virtual void writeToTelemetry(TelemetryStream* stream);
};

#endif // LimitBlockH
