// P_Block.cpp
//
// Version	1.0
// Date		18.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../SequentialDataProcessing/P_Block.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"

// -------------------------------------------------------------------
// Define static variables
int P_Block::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
// Default constructor.

P_Block::P_Block()				
{
	setClassId(ID_P_BLOCK);					

	allocateMemory(1,1,0);				// 1 input, 1 output, 0 states

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = sizeof(Real);	
	tmImageLength[longTm]   = tmImageLength[normalTm]+getStateOrder()*sizeof(Real)+ 
									sizeof(AocsTime);
	tmImageLength[debugTm]  = tmImageLength[longTm]+getInputOrder()*sizeof(Real)+
									getOutputOrder()*sizeof(Real);

	localResetConfiguration();					
}

// -------------------------------------------------------------------

void P_Block::propagationAlgorithm(AocsTime t)
{
	output[0] = gain*input[0];
}

// -------------------------------------------------------------------

void P_Block::localResetConfiguration()
{
	gain = 0.0;
}

// -------------------------------------------------------------------

void P_Block::resetConfiguration()
{
	ControlChannelBlock::resetConfiguration();	// reset super object's state
	localResetConfiguration();
}

// -------------------------------------------------------------------

void P_Block::writeToTelemetry(TelemetryStream* stream)
{
	ControlChannelBlock::writeToTelemetry(stream);		// write super object's state

	switch(getTelemetryFormat()) 
	{
		case shortTm:
			break;
		case normalTm:
			// write gain 
			break;
		case longTm:
			// write gain and state arrays  
			break;
		case debugTm:
			// write gain and input, output and state arrays 
			break;
	}
}

// -------------------------------------------------------------------

int P_Block::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]+
				ControlChannelBlock::getTelemetryImageLength());
}


