// TestCaseControlChannel_1.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseControlChannel_1.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../SequentialDataProcessing/P_Block.h"

// -------------------------------------------------------------------
TestCaseControlChannel_1::TestCaseControlChannel_1():
	TestCaseGenericSetUp(ID_ABSTRACTCONTROLCHANNELBLOCK*10+1,
									"TestControlChannel_1") {
		return;
}

// -------------------------------------------------------------------

void TestCaseControlChannel_1::runTestCase() {

	P_Block block; 
	Scalar scalar;

	// Reset the block
	block.reset();
	
	// Retrieve data item write serving as source for control block
	DataItemWrite inputSource=scalar.getDataItemWrite(0);
	
	// Configure control block
	block.linkInput(scalar.getDataItemRead(0),0);
	block.setGain(2);
	
	// Check initial control block configuration
	if (block.getInputOrder() != 1)
	{	setTestResult(TEST_FAILURE, "Incorrect block configuration");
		return;
	}

	if (block.getOutputOrder() != 1)
	{	setTestResult(TEST_FAILURE, "Incorrect block configuration");
		return;
	}

	if (block.getStateOrder() != 0)
	{	setTestResult(TEST_FAILURE, "Incorrect block configuration");
		return;
	}

	if ( (block.getOutput(0).get()) != 0 )			// initial output should be zero
	{	setTestResult(TEST_FAILURE, "Incorrect block configuration");
		return;
	}

	if ( block.isOnHold() )			
	{	setTestResult(TEST_FAILURE, "Incorrect block configuration");
		return;
	}

	if ( (block.getValidityTime() ) != aocsClock.getTime() )			
	{	setTestResult(TEST_FAILURE, "Incorrect block configuration");
		return;
	}

	if ( (block.getLastPropagatedTime() ) != aocsClock.getTime() )			
	{	setTestResult(TEST_FAILURE, "Incorrect block configuration");
		return;
	}

	// Set the control block input to 10
	inputSource.set(10);

	// Propagate control block input across two time intervals
	AocsTime targetTime = aocsClock.getTime()+((AocsTime)100);
	block.propagate( targetTime );

	// Check correct value of block output
	if ( block.getOutput(0).get() != 20 )
	{	setTestResult(TEST_FAILURE, "Incorrect block output");
		return;
	}

	// Check correct updating of block time tags
	if ( (block.getValidityTime() ) != targetTime )			
	{	setTestResult(TEST_FAILURE, "Incorrect block time tag");
		return;
	}

	if ( (block.getLastPropagatedTime() ) != targetTime )			
	{	setTestResult(TEST_FAILURE, "Incorrect block time tag");
		return;
	}

	// Put the block on hold and propagate its output again
	block.hold();
	AocsTime newTargetTime = targetTime + 100;
	block.propagate( newTargetTime );

	// Check that output was not updated
	if ( block.getOutput(0).get() != 20 )
	{	setTestResult(TEST_FAILURE, "Incorrect block output");
		return;
	}

	// Check correct updating of block time tags
	if ( (block.getValidityTime() ) != targetTime )			
	{	setTestResult(TEST_FAILURE, "Incorrect block time tag");
		return;
	}

	if ( (block.getLastPropagatedTime() ) != newTargetTime )			
	{	setTestResult(TEST_FAILURE, "Incorrect block time tag");
		return;
	}

	// Check that block is in hold mode
	if ( !block.isOnHold() )
	{	setTestResult(TEST_FAILURE, "Block should be on hold");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
