// TestCaseControlChannel_2.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseControlChannel_2.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../SequentialDataProcessing/I_Block.h"

// -------------------------------------------------------------------
TestCaseControlChannel_2::TestCaseControlChannel_2():
	TestCaseGenericSetUp(ID_ABSTRACTCONTROLCHANNELBLOCK*10+2,
									"TestControlChannel_2") {
		return;
}

// -------------------------------------------------------------------

void TestCaseControlChannel_2::runTestCase() {

	I_Block block; 
	Scalar scalar;

	// Reset the block
	block.reset();
	
	// Define input signal intensity
	Real s=5;

	// Define integrator gain
	Real gain=3;
	block.setGain(gain);

	// Retrieve data item write serving as source for control block
	DataItemWrite inputSource=scalar.getDataItemWrite(0);
	
	// Configure control block
	block.linkInput(scalar.getDataItemRead(0),0);

	// Check default initial integrator conditions
	if ( block.getState(0) != 0 )			// default initial output should be zero
	{	setTestResult(TEST_FAILURE, "Initial conditions are wrong");
		return;
	}

	if ( block.getState(1) != 0 )			// default initial output should be zero
	{	setTestResult(TEST_FAILURE, "Initial conditions are wrong");
		return;
	}

	// Set initial integrator value
	Real initialValue=100;
	block.setState(0,initialValue);
	block.setState(1,s);

	// Check initial integrator conditions
	if ( block.getState(0) != initialValue )	// initial output should be zero
	{	setTestResult(TEST_FAILURE, "Initial conditions are wrong");
		return;
	}

	if ( block.getState(1) != s )			// initial output should be zero
	{	setTestResult(TEST_FAILURE, "Initial conditions are wrong");
		return;
	}

	// Set the control block input 
	inputSource.set(s);

	// Integrate over n time intervals
	int n=10;
	AocsTime timeStep=4;
	AocsTime t0=aocsClock.getTime();
	AocsTime t=t0;
	for (int i=0; i<n; i++)
	{	t+=timeStep;
		block.propagate(t);
		Real integral=block.getOutput(0).get();	// facilitates debugging
	}
	
	// Check correct value of block output
	if ( block.getOutput(0).get() != (initialValue+gain*(timeStep*n*s)) )
	{	setTestResult(TEST_FAILURE, "Block output is wrong");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
