// TestCaseControlChannel_3.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseControlChannel_3.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../SequentialDataProcessing/I_Block.h"

// -------------------------------------------------------------------
TestCaseControlChannel_3::TestCaseControlChannel_3():
	TestCaseGenericSetUp(ID_ABSTRACTCONTROLCHANNELBLOCK*10+3,
									"TestControlChannel_3") {
		return;
}

// -------------------------------------------------------------------

void TestCaseControlChannel_3::runTestCase() {

	I_Block block1; 
	I_Block block2; 
	Scalar scalar;

	// Reset the blocks
	block1.reset();
	block2.reset();
	
	// Define input signal intensity
	Real s=5;

	// Define integrator gains
	Real gain=3;
	block1.setGain(gain);
	block2.setGain(gain);

	// Retrieve data item write serving as source for control block
	DataItemWrite inputSource=scalar.getDataItemWrite(0);
	
	// Configure control block
	block1.linkInput(scalar.getDataItemRead(0),0);
	block2.linkInput(&block1,0,0);

	// Set initial integrator values 
	block1.setState(0,0);
	block1.setState(1,s);
	block2.setState(0,0);
	block2.setState(1,0);

	// Set the control block input 
	inputSource.set(s);

	// Integrate over n time intervals
	int n=10;
	AocsTime timeStep=2;
	AocsTime t0=aocsClock.getTime();
	AocsTime t=t0;
	for (int i=0; i<n; i++)
	{	t+=timeStep;
		block2.propagate(t);
		Real integral=block2.getOutput(0).get();	// facilitates debugging
	}
	
	// Check correct value of block output
	if ( block2.getOutput(0).get() != (gain*timeStep*n)*(gain*timeStep*n)*s/2 )
	{	setTestResult(TEST_FAILURE, "Block output is wrong");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
