// TestCaseXmathUcbBlock_1.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseXmathUcbBlock_1.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../SequentialDataProcessing/XmathUcbBlock.h"

// -------------------------------------------------------------------
TestCaseXmathUcbBlock_1::TestCaseXmathUcbBlock_1():
	TestCaseGenericSetUp(ID_XMATHUCBBLOCK*10+1,
									"TestCaseXmathUcbBlock_1") {
		return;
}

// -------------------------------------------------------------------

void TestCaseXmathUcbBlock_1::runTestCase() {

	Scalar scalar1;
	Scalar scalar2;

	// Define the UCB function pointer
	UcbHookFunction ucbHook=&firsttest_ucbhook;

	// Create Xmath UCB block as container for a UCB routine
	XmathUcbBlock block(2,1,1,0,0,0,ucbHook);
	
	// Reset the block
	block.reset();
	
	// Define input signal intensity
	Real s1=5;
	Real s2=10;

	// Retrieve data item write serving as source for control block
	DataItemWrite inputSource1=scalar1.getDataItemWrite(0);
	DataItemWrite inputSource2=scalar2.getDataItemWrite(0);
	
	// Configure control block
	block.linkInput(scalar1.getDataItemRead(0),0);
	block.linkInput(scalar2.getDataItemRead(0),1);

	// Check initial integrator conditions (set in Xmath block)
	if ( block.getState(0) != 0.0 )			
		{	setTestResult(TEST_FAILURE, "Block initial condition is wrong");
			return;
		}

	// Set the control block input 
	inputSource1.set(s1);
	inputSource2.set(s2);

	// Propagate block output over n time intervals
	int n=10;
	AocsTime timeStep=4;
	block.setSamplingTime(timeStep);
	AocsTime t0=aocsClock.getTime();
	AocsTime t=t0;
	for (int i=0; i<n; i++)
	{	t+=timeStep;
		block.propagate(t);
		Real output=block.getOutput(0).get();	// facilitates debugging
		if ((i==0) && (output != 89))
		{	setTestResult(TEST_FAILURE, "Block output is wrong");
			return;
		}
		if ((i>0) && (output != -5))
		{	setTestResult(TEST_FAILURE, "Block output is wrong");
			return;
		}
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
