// TestCaseXmathUcbBlock_2.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseXmathUcbBlock_2.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../SequentialDataProcessing/XmathUcbBlock.h"

// -------------------------------------------------------------------
TestCaseXmathUcbBlock_2::TestCaseXmathUcbBlock_2():
	TestCaseGenericSetUp(ID_XMATHUCBBLOCK*10+2,
									"TestCaseXmathUcbBlock_2") {
		return;
}

// -------------------------------------------------------------------

void TestCaseXmathUcbBlock_2::runTestCase() {

	Scalar scalar1;

	// Define the UCB function pointer
	UcbHookFunction ucbHook=&integratortest_ucbhook;

	// Create Xmath UCB block as container for a UCB routine
	XmathUcbBlock block(1,1,1,0,0,0,ucbHook);
	
	// Reset the block
	block.reset();
	
	// Define input signal intensity
	Real s1init=5;
	Real s1=s1init;

	// Retrieve data item write serving as source for control block
	DataItemWrite inputSource1=scalar1.getDataItemWrite(0);
	
	// Configure control block
	block.linkInput(scalar1.getDataItemRead(0),0);

	// Set the control block input 
	inputSource1.set(s1);

	// Propagate block output over n time intervals
	// The input to the block is a straight line 
	int n=10;
	AocsTime timeStep=4;
	Real slope=2;
	block.setSamplingTime(timeStep);
	AocsTime t0=aocsClock.getTime();
	AocsTime t=t0;
	for (int i=0; i<n; i++)
	{	block.propagate(t);
		Real output=block.getOutput(0).get();	// facilitates debugging
		if ( output != (integratorgain*(integratorinit+
				(s1init+s1)*(t-t0)/2)) )
		{	setTestResult(TEST_FAILURE, "Block output is wrong");
			return;
		}
		t+=timeStep;
		s1 += timeStep*slope;
		inputSource1.set(s1);
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
