// TestCaseXmathUcbBlock_3.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseXmathUcbBlock_3.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../SequentialDataProcessing/XmathUcbBlock.h"

// -------------------------------------------------------------------
TestCaseXmathUcbBlock_3::TestCaseXmathUcbBlock_3():
	TestCaseGenericSetUp(ID_XMATHUCBBLOCK*10+3,
									"TestCaseXmathUcbBlock_3") {
		return;
}

// -------------------------------------------------------------------

void TestCaseXmathUcbBlock_3::runTestCase() {

	// Define the UCB function pointer
	UcbHookFunction ucbHook=&ramp_ucbhook;

	// Create Xmath UCB block as container for a UCB routine
	XmathUcbBlock block(0,1,1,0,0,0,ucbHook);
	
	// Propagate block output over n time intervals
	// The input to the block is a straight line 
	int n=10;
	AocsTime timeStep=4;
	block.setSamplingTime(timeStep);
	AocsTime t0=aocsClock.getTime();
	starttime = t0;
	AocsTime t=t0;
	for (int i=0; i<n; i++)
	{	block.propagate(t);
		Real output=block.getOutput(0).get();	// facilitates debugging
		Real expOutput=(integratorgain*(integratorinit+(slope*(t-t0))*(t-t0)/2));
		if ( output != expOutput )
		{	setTestResult(TEST_FAILURE, "Block output is wrong");
			return;
		}
		t+=timeStep;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
