// TorqueToThrusterPrototype.cpp
//
// Version	1.0
// Date		06.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/ClassId.h"
#include "../SequentialDataProcessing/TorqueToThrusterPrototype.h"

// -------------------------------------------------------------------

TorqueToThrusterPrototype::TorqueToThrusterPrototype(Real t, Real p)				
{
	setClassId(ID_TORQUETOTHRUSTERPROTOTYPE);				

	thuTorque = t;
	period = p;

	allocateMemory(3,12,0);					// 3 inputs, 12 outputs, 0 states
					
}

// -------------------------------------------------------------------

void TorqueToThrusterPrototype::propagationAlgorithm(AocsTime t)
{
	// Saturate the torque requests
	for (int i=0; i<3; i++)
	{	if (input[i]>thuTorque)
			input[i]=thuTorque;
		if (input[i]<-thuTorque)
			input[i]=-thuTorque;
	}
	
	// Set the delay-times to zero
	for (i=0; i<6; i++)
		output[i]=0;

	// Compute the on-times
	for (i=6; i<12; i++)
		output[i]=0;

	Real temp=period/thuTorque;

	if (input[0]>0)
		output[6]=input[0]*temp;
	else
		output[7]=-input[0]*temp;

	if (input[1]>0)
		output[8]=input[1]*temp;
	else
		output[9]=-input[1]*temp;

	if (input[2]>0)
		output[10]=input[2]*temp;
	else
		output[11]=-input[2]*temp;
}
