// TwoByTwoMatrix.cpp
//
// Version	1.0
// Date		18.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../SequentialDataProcessing/TwoByTwoMatrixBlock.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"

// -------------------------------------------------------------------
// Define static variables
int TwoByTwoMatrixBlock::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
// Default constructor.

TwoByTwoMatrixBlock::TwoByTwoMatrixBlock()				
{
	setClassId(ID_TWOBYTWOMATRIX);			

	allocateMemory(2,2,0);				// 2 inputs, 2 outputs, 0 states

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = 4*sizeof(Real);	
	tmImageLength[longTm]   = tmImageLength[normalTm]+sizeof(AocsTime);									
	tmImageLength[debugTm]  = tmImageLength[longTm]+getInputOrder()*sizeof(Real)+
									getOutputOrder()*sizeof(Real);

	localResetConfiguration();					
}

// -------------------------------------------------------------------

void TwoByTwoMatrixBlock::propagationAlgorithm(AocsTime t)
{
	output[0] = matrix[0][0]*input[0]+matrix[0][1]*input[1];
	output[1] = matrix[1][0]*input[0]+matrix[1][1]*input[1];
}

// -------------------------------------------------------------------

void TwoByTwoMatrixBlock::localResetConfiguration()
{
	matrix[0][0] = 0.0;
	matrix[0][1] = 0.0;
	matrix[1][0] = 0.0;
	matrix[1][1] = 0.0;
}

// -------------------------------------------------------------------

void TwoByTwoMatrixBlock::resetConfiguration()
{
	ControlChannelBlock::resetConfiguration();	// reset super block's state
	localResetConfiguration();
}

// -------------------------------------------------------------------

void TwoByTwoMatrixBlock::writeToTelemetry(TelemetryStream* stream)
{
	ControlChannelBlock::writeToTelemetry(stream);		// write super object's state

	switch(getTelemetryFormat()) 
	{
		case shortTm:
			break;
		case normalTm:
			// TBW
			break;
		case longTm:
			// TBW 
			break;
		case debugTm:
			// TBW 
			break;
	}
}

// -------------------------------------------------------------------

int TwoByTwoMatrixBlock::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]+
				ControlChannelBlock::getTelemetryImageLength());
}


