// XmathUcbPidBlock.h
//
// Version	1.0
// Date		17.03.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Control block acting as a container for a UCB-style autocode procedure
// modelling a PID controller. The block offers methods to set and get the
// PID parameters.
//
// This class relies on the following assumptions:
// -- the basic type 'Real' is defined as 'double' (this is generally required
//	  for integration with autocode UCB blocks). 
// -- the PID parameters are called kp, kd and ki
// -- the PID modelling UCB routine is called pid_ucbhook
// -------------------------------------------------------------------

#ifndef XmathUcbPidBlockH
#define XmathUcbPidBlockH

// Control flag to avoid compilation of subsys version of autocode routine
#define SBUSER	1

// -------------------------------------------------------------------
#include "../Xmath/Include.h"
#include "../SequentialDataProcessing/XmathUcbBlock.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"

// -------------------------------------------------------------------
// Declare global variables from UCB routines
extern "C" VAR_FLOAT kp;
extern "C" VAR_FLOAT kd;
extern "C" VAR_FLOAT ki;
extern "C" VAR_FLOAT timeconstant;

// -------------------------------------------------------------------
// Declare UCB routines (this is the routine modelling the PID controller)
extern "C" void pid_ucbhook(int*,Real*,Real*,int*,Real*,Real*,
			int*,Real*,int*,Real*,int*);

// -------------------------------------------------------------------

class XmathUcbPidBlock : public XmathUcbBlock {

	static ClassId classId;
	static int tmImageLength[NTMFORMATS];

public :

	XmathUcbPidBlock(AocsTime samplingTime);

	virtual int getTelemetryImageLength();
	virtual void writeToTelemetry(TelemetryStream* stream);

	void setKp(Real newKp);

	Real getKp();
	
	void setKi(Real newKi);

	Real getKi();

	void setKd(Real newKd);

	Real getKd();

};

#endif
