// SystemManager.h
//
// Version	1.0
// Date		13.01.01
// Author	A. Pasetti
//
// Change Record
// Version 1.1: Added default constructor and method initialize(...)
//
// -------------------------------------------------------------------
// Class for the system manager (objects in charge of resetting all 
// AocsObjects and of checking whether they are configured)
//
// -------------------------------------------------------------------

#ifndef SystemManagerH
#define SystemManagerH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../BasicObjects/RootObject.h"
#include "../ObjectList/ObjectListTemplate.h"

// -------------------------------------------------------------------

class SystemManager : public RootObject {

	SystemEventRepository* systemEventRepository;

	ObjectListTemplate<AocsObject> list;

	// List of registered reconfiguration managers
	ObjectListTemplate<Reconfigurable> listReconfigurable;

	// configuration[i] holds the current configuration of the i-th
	// registered configuration manager
	ConfigurationState* configurationState;

	// restore the configuration of the registered configuration managers
	void loadConfigurations();

	bool enableConfigStateUpdate;

	int counter;
	int maxNumberOfObjects;

	int counterReconfigurable;
	int maxNumberOfReconfigurable;

public :

	SystemManager(); 

	SystemManager(int maxNumberOfObjects, 
					int maxNumberOfReconfigurable,
					  SystemEventRepository* systemEvtRep);

	// Initialization method (to be called when the default 
	// constructor is used
	initialize(int maxNumberOfObjects, 
					int maxNumberOfReconfigurable,
					  SystemEventRepository* systemEvtRep);
	
	// Registration methods for resettable objects
	void add(AocsObject* object);
	void remove(AocsObject* object);

	// Registration methods for reconfigurable objects
	void addReconfigurable(Reconfigurable* r);
	void removeReconfigurable(Reconfigurable* r);

	// Call-back method triggered by a change in configuration state
	// in a reconfigurable object
	void configurationStateChange(Reconfigurable* r);

	void systemReset();

	bool isSystemConfigured();

	SystemEventRepository* getSystemEventRepository()
		{ return systemEventRepository; }

};

// -------------------------------------------------------------------
#endif
