// Erc32DmaTelecommandLoader.h
//
// Version	1.0
// Date		09.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Implementation of base class DmaTelecommandLoader for the Erc32
// environment. 
// The structure of an object of class Telecommand is as follows:
//  - bytes  1- 2: instance identifier
//  - byte   3   : class identifier
//	- byte   4	 : not used
//	- bytes  5-7 : not used
//	- byte   8   : enabled status
//	- bytes  9-12: time tag
//	- bytes 13-16: VTable pointer
//
// The instance identifier is not in the TC block. It is created by
// the telecommand loader. 
// The enabled status is not in the TC block. It is initialized to
// 'true' by the telecommand loader.
// -------------------------------------------------------------------

#ifndef Erc32DmaTelecommandLoaderH
#define Erc32DmaTelecommandLoaderH

#define TC_INSTANCE_ID_BASE			1000;

// -------------------------------------------------------------------

class Erc32DmaTelecommandLoader : public DmaTelecommandLoader {

	InstanceId instanceIdCounter;

	// Construct a simple TC consisting of data only (the code is
	// already in memory)
	Telecommand* makeDataTelecommand(int lengthDataSection);

	// Construct a simple TC consisting of data and code - not  
	// implemented in prototype f/w
	Telecommand* makeCodeTelecommand(int lengthDataSection, int lengthVTableSection)
		{ return NULL; }

	// Construct a method section (a method section contains the code
	// for a method) - not implemented in prototype f/w
	char* makeMethodSection(int methodSectionStart, InstanceId tcId)
		{ return NULL; }

	// Construct a transaction TC consisting of data only (the code is
	// already in memory)
	TransactionTelecommand* makeDataTransactionTelecommand(
		int lengthDataSection, bool isFirst, TelecommandId tcId, AocsTime* timeTag);

	// Construct a transaction TC consisting of data and code - not  
	// implemented in prototype f/w
	TransactionTelecommand* makeCodeTransactionTelecommand(
		int lengthDataSection, int lengthVTableSection, bool isFirst, 
			TelecommandId tcId, AocsTime* timeTag)
		{ return NULL; }

protected:

	// Methods overriden from class DmaTelecommandLoader
	virtual Telecommand* makeSimpleTelecommand();

	virtual TransactionTelecommand* 
				makeTransactionTelecommand(bool isFirst, TelecommandId tcId, AocsTime* timeTag);

public:

	Erc32DmaTelecommandLoader(int* tcBuffer);

};

// -------------------------------------------------------------------
#endif