// ReconfigureTransactionTelecommand.cpp
//
// Version	1.0
// Date		07.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../TelecommandManagement/ReconfigureTransactionTelecommand.h"
#include "../TelecommandManagement/TransactionTelecommand.h"
#include "../TelecommandManagement/TelecommandManager.h"
#include "../ReconfigurationManagement/Reconfigurable.h"
#include "../GeneralInclude/ClassId.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------

ReconfigureTransactionTelecommand::ReconfigureTransactionTelecommand(
	 AocsTime timeTag, Reconfigurable* r):TransactionTelecommand(timeTag)
{
	setClassId(ID_RECONFIGURETRANSACTIONTELECOMMAND);			

	reconfigurationManager = r;
}

// -------------------------------------------------------------------

EventType ReconfigureTransactionTelecommand::execute()
{
	// Recover current configuration
	ConfigurationId oldConfig = reconfigurationManager->getConfiguration();

	// Store current configuration in the TC stack
	TransactionTelecommand::getTelecommandManager()->pushInt(oldConfig);

	// Command reconfiguration
	reconfigurationManager->reconfigure();

	// Check whether reconfiguration was successful
	if (reconfigurationManager->getConfiguration() != oldConfig)
		return TransactionTelecommand::execute();
	else
	{	// restore stack and unExecute previous TC in TC transaction
		TransactionTelecommand::getTelecommandManager()->popInt();	
		return TransactionTelecommand::unExecute();
	}
}

// -------------------------------------------------------------------

EventType ReconfigureTransactionTelecommand::unExecute()
{
	// Recover old configuration
	ConfigurationId oldConfig = TransactionTelecommand::
									getTelecommandManager()->popInt();

	// Mark old configuration as healthy
	reconfigurationManager->markConfiguration(oldConfig,true);

	// Restore old configuration
	reconfigurationManager->setConfiguration(oldConfig);

	// Check whether restore operation was successful
	if (reconfigurationManager->getConfiguration() == oldConfig)
		return TransactionTelecommand::unExecute();
	else
		return TC_TRANS_UNDO_FAILURE;
}
