// TransactionTelecommand.cpp
//
// Version	1.0
// Date		07.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../GeneralInclude/Include.h"
#include "../TelemetryManagement/Telemeterable.h"
#include "../BasicObjects/Include.h"
#include "../OperatingSystemObjects/Include.h"
#include "../FailureDetection/ConsistencyCheckable.h"
#include "../ObjectList/Include.h"
#include "../AocsData/DataItemRead.h"
#include "../ObjectMonitoring/Include.h"
#include "../AocsEvent/Include.h"
#include "../ModeManagement/Include.h"
#include "../TelecommandManagement/Include.h"

// -------------------------------------------------------------------
// Define static variables
TelecommandManager*	TransactionTelecommand::tcManager=NULL;

// -------------------------------------------------------------------

TransactionTelecommand::TransactionTelecommand(AocsTime t):Telecommand(t)
{	
	setClassId(ID_TRANSACTIONTELECOMMAND);
	next = NULL;
	previous = NULL;
}

// -------------------------------------------------------------------

EventType TransactionTelecommand::execute()
{	
	if (next!=NULL)
		return next->execute();
	else
		return TC_TRANS_SUCCESS;
}

// -------------------------------------------------------------------

EventType TransactionTelecommand::unExecute()
{	
	if (previous!=NULL)
		return previous->unExecute();
	else
		return TC_TRANS_FAILURE;
}

// -------------------------------------------------------------------

bool TransactionTelecommand::canExecute()
{	
	if (next!=NULL)
		return next->canExecute();
	else
		return true;
}

// -------------------------------------------------------------------
// If this TC is not part of a transaction, it returns true if the TC 
// is enabled.
// If this TC is part of a transaction, it returns true if all TC's in
// the transaction are enabled.

bool TransactionTelecommand::isEnabled()
{	
	if (!Telecommand::isEnabled())
		return false;
	else
		if (next!=NULL)
			return next->isEnabled();
		else 
			return true;
}

// -------------------------------------------------------------------

void TransactionTelecommand::setNextTransactionTelecommand(
			TransactionTelecommand* t)
{	
	next = t; 
	if (t->getPreviousTransactionTelecommand()==NULL)
		t->setPreviousTransactionTelecommand(this); 
}

// -------------------------------------------------------------------

void TransactionTelecommand::setPreviousTransactionTelecommand(
			TransactionTelecommand* t)
{	
	previous = t; 
	if (t->getNextTransactionTelecommand()==NULL)
		t->setNextTransactionTelecommand(this); 
}

