// VsDmaTelecommandLoader.h
//
// Version	1.0
// Date		09.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Implementation of base class DmaTelecommandLoader for the Visual
// Studio environment. 
// The structure of an object of class Telecommand is as follows:
//	- bytes  0- 3: VTable pointer
//  - bytes  4- 5: instance identifier
//  - byte   6   : class identifier
//	- byte   7	 : not used
//	- byte   8   : enabled status
//	- bytes  9-11: not used
//	- bytes 12-15: time tag
//
// The instance identifier is not in the TC block. It is created by
// the telecommand loader. 
// The enabled status is not in the TC block. It is initialized to
// 'true' by the telecommand loader.
// -------------------------------------------------------------------

#ifndef VsDmaTelecommandLoaderH
#define VsDmaTelecommandLoaderH

#define TC_INSTANCE_ID_BASE			1000;

// -------------------------------------------------------------------

#include "../TelecommandManagement/DmaTelecommandLoader.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"

// -------------------------------------------------------------------

class VsDmaTelecommandLoader : public DmaTelecommandLoader {

	InstanceId instanceIdCounter;

	// Construct a simple TC consisting of data only (the code is
	// already in memory)
	Telecommand* makeDataTelecommand(int lengthDataSection);

	// Construct a simple TC consisting of data and code
	Telecommand* makeCodeTelecommand(int lengthDataSection, int lengthVTableSection);

	// Construct a method section (a method section contains the code
	// for a method).
	char* makeMethodSection(int methodSectionStart, InstanceId tcId);

	// Construct a transaction TC consisting of data only (the code is
	// already in memory)
	TransactionTelecommand* makeDataTransactionTelecommand(
		int lengthDataSection, bool isFirst, TelecommandId tcId, AocsTime* timeTag);

	// Construct a transaction TC consisting of data and code
	TransactionTelecommand* makeCodeTransactionTelecommand(
		int lengthDataSection, int lengthVTableSection, bool isFirst, 
			TelecommandId tcId, AocsTime* timeTag);

protected:

	// Methods overriden from class DmaTelecommandLoader
	virtual Telecommand* makeSimpleTelecommand();

	virtual TransactionTelecommand* 
				makeTransactionTelecommand(bool isFirst, TelecommandId tcId, AocsTime* timeTag);

public:

	VsDmaTelecommandLoader(int* tcBuffer);

};

// -------------------------------------------------------------------
#endif