// DmaTelemetryStream.cpp
//
// Version	1.0
// Date		08.04.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include <string.h>

#include "../TelemetryManagement/DmaTelemetryStream.h"
#include "../GeneralInclude/ClassId.h"

// -------------------------------------------------------------------
// Define static variables
int DmaTelemetryStream::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

DmaTelemetryStream::DmaTelemetryStream()
{
	setClassId(ID_DMATELEMETRYSTREAM);			// assign class identifier

	tmImageLength[shortTm]  = 0;		// not implemented in f/w prototype	
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	tmFrameCounter = 0;

	localResetConfiguration();
}

// -------------------------------------------------------------------

void DmaTelemetryStream::localResetConfiguration()
{
	dmaBuffer = 0;
	bufferSize = 0;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool DmaTelemetryStream::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if ( (dmaBuffer == 0) || (bufferSize == 0) )
		return false;

	return true;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::resetBuffer()
{
	tmFrameCounter++;

	nextByte=5;
	nextBit=0;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::write(double d)
{
//	double* pDouble;
//	pDouble = (double*)(dmaBuffer+nextByte);
//	*pDouble = d;
	memcpy(dmaBuffer+nextByte,&d,8);
	nextByte+=8;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::write(float d)
{
//	float* pFloat;
//	pFloat = (float*)(dmaBuffer+nextByte);
//	*pFloat = d;
	memcpy(dmaBuffer+nextByte,&d,4);
	nextByte+=4;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::write(int d)
{
//	int* pInt;
//	pInt = (int*)(dmaBuffer+nextByte);
//	*pInt = d;
	memcpy(dmaBuffer+nextByte,&d,4);
	nextByte+=4;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::write(unsigned int d)
{
//	int* pInt;
//	pInt = (int*)(dmaBuffer+nextByte);
//	*pInt = d;
	memcpy(dmaBuffer+nextByte,&d,4);
	nextByte+=4;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::write(unsigned char d)
{
	memcpy(dmaBuffer+nextByte,&d,1);
	nextByte+=1;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::write(short d)
{
//	short* pShort;
//	pShort = (short*)(dmaBuffer+nextByte);
//	*pShort = d;
	memcpy(dmaBuffer+nextByte,&d,2);
	nextByte+=2;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::write(unsigned short d)
{
//	unsigned short* pShort;
//	pShort = (unsigned short*)(dmaBuffer+nextByte);
//	*pShort = d;
	memcpy(dmaBuffer+nextByte,&d,2);
	nextByte+=2;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::write(bool d)
{
	if (d)
		dmaBuffer[nextByte]=1;
	else
		dmaBuffer[nextByte]=0;
//	memcpy(dmaBuffer+nextByte,&d,1);
	nextByte+=1;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::write(char* d, int n)
{
	memcpy(dmaBuffer+nextByte,d,n);
	nextByte+=n;

}

// -------------------------------------------------------------------
// The frame number is written to the first two bytes in the DMA buffer

void DmaTelemetryStream::writeFrameNumber(int n)			
{
//	unsigned short* pShort;
//	pShort = (unsigned short*)(dmaBuffer);
//	*pShort = (unsigned short)n;;
	unsigned short d = (unsigned short)n;
	memcpy(dmaBuffer,&d,2);
}

// -------------------------------------------------------------------
// The number of bytes in the DMA buffer is written to the third and
// fourth byte of the buffer

void DmaTelemetryStream::writeNumberOfBytes(int n)			
{
//	unsigned short* pShort;
//	pShort = (unsigned short*)(dmaBuffer+2);
//	*pShort = (unsigned short)n;
	unsigned short d = (unsigned short)n;
	memcpy(dmaBuffer+2,&d,2);
}

// -------------------------------------------------------------------
// The number of bits is the fifth byte in the DMA buffer

void DmaTelemetryStream::writeNumberOfBits(int n)			
{
	n = n & 0x000000FF;		
	dmaBuffer[4]=(char)n;
}

// -------------------------------------------------------------------

void DmaTelemetryStream::flushBuffer()			
{
	// write the telemetry frame number
	writeFrameNumber(tmFrameCounter);		

	// write the byte counter
	writeNumberOfBytes(nextByte);		

	// write the bit counter
	writeNumberOfBits(nextBit);		
}

// -------------------------------------------------------------------
// From Telemeterable interface

int DmaTelemetryStream::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void DmaTelemetryStream::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// Not implemented in framework prototype 
}