// MemorySection.cpp
//
// Version	1.0
// Date		11.04.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../TelemetryManagement/MemorySection.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../AocsEvent/EventTypeId.h"
#include "../TelemetryManagement/TelemetryStream.h"

// -------------------------------------------------------------------
// Define static variables
int MemorySection::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

MemorySection::MemorySection()
{
	setClassId(ID_MEMORYSECTION);			// assign class identifier

	tmImageLength[shortTm]  = 0;			
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	localResetConfiguration();
}

// -------------------------------------------------------------------

void MemorySection::localResetConfiguration()
{
	startAddress = NULL;
	endAddress = NULL;
	sectionLength = 0;
}

// -------------------------------------------------------------------

void MemorySection::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool MemorySection::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if ( (startAddress==NULL) && (endAddress==NULL) )
		return false;

	return true;
}

// -------------------------------------------------------------------

void MemorySection::defineMemorySection(char* start, char* end)
{
	if (start >= end)
		reportConfigurationError(this,ILLEGAL_MEMORY_SECTION,this);

	startAddress = start;
	endAddress = end;
	sectionLength = end-start;

	tmImageLength[normalTm] = 8*sectionLength;
	tmImageLength[longTm]   = 8*sectionLength;
	tmImageLength[debugTm]  = 8*sectionLength;
}

// -------------------------------------------------------------------
// From Telemeterable interface

int MemorySection::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void MemorySection::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	if (getTelemetryFormat() != shortTm)
		tmStream->write(startAddress,sectionLength);
}

