// TestCaseTelemetry_2.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseTelemetry_2.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../BasicObjects/AocsObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"
#include "../../TelemetryManagement/TestTelemetryStream.h"
#include "../../TelemetryManagement/TelemetryManager.h"
#include "../../TelemetryManagement/CyclingTelemetryModeManager.h"

// -------------------------------------------------------------------
TestCaseTelemetry_2::TestCaseTelemetry_2():
	TestCaseGenericSetUp(ID_TELEMETRYMANAGER*10+2,
									"TestCaseTelemetry_2") {
		return;
}

// -------------------------------------------------------------------

void TestCaseTelemetry_2::runTestCase() {

	// Create dummy objects to be used as test TM objects
	AocsObject ao1, ao2, ao3;
	TwoEulerAngles euler1, euler2, euler3;

	// Initialize the Euler angle values
	euler1.getDataItemWrite(0).set(1);
	euler2.getDataItemWrite(0).set(2);
	euler3.getDataItemWrite(0).set(3);

	// Create a telemetry list and load it
	ObjectListTemplate<Telemeterable> tmList;
	tmList.allocateMemory(10);
	tmList.addObject(&ao1);
	tmList.addObject(&ao2);
	tmList.addObject(&ao3);
	tmList.addObject(&euler1);
	tmList.addObject(&euler2);
	tmList.addObject(&euler3);
	tmList.addObject(AocsObject::getConfigurationEventRepository());
	
	// Create and configure a telemetry mode manager with a single mode
	CyclingTelemetryModeManager tmModeManager(1,0,0);
	tmModeManager.loadTelemetryList(0,&tmList);
	NullModeChangeAction nra;
	tmModeManager.setModeChangeAction(&nra);

	// Create a test TM stream
	TestTelemetryStream tmStream;

	// Create and configure a telemetry manager
	TelemetryManager tmManager;
	tmManager.setTelemetryStream(&tmStream);
	tmManager.setTelemetryModeManager(&tmModeManager);
	
	// Run the telemetry manager 
	tmManager.run(0);

	// Change the telemetry format of the Euler angle objects
	euler1.setTelemetryFormat(longTm);
	euler2.setTelemetryFormat(longTm);
	euler3.setTelemetryFormat(longTm);
	tmManager.run(0);

	// Force the creation of configuration events
	tmList.allocateMemory(20);					// illegal --> configuration event
	tmModeManager.loadTelemetryList(1,&tmList);	// illegal --> configuration event

	// Delete all entries from the tmList except for the event repository
	tmList.removeObject(&ao1);
	tmList.removeObject(&ao2);
	tmList.removeObject(&ao3);
	tmList.removeObject(&euler1);
	tmList.removeObject(&euler2);
	tmList.removeObject(&euler3);

	// Run the telemetry manager
	tmManager.run(0);

	// Change the TM format of the failure event repository and rerun TM manager
	AocsObject::getConfigurationEventRepository()->setTelemetryFormat(longTm);
	tmManager.run(0);

	// Create and load a second telemetry list
	ObjectListTemplate<Telemeterable> tmList_2;
	tmList_2.allocateMemory(10);
	tmList_2.addObject(&ao1);
	
	// Create a new telemetry mode manager with 2 modes and load it with two telemetry lists
	CyclingTelemetryModeManager tmModeManager_2(2,0,0);
	tmModeManager_2.loadTelemetryList(0,&tmList);
	tmModeManager_2.loadTelemetryList(1,&tmList_2);
	tmModeManager_2.setModeChangeAction(&nra);

	// Replace the mode manager in the telemetry manager
	tmManager.setTelemetryModeManager(&tmModeManager_2);

	// Run the telemetry manager four times
	tmManager.run(0);
	tmManager.run(0);
	tmManager.run(0);
	tmManager.run(0);

	// Retrieve the number of failure events
	failureEvtN = AocsObject::getFailureEventRepository()->getCounter();

	// Change the size of the test telemetry buffer to a value lower than the
	// size of one of the two telemetry lists
	tmStream.setBufferSize(10);

	// Run the telemetry manager twice and verify that one failure event was created
	tmManager.run(0);
	tmManager.run(0);
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN+1)
	{	setTestResult(TEST_FAILURE, "Telemetry test 2 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->latestFailureEvent()->getEventType()!=TOO_MANY_TM_DATA)
	{	setTestResult(TEST_FAILURE, "Telemetry test 2 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}


