// TelemetryManager.h
//
// Version	1.0
// Date		07.04.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Telemetry manager class
// -------------------------------------------------------------------

#ifndef TelemetryManagerH
#define TelemetryManagerH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"
#include "../OperatingSystemObjects/Runnable.h"
#include "../ObjectList/ObjectListTemplate.h"

// -------------------------------------------------------------------

class TelemetryManager : public AocsObject, public Runnable {

	static int tmImageLength[NTMFORMATS];

	// List of objects to be sent to telemetry
	ObjectListTemplate<Telemeterable>* tmList;

	// Stream to which telemetry data are to be written
	TelemetryStream* tmStream;

	// Mode manager to supply telemetry list
	TelemetryModeManager* modeManager;

	// Recovery action for TM data not fitting in TM buffer
	RecoveryAction* tooManyTmData;

	// Local reset configuration method
	void localResetConfiguration();

public:

	// Constructor
	TelemetryManager();

	// Scheduler entry point
	virtual void run(AocsTime deadline);

	// pure virtual methods from Configurable
	virtual void resetConfiguration();		
	virtual bool isConfigured();		
	
	// Virtual methods inherited from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// Straightforward getter and setter methods
	void setTelemetryModeManager(TelemetryModeManager* tmm)
		{ modeManager = tmm; }
	TelemetryModeManager* getTelemetryModeManager()
		{ return modeManager; }
	
	TelemetryStream* getTelemetryStream()
		{ return tmStream; }
	void setTelemetryStream(TelemetryStream* tms)
		{ tmStream = tms; }

	void setRecoveryAction(RecoveryAction* r)
		{ tooManyTmData = r; }
	RecoveryAction* getRecoveryAction()
		{ return tooManyTmData; }

};

// -------------------------------------------------------------------
#endif