// TestTelemetryStream.cpp
//
// Version	1.0
// Date		07.04.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../GeneralInclude/Include.h"
#include "../TelemetryManagement/Telemeterable.h"
#include "../BasicObjects/Include.h"
#include "../OperatingSystemObjects/Runnable.h"
#include "../FailureDetection/ConsistencyCheckable.h"
#include "../ObjectList/Include.h"
#include "../AocsData/DataItemRead.h"
#include "../ObjectMonitoring/Include.h"
#include "../AocsEvent/Include.h"
#include "../ModeManagement/Include.h"
#include "../TelemetryManagement/Include.h"

// -------------------------------------------------------------------
// Define static variables
int TestTelemetryStream::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

TestTelemetryStream::TestTelemetryStream()
{
	setClassId(ID_TESTTELEMETRYSTREAM);			// assign class identifier

	// Open output file where TM data will be written
    tmStreamFile = fopen("TestTelemetryStream.txt","w");

	tmImageLength[shortTm]  = 0;		// not implemented in f/w prototype	
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	tmFrameCounter = 0;
	bufferSize=1000;						// default buffer size in bytes

}

// -------------------------------------------------------------------

TestTelemetryStream::~TestTelemetryStream()
{
	// Open output file where TM data will be written
    fclose(tmStreamFile);
}

// -------------------------------------------------------------------

void TestTelemetryStream::resetBuffer()
{
	tmFrameCounter++;

	byteCounter=0;
	bitCounter=0;

#ifdef MS_HOST
	fprintf(tmStreamFile,"--------------------------------------\n");
	fprintf(tmStreamFile,"Frame N.: %d\n", tmFrameCounter);
#else
	printf("--------------------------------------\n");
	printf("Frame N.: %d\n", tmFrameCounter);
#endif
}

// -------------------------------------------------------------------

void TestTelemetryStream::write(double d)
{
	byteCounter+=8;
#ifdef MS_HOST
	fprintf(tmStreamFile,"double - %f\n", d);
#else
	printf("double - %f\n", d);
#endif
}

// -------------------------------------------------------------------

void TestTelemetryStream::write(float d)
{
	byteCounter+=4;
#ifdef MS_HOST
	fprintf(tmStreamFile,"float - %f\n", (double)d);
#else
	printf("float - %f\n", (double)d);
#endif
}

// -------------------------------------------------------------------

void TestTelemetryStream::write(int d)
{
	byteCounter+=4;
#ifdef MS_HOST
	fprintf(tmStreamFile,"int (in hex) - %x\n", d);
#else
	printf("int (in hex) - %x\n", d);
#endif
}

// -------------------------------------------------------------------

void TestTelemetryStream::write(unsigned int d)
{
	byteCounter+=4;
#ifdef MS_HOST
	fprintf(tmStreamFile,"unsigned int (in hex) - %x\n", d);
#else
	printf("unsigned int (in hex) - %x\n", d);
#endif
}

// -------------------------------------------------------------------

void TestTelemetryStream::write(short d)
{
	byteCounter+=2;
#ifdef MS_HOST
	fprintf(tmStreamFile,"short (in hex) - %x\n", d);
#else
	printf("short (in hex) - %x\n", d);
#endif
}

// -------------------------------------------------------------------

void TestTelemetryStream::write(unsigned short d)
{
	byteCounter+=2;
#ifdef MS_HOST
	fprintf(tmStreamFile,"unsigned short (in hex) - %x\n", d);
#else
	printf("unsigned short (in hex) - %x\n", d);
#endif
}

// -------------------------------------------------------------------

void TestTelemetryStream::write(bool d)
{
	byteCounter+=1;
#ifdef MS_HOST
	fprintf(tmStreamFile,"bool - %d\n", d);
#else
	printf("bool - %d\n", d);
#endif
}

// -------------------------------------------------------------------

void TestTelemetryStream::writeFrameNumber(int n)			
{
#ifdef MS_HOST
	fprintf(tmStreamFile,"frame number : %d\n", n);
#else
	printf("frame number : %d\n", n);
#endif
}

// -------------------------------------------------------------------

void TestTelemetryStream::writeNumberOfBytes(int n)			
{
#ifdef MS_HOST
	fprintf(tmStreamFile,"number of bytes: %d\n", n);
#else
	printf("number of bytes: %d\n", n);
#endif
}

// -------------------------------------------------------------------

void TestTelemetryStream::flushBuffer()			
{
	// write the telemetry frame number
	writeFrameNumber(tmFrameCounter);		

	// write the byte counter
	writeNumberOfBytes(byteCounter);		

	// write the byte counter
	writeNumberOfBits(bitCounter);		
}

// -------------------------------------------------------------------
// From Telemeterable interface

int TestTelemetryStream::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void TestTelemetryStream::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// Not implemented in framework prototype 
}