/*----------------------------------------------------------------------
--                       AutoCode (TM) 62mx1118
--
--                         Copyright (C) 1999
--                       INTEGRATED SYSTEMS, INC.
--
--  Unpublished work - rights reserved under the U.S. Copyright Act.  Use,
--  duplication, or disclosure by the government is subject to restrictions
--  set forth in the ISI commercial license agreement.
**----------------------------------------------------------------------
**
**  File       : sa_matrix.h
**  Project    : Autocode/C
**
**  Abstract:
**
**      Definitions for additional AutoCode matrix function support.
*/

#ifndef _SA_MATRIX_H
#define _SA_MATRIX_H

#include "sa_math.h"

typedef enum {
   columnMajor = 1
  ,rowMajor    = 2
} OriginStorage;

typedef enum {
   convert2col  = 1
  ,convert2row  = 2
} ConvertStorage;

#if defined(PC)
#define MLQ __stdcall      /* Mixed Language Qualifier */
#else
#define MLQ
#endif

#if (FC_)
         /* The following are FORTRAN routines and are called by C. */
#define  RTgeco   RTgeco_
#define  RTgefa   RTgefa_
#define  RTgesl   RTgesl_
#define  RTgedi   RTgedi_

      /* The following are C routines and are called by FORTRAN. */
#define  RTdot    RTdot_
#define  RTasum   RTasum_
#define  RTaxpy   RTaxpy_
#define  RTscal   RTscal_
#define  RTswap   RTswap_
#define  idamax   idamax_

#elif ( CRAY || PC || VAX )

#define  RTgeco   RTGECO
#define  RTgefa   RTGEFA
#define  RTgesl   RTGESL
#define  RTgedi   RTGEDI

#define  RTdot    RTDOT
#define  RTasum   RTASUM
#define  RTaxpy   RTAXPY
#define  RTscal   RTSCAL
#define  RTswap   RTSWAP
#define  idamax   IDAMAX

#endif

/*-------------------------------------------------------------------
| Prototypes for functions related to both factorization algorithms |
|              LU (Gaussian) and QR (Housholder)                    |
-------------------------------------------------------------------*/

extern void MLQ     RTaxpy (RT_INTEGER *, RT_FLOAT *, RT_FLOAT *,
                            RT_INTEGER *, RT_FLOAT *, RT_INTEGER *);

extern RT_FLOAT MLQ  RTdot (RT_INTEGER *, RT_FLOAT *, RT_INTEGER *,
                            RT_FLOAT *, RT_INTEGER *);

extern void MLQ     RTscal (RT_INTEGER *, RT_FLOAT *, RT_FLOAT *, 
                            RT_INTEGER *);

extern void MLQ     RTswap (RT_INTEGER *, RT_FLOAT *, RT_INTEGER *,
                            RT_FLOAT *, RT_INTEGER *);

extern void MatrixCopy_ChangeStorage (RT_FLOAT *, RT_FLOAT *,
                                      RT_INTEGER, RT_INTEGER,
                                      OriginStorage, ConvertStorage);

/*----------------------------------------------------------------
| Prototypes for functions related to LU factorization algorithm |
| using Gaussian elimination.                                    |
----------------------------------------------------------------*/

extern void Matrix_Inverse (RT_FLOAT *, RT_INTEGER, 
                                  RT_INTEGER *, RT_FLOAT *);

extern void Matrix_RightDivide (RT_FLOAT *, RT_FLOAT *, RT_INTEGER,
                                      RT_INTEGER, RT_INTEGER *, RT_FLOAT *);

extern void Matrix_LeftDivide (RT_FLOAT *, RT_FLOAT *, RT_FLOAT *,
                                     RT_INTEGER, RT_INTEGER,
                                     RT_INTEGER *, RT_FLOAT *);

extern void MLQ RTgeco (RT_FLOAT *, RT_INTEGER *, RT_INTEGER *,
                        RT_INTEGER *, RT_FLOAT *, RT_FLOAT *) ;

extern void MLQ RTgesl (RT_FLOAT *, RT_INTEGER *, RT_INTEGER *,
                        RT_INTEGER *, RT_FLOAT *, RT_INTEGER *);

extern void MLQ RTgedi (RT_FLOAT *, RT_INTEGER *, RT_INTEGER *, 
                        RT_INTEGER *, RT_FLOAT *, RT_FLOAT *,
                        RT_INTEGER *);

extern void MLQ RTgefa (RT_FLOAT *, RT_INTEGER *, RT_INTEGER *,
                        RT_INTEGER *, RT_INTEGER *);

extern RT_FLOAT MLQ RTasum (RT_INTEGER *, RT_FLOAT *, RT_INTEGER *);

extern RT_INTEGER MLQ IdaMax (RT_INTEGER *, RT_FLOAT *, RT_INTEGER *);


/*----------------------------------------------------------------
| Prototypes for functions related to QR factorization algorithm |
| using orthogonal Housholder transformations.                   |
----------------------------------------------------------------*/

extern void Matrix_RightDivideQR (RT_FLOAT *, RT_FLOAT *, RT_INTEGER,
                                  RT_INTEGER, RT_INTEGER *, RT_FLOAT *);

extern void Matrix_LeftDivideQR  (RT_FLOAT *, RT_FLOAT *, RT_FLOAT *,
                                  RT_INTEGER, RT_INTEGER,
                                  RT_INTEGER *,RT_FLOAT *);

extern void MLQ RTqrdc (RT_FLOAT *, RT_INTEGER *, RT_INTEGER *,
                        RT_INTEGER *, RT_FLOAT *, RT_INTEGER *,
                        RT_FLOAT *, RT_INTEGER *);

extern void MLQ RTqrsl (RT_FLOAT *, RT_INTEGER *, RT_INTEGER *, RT_INTEGER *,
                 RT_FLOAT *, RT_FLOAT *, RT_FLOAT *, RT_FLOAT *, RT_FLOAT *,
                 RT_FLOAT *, RT_FLOAT *, RT_INTEGER *, RT_INTEGER *);

extern RT_FLOAT MLQ RTnrm2 (RT_INTEGER *, RT_FLOAT *, RT_INTEGER *);

extern void         RTcopy1 (RT_INTEGER, RT_FLOAT *, RT_FLOAT *);

#endif         /* _SA_MATRIX_H */
