/*----------------------------------------------------------------------------
--                       AutoCode (TM) 62mx1118
--
--                         Copyright (C) 1999
--                       INTEGRATED SYSTEMS, INC.
--
--  Unpublished work - rights reserved under the U.S. Copyright Act.  Use,
--  duplication, or disclosure by the government is subject to restrictions
--  set forth in the ISI commercial license agreement.
**----------------------------------------------------------------------------
**
**  File       : sa_types.h
**  Project    : Autocode/C
**
**  Abstract:
**
**      Basic ISI typedefs for the stand-alone utilities and AutoCode/C.
*/

#ifndef _SA_TYPES

/*================================================================
                   S T A N D - A L O N E   T Y P E S                
 ================================================================*/

/*================================================================
    Data Type Names Data Type Description                          
    RT_UBYTE        Unsigned 8 bit data (unsigned char)      (ub)   
    RT_SBYTE        Signed 8 bit data   (signed char )       (sb)   
    RT_USHORT       Unsigned 16 bit data (unsigned int or    (us)   
                                         unsigned short int)       
    RT_SSHORT      Signed 16 bit data (int or short int)     (ss)   
    RT_ULONG       Unsigned 32 bit data (unsigned int or     (ul)  
                                         unsigned long int)        
    RT_SLONG       Signed 32 bit data (int or long)          (sl)  
 ================================================================*/


/* default typedefs in sa_types.h (user configurable) */

#if (__STDC__)
typedef signed char        RT_BYTE;
typedef signed short       RT_SHORT;

#if defined(OSF1)
    typedef signed int     RT_LONG;
#else
    typedef signed long    RT_LONG;
#endif

#else

typedef char               RT_BYTE;
typedef short              RT_SHORT;
#if defined(OSF1)
    typedef int            RT_LONG;
#else
    typedef long           RT_LONG;
#endif

#endif

typedef unsigned char      RT_UBYTE;
typedef RT_BYTE            RT_SBYTE;
typedef unsigned short     RT_USHORT;      
typedef RT_SHORT           RT_SSHORT;      

#if defined(OSF1)
    typedef unsigned int   RT_ULONG;       
#else
    typedef unsigned long  RT_ULONG;       
#endif
typedef RT_LONG            RT_SLONG;       


#define RT_UBYTE_MIN        0x0
#define RT_UBYTE_MAX        0xFF 
#define RT_SBYTE_MIN        -0x80
#define RT_SBYTE_MAX        0x7F 
#define RT_USHORT_MIN       0x0
#define RT_USHORT_MAX       0xFFFF 
#define RT_SSHORT_MIN       -0x8000 
#define RT_SSHORT_MAX       0x7FFF 
#define RT_ULONG_MIN        0x0
#define RT_ULONG_MAX        0xFFFFFFFF 
#define RT_SLONG_MIN        (RT_SLONG) 0x80000000
#define RT_SLONG_MAX        0x7FFFFFFF 


/*======================================================================*/
/* The default type definition for RT_INTEGER is RT_SLONG, i.e.         */
/* 32-bit signed data.  If the user needs to change the type of         */
/* RT_INTEGER to a smaller type,  the following RT_INTEGER related      */
/* #defines need to be update appropriately.                            */
/*======================================================================*/

typedef RT_SLONG             RT_INTEGER;   
#define RT_INTEGER_IS_SLONG  1
#define RT_INTEGER_IS_SSHORT 0

#define RT_INTEGER_MIN       RT_SLONG_MIN
#define RT_INTEGER_MAX       RT_SLONG_MAX 

#if (WS_TARGET || AC100DOUBLE)
typedef double          RT_FLOAT;
#else
typedef float           RT_FLOAT;
#endif

typedef RT_FLOAT    VAR_FLOAT;
typedef RT_INTEGER  VAR_INTEGER;
typedef char        VAR_CHAR;

typedef int                 RT_BOOLEAN;
typedef double              RT_DURATION;

/*
** Define pointers for use in the application.
*/
typedef RT_DURATION*  PRT_DURATION;
typedef RT_FLOAT*     PRT_FLOAT;
typedef PRT_FLOAT*    PPRT_FLOAT;
typedef RT_INTEGER*   PRT_INTEGER;
typedef PRT_INTEGER*  PPRT_INTEGER;
typedef char*         Pchar;
 
/* ERROR is a #define in VxWorks */
#ifdef ERROR
#define tempERROR
#undef ERROR
#endif

/*
** Note: There are many places in the generated code where the STATUS_RECORD
** is aliased onto an array of integers (RT_INTEGERS) called iinfo. So
** we can't in all truthfullnes have anything but an RT_INTEGER int the
** elements of this structure (no RT_BOOLEANs).
**
** Note: for backward compatibility the Model and Subsys elements are
** preserved in this structure.  The information pointed to can be
** accessed alternatively as TDBModel->attachMsg and TDBSubsys->attachMsg;
*/
#if defined(AC1000) || defined(_NT) || defined(RS_VXWORKS) || defined(RS_UNIX)
typedef struct STATUS_RECORD { 
        RT_INTEGER      ERROR;
        RT_INTEGER      INIT;
        RT_INTEGER      STATES;
        RT_INTEGER      OUTPUTS;
        RT_INTEGER      CONT_INFO;
        struct _TMSGAttachModel  *Model;     /* Attach message for calling model
*/
        struct _TMSGAttachSubsys *Subsys;    /* Attach message for calling subsystem */
        struct _TDBModelObject   *TDBModel;  /* TDB entry for model */
        struct _TDBSubsysObject  *TDBSubsys; /* TDB entry for subsystem */
} RT_STATUS_RECORD;
#else
typedef struct STATUS_RECORD { 
        RT_INTEGER      ERROR;
        RT_BOOLEAN      INIT;
        RT_BOOLEAN      STATES;
        RT_BOOLEAN      OUTPUTS;
} RT_STATUS_RECORD;
#endif

/*
** Definitions for backward compatibility (Release 2.14 and earlier of MATE MMS S/W)
** of model instance information in  user-code blocks.
*/
#define MODELNAME     Model->ModelName
#define MODELINSTANCE Model->ModelInstance
#define RT_FLOAT_SIZE Model->RTFloatSize 
 
#ifdef tempERROR
#define ERROR (-1)
#endif

/*
** Error codes used in AutoCode generated code for the ERROR parameter in
** STATUS_RECORD
*/
#ifndef STOP_BLOCK
#define STOP_BLOCK 1
#endif

#ifndef MATH_ERROR
#define MATH_ERROR 2
#endif

#ifndef UCB_ERROR
#define UCB_ERROR  -2      /* sameas SYSBLD */
#endif

#ifndef INIT_ERROR
#define INIT_ERROR 8
#endif

#ifndef TIME_OVERFLOW
#define TIME_OVERFLOW -1
#endif

#ifndef UNKNOWN_ERROR
#define UNKNOWN_ERROR 100
#endif

/*=======================================================================
           U S E R   D E F I N E D   C O N S T A N T   T Y P E  
 =======================================================================*/

#ifndef RT_CONSTANT
#ifdef __STDC__
#define RT_CONSTANT static const
#else  /* __STDC__ */
#define RT_CONSTANT static
#endif /* __STDC__ */
#endif /* RT_CONSTANT */

/*=======================================================================
                F I X E D - P O I N T  T Y P E S 
 =======================================================================*/

#define RT_UBYTEn16 RT_UBYTE
#define RT_UBYTEn15 RT_UBYTE
#define RT_UBYTEn14 RT_UBYTE
#define RT_UBYTEn13 RT_UBYTE
#define RT_UBYTEn12 RT_UBYTE
#define RT_UBYTEn11 RT_UBYTE
#define RT_UBYTEn10 RT_UBYTE
#define RT_UBYTEn09 RT_UBYTE
#define RT_UBYTEn08 RT_UBYTE
#define RT_UBYTEn07 RT_UBYTE
#define RT_UBYTEn06 RT_UBYTE
#define RT_UBYTEn05 RT_UBYTE
#define RT_UBYTEn04 RT_UBYTE
#define RT_UBYTEn03 RT_UBYTE
#define RT_UBYTEn02 RT_UBYTE
#define RT_UBYTEn01 RT_UBYTE
#define RT_UBYTE00 RT_UBYTE
#define RT_UBYTE01 RT_UBYTE
#define RT_UBYTE02 RT_UBYTE
#define RT_UBYTE03 RT_UBYTE
#define RT_UBYTE04 RT_UBYTE
#define RT_UBYTE05 RT_UBYTE
#define RT_UBYTE06 RT_UBYTE
#define RT_UBYTE07 RT_UBYTE
#define RT_UBYTE08 RT_UBYTE
#define RT_UBYTE09 RT_UBYTE
#define RT_UBYTE10 RT_UBYTE
#define RT_UBYTE11 RT_UBYTE
#define RT_UBYTE12 RT_UBYTE
#define RT_UBYTE13 RT_UBYTE
#define RT_UBYTE14 RT_UBYTE
#define RT_UBYTE15 RT_UBYTE
#define RT_UBYTE16 RT_UBYTE
#define RT_UBYTE17 RT_UBYTE
#define RT_UBYTE18 RT_UBYTE
#define RT_UBYTE19 RT_UBYTE
#define RT_UBYTE20 RT_UBYTE
#define RT_UBYTE21 RT_UBYTE
#define RT_UBYTE22 RT_UBYTE
#define RT_UBYTE23 RT_UBYTE
#define RT_UBYTE24 RT_UBYTE
#define RT_UBYTE25 RT_UBYTE
#define RT_UBYTE26 RT_UBYTE
#define RT_UBYTE27 RT_UBYTE
#define RT_UBYTE28 RT_UBYTE
#define RT_UBYTE29 RT_UBYTE
#define RT_UBYTE30 RT_UBYTE
#define RT_UBYTE31 RT_UBYTE
#define RT_UBYTE32 RT_UBYTE
#define RT_UBYTE33 RT_UBYTE
#define RT_UBYTE34 RT_UBYTE
#define RT_UBYTE35 RT_UBYTE
#define RT_UBYTE36 RT_UBYTE
#define RT_UBYTE37 RT_UBYTE
#define RT_UBYTE38 RT_UBYTE
#define RT_UBYTE39 RT_UBYTE
#define RT_UBYTE40 RT_UBYTE
#define RT_UBYTE41 RT_UBYTE
#define RT_UBYTE42 RT_UBYTE
#define RT_UBYTE43 RT_UBYTE
#define RT_UBYTE44 RT_UBYTE
#define RT_UBYTE45 RT_UBYTE
#define RT_UBYTE46 RT_UBYTE
#define RT_UBYTE47 RT_UBYTE
#define RT_UBYTE48 RT_UBYTE

#define RT_SBYTEn16 RT_SBYTE
#define RT_SBYTEn15 RT_SBYTE
#define RT_SBYTEn14 RT_SBYTE
#define RT_SBYTEn13 RT_SBYTE
#define RT_SBYTEn12 RT_SBYTE
#define RT_SBYTEn11 RT_SBYTE
#define RT_SBYTEn10 RT_SBYTE
#define RT_SBYTEn09 RT_SBYTE
#define RT_SBYTEn08 RT_SBYTE
#define RT_SBYTEn07 RT_SBYTE
#define RT_SBYTEn06 RT_SBYTE
#define RT_SBYTEn05 RT_SBYTE
#define RT_SBYTEn04 RT_SBYTE
#define RT_SBYTEn03 RT_SBYTE
#define RT_SBYTEn02 RT_SBYTE
#define RT_SBYTEn01 RT_SBYTE
#define RT_SBYTE00 RT_SBYTE
#define RT_SBYTE01 RT_SBYTE
#define RT_SBYTE02 RT_SBYTE
#define RT_SBYTE03 RT_SBYTE
#define RT_SBYTE04 RT_SBYTE
#define RT_SBYTE05 RT_SBYTE
#define RT_SBYTE06 RT_SBYTE
#define RT_SBYTE07 RT_SBYTE
#define RT_SBYTE08 RT_SBYTE
#define RT_SBYTE09 RT_SBYTE
#define RT_SBYTE10 RT_SBYTE
#define RT_SBYTE11 RT_SBYTE
#define RT_SBYTE12 RT_SBYTE
#define RT_SBYTE13 RT_SBYTE
#define RT_SBYTE14 RT_SBYTE
#define RT_SBYTE15 RT_SBYTE
#define RT_SBYTE16 RT_SBYTE
#define RT_SBYTE17 RT_SBYTE
#define RT_SBYTE18 RT_SBYTE
#define RT_SBYTE19 RT_SBYTE
#define RT_SBYTE20 RT_SBYTE
#define RT_SBYTE21 RT_SBYTE
#define RT_SBYTE22 RT_SBYTE
#define RT_SBYTE23 RT_SBYTE
#define RT_SBYTE24 RT_SBYTE
#define RT_SBYTE25 RT_SBYTE
#define RT_SBYTE26 RT_SBYTE
#define RT_SBYTE27 RT_SBYTE
#define RT_SBYTE28 RT_SBYTE
#define RT_SBYTE29 RT_SBYTE
#define RT_SBYTE30 RT_SBYTE
#define RT_SBYTE31 RT_SBYTE
#define RT_SBYTE32 RT_SBYTE
#define RT_SBYTE33 RT_SBYTE
#define RT_SBYTE34 RT_SBYTE
#define RT_SBYTE35 RT_SBYTE
#define RT_SBYTE36 RT_SBYTE
#define RT_SBYTE37 RT_SBYTE
#define RT_SBYTE38 RT_SBYTE
#define RT_SBYTE39 RT_SBYTE
#define RT_SBYTE40 RT_SBYTE
#define RT_SBYTE41 RT_SBYTE
#define RT_SBYTE42 RT_SBYTE
#define RT_SBYTE43 RT_SBYTE
#define RT_SBYTE44 RT_SBYTE
#define RT_SBYTE45 RT_SBYTE
#define RT_SBYTE46 RT_SBYTE
#define RT_SBYTE47 RT_SBYTE
#define RT_SBYTE48 RT_SBYTE

#define RT_USHORTn16 RT_USHORT
#define RT_USHORTn15 RT_USHORT
#define RT_USHORTn14 RT_USHORT
#define RT_USHORTn13 RT_USHORT
#define RT_USHORTn12 RT_USHORT
#define RT_USHORTn11 RT_USHORT
#define RT_USHORTn10 RT_USHORT
#define RT_USHORTn09 RT_USHORT
#define RT_USHORTn08 RT_USHORT
#define RT_USHORTn07 RT_USHORT
#define RT_USHORTn06 RT_USHORT
#define RT_USHORTn05 RT_USHORT
#define RT_USHORTn04 RT_USHORT
#define RT_USHORTn03 RT_USHORT
#define RT_USHORTn02 RT_USHORT
#define RT_USHORTn01 RT_USHORT
#define RT_USHORT00 RT_USHORT
#define RT_USHORT01 RT_USHORT
#define RT_USHORT02 RT_USHORT
#define RT_USHORT03 RT_USHORT
#define RT_USHORT04 RT_USHORT
#define RT_USHORT05 RT_USHORT
#define RT_USHORT06 RT_USHORT
#define RT_USHORT07 RT_USHORT
#define RT_USHORT08 RT_USHORT
#define RT_USHORT09 RT_USHORT
#define RT_USHORT10 RT_USHORT
#define RT_USHORT11 RT_USHORT
#define RT_USHORT12 RT_USHORT
#define RT_USHORT13 RT_USHORT
#define RT_USHORT14 RT_USHORT
#define RT_USHORT15 RT_USHORT
#define RT_USHORT16 RT_USHORT
#define RT_USHORT17 RT_USHORT
#define RT_USHORT18 RT_USHORT
#define RT_USHORT19 RT_USHORT
#define RT_USHORT20 RT_USHORT
#define RT_USHORT21 RT_USHORT
#define RT_USHORT22 RT_USHORT
#define RT_USHORT23 RT_USHORT
#define RT_USHORT24 RT_USHORT
#define RT_USHORT25 RT_USHORT
#define RT_USHORT26 RT_USHORT
#define RT_USHORT27 RT_USHORT
#define RT_USHORT28 RT_USHORT
#define RT_USHORT29 RT_USHORT
#define RT_USHORT30 RT_USHORT
#define RT_USHORT31 RT_USHORT
#define RT_USHORT32 RT_USHORT
#define RT_USHORT33 RT_USHORT
#define RT_USHORT34 RT_USHORT
#define RT_USHORT35 RT_USHORT
#define RT_USHORT36 RT_USHORT
#define RT_USHORT37 RT_USHORT
#define RT_USHORT38 RT_USHORT
#define RT_USHORT39 RT_USHORT
#define RT_USHORT40 RT_USHORT
#define RT_USHORT41 RT_USHORT
#define RT_USHORT42 RT_USHORT
#define RT_USHORT43 RT_USHORT
#define RT_USHORT44 RT_USHORT
#define RT_USHORT45 RT_USHORT
#define RT_USHORT46 RT_USHORT
#define RT_USHORT47 RT_USHORT
#define RT_USHORT48 RT_USHORT

#define RT_SSHORTn16 RT_SSHORT
#define RT_SSHORTn15 RT_SSHORT
#define RT_SSHORTn14 RT_SSHORT
#define RT_SSHORTn13 RT_SSHORT
#define RT_SSHORTn12 RT_SSHORT
#define RT_SSHORTn11 RT_SSHORT
#define RT_SSHORTn10 RT_SSHORT
#define RT_SSHORTn09 RT_SSHORT
#define RT_SSHORTn08 RT_SSHORT
#define RT_SSHORTn07 RT_SSHORT
#define RT_SSHORTn06 RT_SSHORT
#define RT_SSHORTn05 RT_SSHORT
#define RT_SSHORTn04 RT_SSHORT
#define RT_SSHORTn03 RT_SSHORT
#define RT_SSHORTn02 RT_SSHORT
#define RT_SSHORTn01 RT_SSHORT
#define RT_SSHORT00 RT_SSHORT
#define RT_SSHORT01 RT_SSHORT
#define RT_SSHORT02 RT_SSHORT
#define RT_SSHORT03 RT_SSHORT
#define RT_SSHORT04 RT_SSHORT
#define RT_SSHORT05 RT_SSHORT
#define RT_SSHORT06 RT_SSHORT
#define RT_SSHORT07 RT_SSHORT
#define RT_SSHORT08 RT_SSHORT
#define RT_SSHORT09 RT_SSHORT
#define RT_SSHORT10 RT_SSHORT
#define RT_SSHORT11 RT_SSHORT
#define RT_SSHORT12 RT_SSHORT
#define RT_SSHORT13 RT_SSHORT
#define RT_SSHORT14 RT_SSHORT
#define RT_SSHORT15 RT_SSHORT
#define RT_SSHORT16 RT_SSHORT
#define RT_SSHORT17 RT_SSHORT
#define RT_SSHORT18 RT_SSHORT
#define RT_SSHORT19 RT_SSHORT
#define RT_SSHORT20 RT_SSHORT
#define RT_SSHORT21 RT_SSHORT
#define RT_SSHORT22 RT_SSHORT
#define RT_SSHORT23 RT_SSHORT
#define RT_SSHORT24 RT_SSHORT
#define RT_SSHORT25 RT_SSHORT
#define RT_SSHORT26 RT_SSHORT
#define RT_SSHORT27 RT_SSHORT
#define RT_SSHORT28 RT_SSHORT
#define RT_SSHORT29 RT_SSHORT
#define RT_SSHORT30 RT_SSHORT
#define RT_SSHORT31 RT_SSHORT
#define RT_SSHORT32 RT_SSHORT
#define RT_SSHORT33 RT_SSHORT
#define RT_SSHORT34 RT_SSHORT
#define RT_SSHORT35 RT_SSHORT
#define RT_SSHORT36 RT_SSHORT
#define RT_SSHORT37 RT_SSHORT
#define RT_SSHORT38 RT_SSHORT
#define RT_SSHORT39 RT_SSHORT
#define RT_SSHORT40 RT_SSHORT
#define RT_SSHORT41 RT_SSHORT
#define RT_SSHORT42 RT_SSHORT
#define RT_SSHORT43 RT_SSHORT
#define RT_SSHORT44 RT_SSHORT
#define RT_SSHORT45 RT_SSHORT
#define RT_SSHORT46 RT_SSHORT
#define RT_SSHORT47 RT_SSHORT
#define RT_SSHORT48 RT_SSHORT

#define RT_ULONGn16 RT_ULONG
#define RT_ULONGn15 RT_ULONG
#define RT_ULONGn14 RT_ULONG
#define RT_ULONGn13 RT_ULONG
#define RT_ULONGn12 RT_ULONG
#define RT_ULONGn11 RT_ULONG
#define RT_ULONGn10 RT_ULONG
#define RT_ULONGn09 RT_ULONG
#define RT_ULONGn08 RT_ULONG
#define RT_ULONGn07 RT_ULONG
#define RT_ULONGn06 RT_ULONG
#define RT_ULONGn05 RT_ULONG
#define RT_ULONGn04 RT_ULONG
#define RT_ULONGn03 RT_ULONG
#define RT_ULONGn02 RT_ULONG
#define RT_ULONGn01 RT_ULONG
#define RT_ULONG00 RT_ULONG
#define RT_ULONG01 RT_ULONG
#define RT_ULONG02 RT_ULONG
#define RT_ULONG03 RT_ULONG
#define RT_ULONG04 RT_ULONG
#define RT_ULONG05 RT_ULONG
#define RT_ULONG06 RT_ULONG
#define RT_ULONG07 RT_ULONG
#define RT_ULONG08 RT_ULONG
#define RT_ULONG09 RT_ULONG
#define RT_ULONG10 RT_ULONG
#define RT_ULONG11 RT_ULONG
#define RT_ULONG12 RT_ULONG
#define RT_ULONG13 RT_ULONG
#define RT_ULONG14 RT_ULONG
#define RT_ULONG15 RT_ULONG
#define RT_ULONG16 RT_ULONG
#define RT_ULONG17 RT_ULONG
#define RT_ULONG18 RT_ULONG
#define RT_ULONG19 RT_ULONG
#define RT_ULONG20 RT_ULONG
#define RT_ULONG21 RT_ULONG
#define RT_ULONG22 RT_ULONG
#define RT_ULONG23 RT_ULONG
#define RT_ULONG24 RT_ULONG
#define RT_ULONG25 RT_ULONG
#define RT_ULONG26 RT_ULONG
#define RT_ULONG27 RT_ULONG
#define RT_ULONG28 RT_ULONG
#define RT_ULONG29 RT_ULONG
#define RT_ULONG30 RT_ULONG
#define RT_ULONG31 RT_ULONG
#define RT_ULONG32 RT_ULONG
#define RT_ULONG33 RT_ULONG
#define RT_ULONG34 RT_ULONG
#define RT_ULONG35 RT_ULONG
#define RT_ULONG36 RT_ULONG
#define RT_ULONG37 RT_ULONG
#define RT_ULONG38 RT_ULONG
#define RT_ULONG39 RT_ULONG
#define RT_ULONG40 RT_ULONG
#define RT_ULONG41 RT_ULONG
#define RT_ULONG42 RT_ULONG
#define RT_ULONG43 RT_ULONG
#define RT_ULONG44 RT_ULONG
#define RT_ULONG45 RT_ULONG
#define RT_ULONG46 RT_ULONG
#define RT_ULONG47 RT_ULONG
#define RT_ULONG48 RT_ULONG

#define RT_SLONGn16 RT_SLONG
#define RT_SLONGn15 RT_SLONG
#define RT_SLONGn14 RT_SLONG
#define RT_SLONGn13 RT_SLONG
#define RT_SLONGn12 RT_SLONG
#define RT_SLONGn11 RT_SLONG
#define RT_SLONGn10 RT_SLONG
#define RT_SLONGn09 RT_SLONG
#define RT_SLONGn08 RT_SLONG
#define RT_SLONGn07 RT_SLONG
#define RT_SLONGn06 RT_SLONG
#define RT_SLONGn05 RT_SLONG
#define RT_SLONGn04 RT_SLONG
#define RT_SLONGn03 RT_SLONG
#define RT_SLONGn02 RT_SLONG
#define RT_SLONGn01 RT_SLONG
#define RT_SLONG00 RT_SLONG
#define RT_SLONG01 RT_SLONG
#define RT_SLONG02 RT_SLONG
#define RT_SLONG03 RT_SLONG
#define RT_SLONG04 RT_SLONG
#define RT_SLONG05 RT_SLONG
#define RT_SLONG06 RT_SLONG
#define RT_SLONG07 RT_SLONG
#define RT_SLONG08 RT_SLONG
#define RT_SLONG09 RT_SLONG
#define RT_SLONG10 RT_SLONG
#define RT_SLONG11 RT_SLONG
#define RT_SLONG12 RT_SLONG
#define RT_SLONG13 RT_SLONG
#define RT_SLONG14 RT_SLONG
#define RT_SLONG15 RT_SLONG
#define RT_SLONG16 RT_SLONG
#define RT_SLONG17 RT_SLONG
#define RT_SLONG18 RT_SLONG
#define RT_SLONG19 RT_SLONG
#define RT_SLONG20 RT_SLONG
#define RT_SLONG21 RT_SLONG
#define RT_SLONG22 RT_SLONG
#define RT_SLONG23 RT_SLONG
#define RT_SLONG24 RT_SLONG
#define RT_SLONG25 RT_SLONG
#define RT_SLONG26 RT_SLONG
#define RT_SLONG27 RT_SLONG
#define RT_SLONG28 RT_SLONG
#define RT_SLONG29 RT_SLONG
#define RT_SLONG30 RT_SLONG
#define RT_SLONG31 RT_SLONG
#define RT_SLONG32 RT_SLONG
#define RT_SLONG33 RT_SLONG
#define RT_SLONG34 RT_SLONG
#define RT_SLONG35 RT_SLONG
#define RT_SLONG36 RT_SLONG
#define RT_SLONG37 RT_SLONG
#define RT_SLONG38 RT_SLONG
#define RT_SLONG39 RT_SLONG
#define RT_SLONG40 RT_SLONG
#define RT_SLONG41 RT_SLONG
#define RT_SLONG42 RT_SLONG
#define RT_SLONG43 RT_SLONG
#define RT_SLONG44 RT_SLONG
#define RT_SLONG45 RT_SLONG
#define RT_SLONG46 RT_SLONG
#define RT_SLONG47 RT_SLONG
#define RT_SLONG48 RT_SLONG

/* this type is used in 32 bit fixed point multiplication */
	
typedef struct _DOUBLESHORT {  
   unsigned int sign:1;
   RT_USHORT high;
   RT_USHORT low;
} RT_DOUBLESHORT;

/* this type is used in  narrowing division (16/8) division */
typedef struct _SHORTSTRUCT {  
   unsigned int sign:1;
   RT_USHORT val;
} RT_SHORTSTRUCT;

/* this type is used in  narrowing division (32/16) division */
typedef struct _LONGSTRUCT {  
   unsigned int sign:1;
   RT_ULONG val;
} RT_LONGSTRUCT;


#define _SA_TYPES
#endif
