/*----------------------------------------------------------------------------
--                       AutoCode (TM) 62mx1118
--
--                         Copyright (C) 1999
--                       INTEGRATED SYSTEMS, INC.
--
--  Unpublished work - rights reserved under the U.S. Copyright Act.  Use,
--  duplication, or disclosure by the government is subject to restrictions
--  set forth in the ISI commercial license agreement.
**----------------------------------------------------------------------------
**
**  File       : sa_utils.h
**  Project    : Autocode/C
**
**  Abstract:
**
**      Interface to AutoCode/C stand-alone support utilities.
*/

#ifndef  _SA_UTILS
#define  _SA_UTILS

#include "sa_sys.h"
#include "sa_types.h"

#if defined(AC1000) || defined(_NT) || defined(RS_VXWORKS) || defined(RS_UNIX)
#include "model.h"
#endif

/*================================================================
              O T H E R   C O N V E R S I O N S              
 ================================================================*/
#define I_Ipr(a)     (a)
#define F_Fpr(a)     (a)
#define I_Fpr(a)     ((RT_INTEGER)ROUND(a))
#define F_Ipr(a)     ((RT_FLOAT)a)

/*================================================================
              G E T  A N D  S E T  M A C R O S              
 ================================================================*/

#define GET_EXTIN(type, b)   \
		  JUXTAPOSE(type,_Fpr)(b)  
		  
#define SET_EXTOUT(type, b)   \
		  JUXTAPOSE(F_,type)(b)


#if (AC100DOUBLE)
#  define SA_Implementation_Initialize SA_Implementation_Initialize_dbl
#endif

  /*** Prefixed synonyms for procedure names ***/

#if (AC100_TARGET)

  /**********************
  * Function Prototypes *
  **********************/

#if defined(AC1000) || defined(_NT) || defined(RS_VXWORKS) || defined(RS_UNIX)
 /* Function: SA_External_Input ++++++++++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Called by the application scheduler on every minor tick to request
  **   that external inputs be updated in the external input block.
  **   Returns (0) if no error.
  */
#if defined(AC1000)
  EXTERN_FUNC( RT_INTEGER  SA_External_Input, ( void ));
#else
  EXTERN_FUNC( RT_INTEGER  SA_External_Input, (modelInstData *));
#endif



  /* Function: SA_External_Output +++++++++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Called by the application scheduler on every minor tick to request
  **   that new external outputs in the external output block be sent to
  **   the I/O devices.  Returns (0) if no error.
  ** Note:
  **   The modelInstData pointer is zero when the scheduler has finished updating
  **   all model instances for a particular cycle.
  */
#if defined(AC1000)
  EXTERN_FUNC( RT_INTEGER  SA_External_Output, ( void ));
#else
  EXTERN_FUNC( RT_INTEGER  SA_External_Output, (modelInstData *));
#endif
 
 
  /* Function: SA_Instance_Initialize +++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Performs once only system level intialization.
  **   Returns (0) if no error.
  */
#if defined(AC1000)
  EXTERN_FUNC( RT_INTEGER  SA_Instance_Initialize, ( void ));
#else
  EXTERN_FUNC( RT_INTEGER  SA_Instance_Initialize, ( modelInstData *));
#endif
 
 
  /* Function: SA_Instance_Terminate ++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   This routine will be called only once when the system wants to
  **   terminate.  An errcode of zero specifies a normal exit, non-zero
  **   specifies an abort exit.
  */
#if defined(AC1000)
  EXTERN_FUNC( RT_INTEGER  SA_Instance_Terminate, ( void ));
#else
  EXTERN_FUNC( RT_INTEGER  SA_Instance_Terminate, (modelInstData *));
#endif
 
  /* Function: SA_Error +++++++++++++++++++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Called by the application scheduler to invoke the message reporting
  **   services and signal that the application needs to stop.
  */
  EXTERN_FUNC( void  SA_Error,
       ( 
        RT_INTEGER   n_task,         /* Task_id of the reporting task  */
        RT_INTEGER   error_num,      /* Error code                     */
        RT_INTEGER   pnum            /* Processor_id of reporting task */
       ));

#else

  /* Function: SA_External_Input ++++++++++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Called by the application scheduler on every minor tick to request
  **   that external inputs be updated in the external input block.
  **   Returns (0) if no error.
  */
  EXTERN_FUNC( RT_INTEGER  SA_External_Input, (void));



  /* Function: SA_External_Output +++++++++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Called by the application scheduler on every minor tick to request
  **   that new external outputs in the external output block be sent to
  **   the I/O devices.  Returns (0) if no error. 
  */
  EXTERN_FUNC( RT_INTEGER  SA_External_Output, (void));



  /* Function: SA_Implementation_Initialize +++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Performs once only system level intialization.
  **   Returns (0) if no error.
  */
  EXTERN_FUNC( RT_INTEGER  SA_Implementation_Initialize, 
       (
        RT_FLOAT     *u_ptr,         /* pointer to external input block  */
        RT_INTEGER   nu,             /* number of external inputs        */
        RT_FLOAT     *y_ptr,         /* pointer to external output block */
        RT_INTEGER   ny,             /* number of external outputs       */
        RT_FLOAT     scheduler_freq  /* application update frequency     */
       ));



  /* Function: SA_Implementation_Start ++++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Performs any final actions required to run the application then
  **   returns when a start request is received.  Returns (0) if no error.
  */
  EXTERN_FUNC( RT_INTEGER  SA_Implementation_Start, (void));



  /* Function: SA_Implementation_Stop +++++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   This routine is called at the end of a simulation run to wait for
  **   further requests.  It returns the current state when a RUN_STATE or
  **   EXIT_STATE has been requested.  Possible values for current state are:
  **
  **   INIT_STATE  =  0
  **   IDLE_STATE  =  1
  **   RUN_STATE   =  2
  **   EXIT_STATE  =  3
  */
  EXTERN_FUNC( RT_INTEGER  SA_Implementation_Stop, (void));



  /* Function: SA_Implementation_Terminate ++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   This routine will be called only once when the system wants to
  **   terminate.  An errcode of zero specifies a normal exit, non-zero
  **   specifies an abort exit. 
  */
  EXTERN_FUNC( void  SA_Implementation_Terminate,
       (
        RT_INTEGER   errcode         /* specifies the type of exit */
       ));



  /* Function: SA_Background ++++++++++++++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Starts the application and waits for it to complete.
  **   Returns (0) if no error.
  */
  EXTERN_FUNC( RT_INTEGER  SA_Background, (void));


#if defined(RS_PSOS)
  /* Function: SA_Error +++++++++++++++++++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Called by the application scheduler to invoke the message reporting
  **   services and signal that the application needs to stop.
  */
  EXTERN_FUNC( void  SA_Error, 
       (
        RT_INTEGER   n_task,         /* Task_id of the reporting task  */
        RT_INTEGER   error_num,      /* Error code                     */
	RT_INTEGER   perror_code,    /* pSOS error code                */
        RT_INTEGER   pnum            /* Processor_id of reporting task */
       ));

#else
  /* Function: SA_Error +++++++++++++++++++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Called by the application scheduler to invoke the message reporting
  **   services and signal that the application needs to stop.
  */
  EXTERN_FUNC( void  SA_Error, 
       (
        RT_INTEGER   n_task,         /* Task_id of the reporting task  */
        RT_INTEGER   error_num,      /* Error code                     */
        RT_INTEGER   pnum            /* Processor_id of reporting task */
       ));
#endif


  /* Function: SA_Signal_Remote_Dispatch ++++++++++++++++++++++++++++++++++++++
  **
  ** Purpose:
  **   Called by the application scheduler (only in multiprocessor
  **   configurations) to invoke an application update cycle on a
  **   remote processor.
  */
  EXTERN_FUNC( void  SA_Signal_Remote_Dispatch, (void));

#endif

  /********************
  * Macro definitions *
  ********************/

  /**************************************************************************
  ** SA_Disable
  **
  ** Purpose:
  **   Called by the application scheduler to disable (mask) scheduler
  **   interrupts that protect critical code sections.  (This is only
  **   required in certain procedural forms of the generated code.)
  **************************************************************************/

  /**************************************************************************
  ** SA_Enable
  **
  ** Purpose:
  **   Called by the application scheduler to enable (unmask) scheduler
  **   interrupts that protect critical code sections.  (This is only
  **   required in certain procedural forms of the generated code.)
  **************************************************************************/


# if (RS_UNIX || RS_VXWORKS)
#   define SA_Disable              TSKLock();
#   define SA_Enable               TSKUnlock();
# endif 

# if (RS_PSOS || C_WINNT_TGT)
#   define SA_Disable
#   define SA_Enable
# endif 

# if (METAWARE || C_PC) && !defined MSVC32
#   define CLI_Opcode              0xfa
#   define STI_Opcode              0xfb
#   define SA_Disable              _inline(CLI_Opcode)
#   define SA_Enable               _inline(STI_Opcode)
# endif

# if (MSVC32) && !defined C_WINNT_TGT
#   define SA_Disable              _asm cli
#   define SA_Enable               _asm sti
# endif

# if (PGI860)
    extern int Disable_Interrupts();
    extern int Enable_Interrupts();
#   define SA_Disable              Disable_Interrupts()
#   define SA_Enable               Enable_Interrupts()
# endif

# if (BORLANDC)
#   define SA_Disable              DisableTasking()
#   define SA_Enable               EnableTasking()
# endif

# if (_TMS320C30 || _TMS320C40)
    extern void DisableTasking (void);
    extern void EnableTasking (void);
#   define SA_Disable              DisableTasking ()
#   define SA_Enable               EnableTasking ()
# endif


# define Enable  SA_Enable
# define Disable SA_Disable

#else

EXTERN_FUNC(RT_INTEGER   External_Input,  ( ) );
EXTERN_FUNC(RT_INTEGER   External_Output, ( ) );

EXTERN_FUNC(RT_INTEGER   Background, ( ) );

EXTERN_FUNC(void   Implementation_Initialize, (RT_FLOAT BUS_IN[],  
                                                 RT_INTEGER NI, 
                                                 RT_FLOAT BUS_OUT[], 
                                                 RT_INTEGER NO,
                                                 RT_FLOAT SCHEDULER_FREQ) );

EXTERN_FUNC(void    Signal_Remote_Dispatch, ( ) );
EXTERN_FUNC(void    Implementation_Terminate, ( ) );
EXTERN_FUNC(void    Error, (RT_INTEGER  NTSK, RT_INTEGER  errorCode) );
EXTERN_FUNC(void    SCHEDULER, ( ) );

# define Disable
# define Enable

#endif


#endif /*_SA_UTILS*/

