/*----------------------------------------------------------------------------
--                       AutoCode (TM) 62mx1118
--
--                         Copyright (C) 1999
--                       INTEGRATED SYSTEMS, INC.
--
--  Unpublished work - rights reserved under the U.S. Copyright Act.  Use,
--  duplication, or disclosure by the government is subject to restrictions
--  set forth in the ISI commercial license agreement.
**----------------------------------------------------------------------------
**
**  File       : sa_fuzzy.h
**  Project    : Autocode/C
**
---                                                                  ---
--- Desription:                                                      ---
---     These subroutines compose the specification file for         ---
---     the user-defined and pre-defined Fuzzy Logic Routines        ---
---     for C Code. The user may want to look at SA_FUZZY for        ---
---     some examples.                                               ---
---                                                                  ---
---                                                                  ---
---            +---------------------------------------+             ---
---            | User-Defined Function Specifications  |             ---
---            +---------------------------------------+             ---
---                                                                  ---
---                                                                  ---
--- DECISION-MAKING LOGIC                                            ---
---                                                                  ---
---    DATA1 (and DATA2) represent the number(s) passed to           ---
---    the AND, OR, and NOT operators.                               ---
---                                                                  ---
---    Conjunction                                                   ---
---       RT_FLOAT fuzand( RT_FLOAT DATA1,                           ---
---                        RT_FLOAT DATA2 );                         ---
---                                                                  ---
---    Disjunction                                                   ---
---       RT_FLOAT fuzzor( RT_FLOAT DATA1,                           ---
---                        RT_FLOAT DATA2 );                         ---
---                                                                  ---
---                                                                  ---
--- DEFUZZIFICATION                                                  ---
---                                                                  ---
---    A curve VALUE, starting at OFFSET, and containing NPOINT      ---
---    evenly spaced discrete points from 0 to 1 inclusive,          ---
---    is to be defuzzified into a single point which is valued      ---
---    between zero and one.  Of these four, only defuzz is user     ---
---    definable.  Do NOT change cntrod, mnomax, fstmax.             ---
---                                                                  ---
---    RT_FLOAT cntrod ( RT_INTEGER NPOINT,                          ---
---                      RT_FLOAT   VALUE[],                         ---
---                      RT_INTEGER OFFSET );                        ---
---    RT_FLOAT mnomax ( RT_INTEGER NPOINT,                          ---
---                      RT_FLOAT   VALUE[],                         ---
---                      RT_INTEGER OFFSET );                        ---
---    RT_FLOAT fstmax ( RT_INTEGER NPOINT,                          ---
---                      RT_FLOAT   VALUE[],                         ---
---                      RT_INTEGER OFFSET );                        ---
---    RT_FLOAT defuzz( RT_INTEGER NPOINT,                          ---
---                      RT_FLOAT   VALUE[],                         ---
---                      RT_INTEGER OFFSET );                        ---
---                                                                  ---
---                                                                  ---
---                                                                  ---
--- IMPLICATION                                                      ---
---                                                                  ---
---    A curve VALUE, starting at OFFSET, and containing NPOINT      ---
---    evenly spaced discrete points from 0 to 1 inclusive,          ---
---    is to be implicated by the belief in the condition CUTOFF,    ---
---    with the results to be stored in VALUE starting at OFFSET.    ---
---                                                                  ---
---                                                                  ---
---    void fuzimp ( RT_INTEGER NPOINT,                              ---
---                  RT_FLOAT   VALUE[],                             ---
---                  RT_INTEGER OFFSET,                              ---
---                  RT_FLOAT   CUTOFF );                            ---
---                                                                  ---
---                                                                  ---
---                                                                  ---
--- AGGREGATION                                                      ---
---                                                                  ---
---    In the Initialization MODE = 0, this routine is invoked with  ---
---       both TCURVE and TDENOM set to zero already.  The user may  ---
---       want to set up some other internal variables.  Note that   ---
---       both WEIGHT and CUTOFF have no guaranteed meaning here.    ---
---    In the Aggregation MODE = 1, this routine is invoked with     ---
---       TCURVE and TDENOM representing static storage provided     ---
---       for the user.  Typically information is stored in some     ---
---       form of numerator (TCURVE) and denominator (TDENOM).       ---
---       CUTOFF desribes the value to be aggregated while WEIGHT    ---
---       describes the weighting from 0 to 1 inclusive.             ---
---    In the Done MODE = 2, this routine is invoked with the user   ---
---       calculating the result and copying it to CUTOFF.           ---
---                                                                  ---
---    void fuzagr ( RT_INTEGER MODE,                                ---
---                  RT_FLOAT  *TCURVE,                              ---
---                  RT_FLOAT  *TDENOM,                              ---
---                  RT_FLOAT  *CUTOFF,                              ---
---                  RT_FLOAT   WEIGHT );                            ---
---                                                                  ---
------------------------------------------------------------------------
*/
#ifndef _SA_FUZZY
#define _SA_FUZZY

#include "sa_defn.h"


/*The following extern declarations make sure that there is no name conflict 
  with XMATH libraries when linking UCBs containing fuzzy blocks back with 
  XMATH
*/

EXTERN_FUNC(RT_FLOAT fuzand, (RT_FLOAT input1, RT_FLOAT input2) );
EXTERN_FUNC(RT_FLOAT fuzzor, (RT_FLOAT input1, RT_FLOAT input2) );
EXTERN_FUNC(RT_FLOAT defuzz, (RT_INTEGER npoint, RT_FLOAT value[], 
                                 RT_INTEGER indx) );
EXTERN_FUNC(void     fuzagr, (RT_INTEGER mode, RT_FLOAT *store1, 
                                 RT_FLOAT *store2, RT_FLOAT *value, 
                                 RT_FLOAT weight ) );
EXTERN_FUNC(void     fuzimp, (RT_INTEGER npoint, RT_FLOAT value[], 
                                 RT_INTEGER indx, RT_FLOAT cutoff) );

/* Pre-Defined Functions */
EXTERN_FUNC(RT_FLOAT cntrod, (RT_INTEGER npoint, RT_FLOAT value_array[],
                                 RT_INTEGER indx) );
EXTERN_FUNC(RT_FLOAT mnomax, (RT_INTEGER npoint, RT_FLOAT value_array[],
                                 RT_INTEGER indx) );

#endif
